import { fromEvent, interval, Subject } from 'rxjs';
import { debounce, takeUntil } from 'rxjs/operators';

import {
  Component,
  OnDestroy,
  OnInit,
  AfterViewInit,
  Renderer2,
  Inject
} from '@angular/core';
import { DOCUMENT } from '@angular/common';
import { Router } from '@angular/router';

import { Store } from '@ngrx/store';

import { NativeService } from '@ucap/native';

import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';

import { AppActions } from '@app/store/actions';

import { AppAuthenticationService } from './services/app-authentication.service';
import { AnimationBuilder, style, animate } from '@angular/animations';

@Component({
  selector: 'app-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.scss']
})
export class AppComponent implements OnInit, OnDestroy {
  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private router: Router,
    private renderer2: Renderer2,
    private store: Store<any>,
    private appAuthenticationService: AppAuthenticationService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    @Inject(DOCUMENT) private _document: any,
    private _animationBuilder: AnimationBuilder
  ) {
    fromEvent(window, 'resize')
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        debounce(() => interval(100))
      )
      .subscribe((event: any) => {
        this.dispatchWindowSize({
          width: event.target.innerWidth,
          height: event.target.innerHeight
        });
      });

    fromEvent(window, 'unload')
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((event: any) => {
        this.nativeService.app_postDestroy();
      });

    // fromEvent(window, 'beforeunload')
    //   .pipe(takeUntil(this.ngOnDestroySubject))
    //   .subscribe((event: any) => {
    //     this.appAuthenticationService.logout();
    //   });
  }

  ngOnInit(): void {
    this.dispatchWindowSize({
      width: window.innerWidth,
      height: window.innerHeight
    });
    this.nativeService.app_postInit().then((info) => {
      if (!info) {
        return;
      }
      if (!!info.initUrl) {
        this.router.navigateByUrl(info.initUrl);
      }
    });

    const splash = this._document.body.querySelector('#ucap-lg-web-splash');
    const player = this._animationBuilder
      .build([
        style({ opacity: '1' }),
        animate(
          '400ms ease',
          style({
            opacity: '0',
            zIndex: '-10'
          })
        )
      ])
      .create(splash);

    setTimeout(() => {
      player.play();
    }, 0);

    // const preloader = this.renderer2.selectRootElement(
    //   '#ucap-lg-web-preloader'
    // );
    // const fadeEffect = setInterval(() => {
    //   if (!preloader.style.opacity) {
    //     this.renderer2.setStyle(preloader, 'opacity', 1);
    //   }
    //   if (preloader.style.opacity > 0) {
    //     this.renderer2.setStyle(
    //       preloader,
    //       'opacity',
    //       preloader.style.opacity - 0.1
    //     );
    //   } else {
    //     this.renderer2.setStyle(preloader, 'display', 'none');
    //     clearInterval(fadeEffect);
    //   }
    // }, 200);

    // this.renderer2.setStyle(preloader, 'display', 'none');
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  private dispatchWindowSize(size: { width: number; height: number }) {
    this.store.dispatch(AppActions.windowResized(size));
  }
}
