import moment from 'moment';
import 'moment-timezone';

import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject,
  ViewChild
} from '@angular/core';
import { FormControl } from '@angular/forms';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';
import { MatOptionSelectionChange } from '@angular/material/core';
import { MatSelectChange, MatSelect } from '@angular/material/select';
import { MatCheckboxChange } from '@angular/material/checkbox';
import { MatRadioChange } from '@angular/material/radio';

import { ObjectUtil } from '@ucap/core';
import { NotificationMethod } from '@ucap/domain-common';
import { NativeService, NativeType } from '@ucap/native';

import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { I18nService } from '@ucap/ng-i18n';
import { DateService } from '@ucap/ng-ui/date';
import { VirtualScrollViewportComponent } from '@ucap/ng-ui/scrolling';
import { TranslateService } from '@ucap/ng-ui-organization';

import { environment } from '@environments';

import {
  Settings,
  GeneralSetting,
  NotificationSetting,
  ChatSetting,
  PresenceSetting
} from '@app/models/settings';

export interface TimezoneData {
  displayName: string;
  name: string;
}

export interface SettingsDialogData {
  settings: Settings;
}
export interface SettingsDialogResult {
  settings: Settings;
}

@Component({
  selector: 'app-sections-account-settings',
  templateUrl: './settings.dialog.component.html',
  styleUrls: ['./settings.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SettingsDialogComponent implements OnInit, OnDestroy {
  @ViewChild('vsTimezone', { static: false })
  vsTimezone: VirtualScrollViewportComponent;

  @ViewChild('selectForTimezone', { static: false })
  selectForTimezone: MatSelect;

  formControlForTimezone = new FormControl();

  platform: 'browser' | 'electron' = 'electron';

  settings: Settings;

  timezoneList: TimezoneData[];
  timezonePlaceholder: string;

  supportedLanguages = environment.productConfig.supportedLanguages;
  supportedHrLanguages =
    environment.productConfig.organization.supportedLanguages;

  String = String;
  NotificationMethod = NotificationMethod;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    public dialogRef: MatDialogRef<SettingsDialogData, SettingsDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: SettingsDialogData,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private dateService: DateService,
    private i18nService: I18nService,
    private translateService: TranslateService,
    private changeDetectorRef: ChangeDetectorRef,
    public matDialog: MatDialog
  ) {
    this.nativeService.platform_nativeType().then((type) => {
      // this.platform = 'electron';
      // return;
      switch (type) {
        case NativeType.Browser:
          this.platform = 'browser';
          break;
        case NativeType.Electron:
          this.platform = 'electron';
          break;
        default:
          break;
      }
    });

    this.settings = ObjectUtil.deepClone(data.settings);
  }

  ngOnInit(): void {
    this.generateTimezoneData();

    this.i18nService.languageChanged$
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((lang) => {
        this.generateTimezoneData();
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onChangeForAutoStartOnBoot(event: MatCheckboxChange) {
    this.settings.general.autoLaunch = event.checked;
  }

  onChangeForAutoLogin(event: MatCheckboxChange) {
    this.settings.general.autoLogin = event.checked;
  }

  onChangeForAutoHide(event: MatCheckboxChange) {
    this.settings.general.startupHideWindow = event.checked;
  }

  onSelectionChangeLanguage(event: MatSelectChange) {
    this.i18nService.changeLanguage(event.value);
    this.settings.general.locale = event.value;
  }

  onSelectionChangeHrLanguage(event: MatSelectChange) {
    this.translateService.use(event.value);
    this.settings.general.hrInfoLocale = event.value;
  }

  onOpenedChangeTimezone(opened: boolean) {
    if (opened) {
      this.setTimezoneData();
      this.vsTimezone.checkViewportSize();
    }
  }

  onSelectionChangeTimezone(event: MatOptionSelectionChange) {
    if (!event.isUserInput) {
      return;
    }
    this.dateService.use(event.source.value);
    this.settings.general.timezone = event.source.value;
  }

  onChangeForReceiveNotification(event: MatRadioChange) {
    const use = 'true' === event.value;
    console.log('onChangeForReceiveNotification', use);
  }

  onSelectionChangeMethodOfNotification(event: MatSelectChange) {
    console.log('onSelectionChangeMethodOfNotification', event.value);
  }

  onSelectionChangeAlertExposureTimeOfNotification(event: MatSelectChange) {
    const v = Number(event.value);
    console.log('onSelectionChangeAlertExposureTimeOfNotification', v);
  }

  onChangeForReceiveForMessageOfNotification(event: MatCheckboxChange) {
    console.log('onChangeForReceiveForMessageOfNotification', event.checked);
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close({ settings: this.data.settings });
  }

  onCancel() {
    this.dialogRef.close({ settings: this.data.settings });
  }

  onConfirm() {
    this.dialogRef.close({
      settings: this.settings
    });
  }

  private generateTimezoneData() {
    const timezoneData = this.i18nService.t('locale:timezone', {
      returnObjects: true
    });

    let timezoneList: TimezoneData[] = [];
    for (const name of moment.tz.names()) {
      const displayName = `(UTC${moment.tz(name).format('Z')}) ${
        timezoneData[name]
      }`;
      if (-1 < displayName.indexOf('undefined')) {
        console.log('timezone', name);
      }
      timezoneList.push({
        displayName,
        name
      });
      if (name === this.settings.general.timezone) {
        this.timezonePlaceholder = displayName;
      }
    }
    timezoneList = timezoneList.sort((a: TimezoneData, b: TimezoneData) => {
      return a.displayName.localeCompare(b.displayName);
    });

    this.timezoneList = timezoneList;
  }

  private setTimezoneData() {
    const timezoneIndex = this.timezoneList.findIndex(
      (t) => t.name === this.settings.general.timezone
    );

    if (-1 !== timezoneIndex) {
      if (!!this.vsTimezone && !!this.selectForTimezone) {
        this.vsTimezone.scrollToIndex(timezoneIndex, 'start');
        this.selectForTimezone.value = this.timezoneList[timezoneIndex].name;
      }
    }
  }
}
