import { Subject, combineLatest } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { User } from '@ucap/domain-organization';
import { LoginSession, LoginInfo } from '@ucap/domain-authentication';
import { SearchType, SearchDirectionByDateType } from '@ucap/domain-call';
import { CallHistoryInfoRequest } from '@ucap/api-contact';

import { LogService } from '@ucap/ng-logger';
import { CallActions } from '@ucap/ng-store-call';

import { UserSelector } from '@ucap/ng-store-organization';
import { LoginSelector } from '@ucap/ng-store-authentication';

import { SearchData } from '@app/ucap/organization/models/search-data';
import { AppAuthenticationService } from '@app/services/app-authentication.service';

import moment from 'moment';
import { environment } from '@environments';

@Component({
  selector: 'app-pages-call-sidenav',
  templateUrl: './sidenav.page.component.html',
  styleUrls: ['./sidenav.page.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SidenavPageComponent implements OnInit, OnDestroy {
  set companySearchData(searchData: SearchData) {
    if (!!searchData && searchData.searchWord !== '') {
      this._companySearchData = { ...searchData, bySearch: true };
    } else {
      this._companySearchData = { ...searchData, bySearch: false };
    }
  }
  get companySearchData() {
    return this._companySearchData;
  }
  // tslint:disable-next-line: variable-name
  _companySearchData: SearchData;
  isBackspaceCanceled = true;

  user: User;
  loginInfo: LoginInfo;
  loginSession: LoginSession;

  private ngOnDestroySubject: Subject<void> = new Subject();
  constructor(
    private store: Store<any>,
    private appAuthenticationService: AppAuthenticationService,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {}

  ngOnInit(): void {
    combineLatest([
      this.store.pipe(select(UserSelector.user)),
      this.store.pipe(select(LoginSelector.loginInfo))
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([user, loginInfo]) => {
        if (!!user && !!loginInfo) {
          this.user = user;
          this.loginInfo = loginInfo;
          this.loginSession = this.appAuthenticationService.getLoginSession();

          this._retrieveCallHistory();
        }
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onSearchCancel() {
    this.companySearchData = { ...this.companySearchData, searchWord: '' };
  }

  private _retrieveCallHistory() {
    const searchStartDate = moment(new Date()).format('YYYY-MM-DD_HH:mm:ss');
    const req: CallHistoryInfoRequest = {
      userSeq: String(this.user.info.seq),
      deviceType: this.loginSession.deviceType,
      token: this.loginInfo.tokenString,
      searchType: SearchType.All,
      searchUserSeq: '',
      searchNumber: '',
      searchDate: '',
      searchStartDate,
      searchListCount:
        environment.productConfig.call.historyRequestDefaultCount,
      searchDirection: SearchDirectionByDateType.Before
    };
    this.store.dispatch(CallActions.callHistory({ req }));
  }
}
