import { Subject } from 'rxjs';
import { takeUntil, take, map } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectorRef,
  ChangeDetectionStrategy
} from '@angular/core';
import { Router, ActivatedRoute, Params } from '@angular/router';

import { MatDialog } from '@angular/material/dialog';

import { Store, select } from '@ngrx/store';

import { FileUploadItem } from '@ucap/domain-common';
import {
  UserInfoSS,
  UserInfoF,
  UserInfoUpdateType,
  User,
  Company
} from '@ucap/domain-organization';

import { FileProfileSaveRequest } from '@ucap/api-common';

import { ConferenceCreateRequest } from '@ucap/api-prompt';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';
import { UserSelector, CompanySelector } from '@ucap/ng-store-organization';
import { BuddySelector } from '@ucap/ng-store-group';
import {
  LoginSelector,
  ConfigurationSelector
} from '@ucap/ng-store-authentication';

import { UserInfoTypes, GroupManageType } from '@app/types';
import { AppFileService } from '@app/services/app-file.service';
import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { AppGroupService } from '@app/services/app-group.service';

import { QueryParams } from '../types/params.type';
import { AppChatService } from '@app/services/app-chat.service';

import {
  EditUserDialogComponent,
  EditUserDialogData,
  EditUserDialogResult
} from '@app/sections/group/dialogs/edit-user.dialog.component';
import { LoginInfo, VersionInfo } from '@ucap/domain-authentication';

@Component({
  selector: 'app-pages-group-index',
  templateUrl: './index.page.component.html',
  styleUrls: ['./index.page.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class IndexPageComponent implements OnInit, OnDestroy {
  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private activatedRoute: ActivatedRoute,
    private store: Store<any>,
    private router: Router,
    private logService: LogService,
    private dialog: MatDialog,
    private i18nService: I18nService,
    private appFileServie: AppFileService,
    private appAuthenticationService: AppAuthenticationService,
    private changeDetectorRef: ChangeDetectorRef,
    private appChatService: AppChatService,
    private appGroupService: AppGroupService
  ) {
    this.profileName = this.i18nService.t('organization:profile.me');
    this.tabName = this.i18nService.t('organization:profile.chatWithUs');
  }

  userSeq: string = undefined;
  user: User;
  loginInfo: LoginInfo;
  versionInfo: VersionInfo;
  activeLink = 0;
  tabName: string;
  profileName: string;
  buddyList: UserInfoTypes[];
  companyList: Company[];

  ngOnInit(): void {
    this.activatedRoute.queryParams
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((params: Params) => {
        const seqParam = params[QueryParams.ID];
        if (!!seqParam) {
          this.userSeq = seqParam;
        } else {
          this.userSeq = '';
        }
        this._refreshProfile();
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
        this._refreshProfile();
      });
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(LoginSelector.loginInfo))
      .subscribe((loginInfo) => {
        this.loginInfo = loginInfo;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(BuddySelector.buddies))
      .subscribe((buddies) => {
        this.buddyList = buddies;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(CompanySelector.companyList)
      )
      .subscribe((companyList) => {
        if (!companyList) {
          return;
        }
        this.companyList = companyList;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onOpenChat(userInfo: UserInfoSS) {
    this.appChatService.newOpenRoom(
      [Number(userInfo.seq) as any],
      false,
      this.user
    );
  }
  onSendMessage(userInfo: UserInfoSS) {}
  onSendCall(call: string) {}
  onSendSms(employeeNum: string) {}
  onCreateConference(userSeq: number) {
    const loginSession = this.appAuthenticationService.getLoginSession();
    const req: ConferenceCreateRequest = {
      userSeq: String(this.user.info.seq),
      deviceType: loginSession.deviceType,
      tokenKey: this.loginInfo.tokenString,
      targetUserSeqs: [userSeq]
    };
    this.appChatService.openVideoConference(req);
  }
  onToggleFavorit(params: { userInfo: UserInfoSS; isFavorite: boolean }) {
    this.appGroupService.updateBuddy(params.userInfo, params.isFavorite);
  }
  onToggleBuddy(params: { userInfo: UserInfoSS; isBuddy: boolean }) {
    if (params.isBuddy) {
      // 동료추가
      const dialogRef = this.dialog.open<
        EditUserDialogComponent,
        EditUserDialogData,
        EditUserDialogResult
      >(EditUserDialogComponent, {
        panelClass: 'max-create-dialog',
        data: {
          title: this.i18nService.t('group:dialog.title.addBuddy'),
          type: GroupManageType.Add,
          userInfos: [params.userInfo]
        }
      });
      dialogRef
        .afterClosed()
        .pipe(
          take(1),
          map((result: EditUserDialogResult) => {
            if (!!result) {
              this.appGroupService.addBuddy(result);
            }
          })
        )
        .subscribe();
    } else {
      // 동료삭제
      this.appGroupService.removeBuddy(params.userInfo).then(() => {
        this.router.navigate(
          [
            'group',
            {
              outlets: { content: 'index' }
            }
          ],
          {
            queryParams: { id: Number(params.userInfo.seq) }
          }
        );
      });
    }
  }
  onUploadProfileImage(profileImageFileUploadItem: FileUploadItem) {
    const loginSession = this.appAuthenticationService.getLoginSession();

    const profile = {
      userSeq: String(this.user.info.seq),
      deviceType: loginSession.deviceType,
      token: this.loginInfo.tokenString,
      file: profileImageFileUploadItem.file,
      fileUploadItem: profileImageFileUploadItem
    } as FileProfileSaveRequest;

    this.appFileServie.fileProfileSave(
      profile,
      this.versionInfo.profileUploadUrl
    );
  }
  onUpdateIntro(intro: string) {
    this.appGroupService.updateIntro(intro, UserInfoUpdateType.Intro);
  }

  onUpdateNickname(params: { userInfo: UserInfoTypes; nickname: string }) {
    const isBuddy = this.appGroupService.checkBuddy(
      this.buddyList,
      params.userInfo
    );

    if (!isBuddy) {
      return;
    }

    this.appGroupService.updateNickname(
      params.userInfo as UserInfoF,
      params.nickname
    );
  }

  private _refreshProfile() {
    if (!this.user || undefined === this.userSeq) {
      return;
    }
    if ('' === this.userSeq) {
      this.userSeq = String(this.user.info.seq);
    }

    if (String(this.user.info.seq) === String(this.userSeq)) {
      this.profileName = this.i18nService.t('organization:profile.me');
      this.tabName = this.i18nService.t('organization:profile.unreadChat');
    } else {
      this.profileName = this.i18nService.t('organization:profile.other');
      this.tabName = this.i18nService.t('organization:profile.chatWithUs');
    }
    this.changeDetectorRef.markForCheck();
  }
}
