import { of, Subject } from 'rxjs';
import { take, map, catchError, takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectorRef,
  ViewChild,
  ChangeDetectionStrategy,
  NgZone
} from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';

import { Store } from '@ngrx/store';

import { MatDialog } from '@angular/material/dialog';

import { UserInfo } from '@ucap/domain-organization';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';

import { SearchData } from '@app/ucap/organization/models/search-data';

import { LoginSession } from '@app/models/login-session';
import { AppUiService } from '@app/services/app-ui.service';
import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { CreateDialogComponent } from '@app/sections/group/dialogs/create.dialog.component';
import { ListSectionComponent } from '@app/sections/group/components/list.section.component';

import { SortViewType } from '../types/sort-view.type';

@Component({
  selector: 'app-pages-group-sidenav',
  templateUrl: './sidenav.page.component.html',
  styleUrls: ['./sidenav.page.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SidenavPageComponent implements OnInit, OnDestroy {
  @ViewChild('sectionGroupList', { static: false })
  sectionGroupList: ListSectionComponent;

  set companySearchData(searchData: SearchData) {
    if (!!searchData && searchData.searchWord !== '') {
      this._companySearchData = { ...searchData, bySearch: true };
    } else {
      this._companySearchData = { ...searchData, bySearch: false };
    }
  }
  get companySearchData() {
    return this._companySearchData;
  }
  // tslint:disable-next-line: variable-name
  _companySearchData: SearchData;
  isBackspaceCanceled = true;
  showType: SortViewType;
  sortViewType = SortViewType;

  private ngOnDestroySubject: Subject<void> = new Subject();
  private loginSession: LoginSession;
  constructor(
    private activatedRoute: ActivatedRoute,
    private router: Router,
    private appAuthenticationService: AppAuthenticationService,
    private logService: LogService,
    private i18nService: I18nService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    public dialog: MatDialog,
    private appUiService: AppUiService,
    private ngZone: NgZone
  ) {}

  ngOnInit(): void {
    this.showType = SortViewType.all;
    this.showGroupMenuIcon(SortViewType.all);

    this.appAuthenticationService
      .getLoginSession$()
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((loginSession) => {
        this.loginSession = loginSession;
        if (
          !!this.loginSession &&
          !!this.loginSession.groupInfo &&
          !!this.loginSession.groupInfo.showType
        ) {
          this.showType = this.loginSession.groupInfo.showType;
          this.showGroupMenuIcon(this.loginSession.groupInfo.showType);
        }
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClickFab(event: MouseEvent) {}

  onClickGroupMenu(menuType: string) {
    switch (menuType) {
      case 'GROUP_NEW':
        {
          const dialogRef = this.dialog.open(CreateDialogComponent, {
            panelClass: 'max-create-dialog'
          });

          dialogRef
            .afterClosed()
            .pipe(
              take(1),
              map((result) => {}),
              catchError((err) => {
                return of(err);
              })
            )
            .subscribe();
        }
        break;
      case 'GROUP_EXPAND_MORE':
        {
          this.sectionGroupList.onExpandMore();
        }
        break;
      case 'GROUP_EXPAND_LESS':
        {
          this.sectionGroupList.onExpandLess();
        }
        break;
      case 'GROUP_CHANGE_ODER':
        {
        }
        break;
    }
  }

  onClickShowGroupMenu(menuType: SortViewType) {
    switch (menuType) {
      case SortViewType.all:
        {
          this.showType = SortViewType.all;
        }
        break;
      case SortViewType.onlineBuddy:
        {
          this.showType = SortViewType.onlineBuddy;
        }
        break;
      case SortViewType.onOff:
        {
          this.showType = SortViewType.onOff;
        }
        break;
    }

    if (
      !!this.loginSession &&
      !!this.loginSession.groupInfo &&
      !!this.loginSession.groupInfo.showType
    ) {
      this.appAuthenticationService.setLoginSession({
        ...this.loginSession,
        groupInfo: {
          groupSeqs: [],
          lastGroupSeq: 0,
          showType: this.showType
        }
      });
    }
  }
  onSearchCancel() {
    this.companySearchData = { ...this.companySearchData, searchWord: '' };
  }

  showGroupMenuIcon(menuType: SortViewType): string {
    if (this.showType === menuType) {
      return 'check_circle';
    }

    return 'check_circle_outline';
  }

  onClickUser(userInfo: UserInfo) {
    this.ngZone.run(() => {
      this.router.navigate(
        [
          'group',
          {
            outlets: { content: 'index' }
          }
        ],
        {
          queryParams: { id: Number(userInfo.seq) }
        }
      );

      this.appUiService.closeLeftSidenavOnNarrowMode();
    });
  }

  onOpenProfile(userInfo: UserInfo) {
    this.onClickUser(userInfo);
  }
}
