import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectorRef,
  ChangeDetectionStrategy
} from '@angular/core';
import { ActivatedRoute, Router, Params } from '@angular/router';

import { ParamsUtil } from '@ucap/ng-core';

import { UserStore } from '@app/models/user-store';
import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { SearchData } from '@app/ucap/organization/models/search-data';

import { QueryParams } from '../types/params.type';

@Component({
  selector: 'app-pages-organization-index',
  templateUrl: './index.page.component.html',
  styleUrls: ['./index.page.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class IndexPageComponent implements OnInit, OnDestroy {
  set companySearchData(searchData: SearchData) {
    this._companySearchData = searchData;
    this.onChangedCompanySearch();
  }
  get companySearchData() {
    return this._companySearchData;
  }
  // tslint:disable-next-line: variable-name
  _companySearchData: SearchData;

  deptSearchData: SearchData;
  isBackspaceCanceled = true;
  deptSeq: string;

  private ngOnDestroySubject: Subject<void> = new Subject();
  private userStore: UserStore;

  constructor(
    private appAuthenticationService: AppAuthenticationService,
    private router: Router,
    private activatedRoute: ActivatedRoute,
    private changeDetectorRef: ChangeDetectorRef
  ) {
    this.userStore = this.appAuthenticationService.getUserStore();
  }

  ngOnInit(): void {
    this.activatedRoute.queryParams
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((params) => {
        if (!!params) {
          const deptSeq = params[QueryParams.DEPT_SEQ];
          const companyCode = params[QueryParams.COMPANY_CODE];
          const searchWord = params[QueryParams.SEARCH_WORD];
          const bySearch = ParamsUtil.getParameter<boolean>(
            params,
            QueryParams.BY_SEARCH,
            false
          );

          if (!!deptSeq && this.deptSeq !== deptSeq) {
            this.deptSeq = deptSeq;
          }

          this.deptSearchData = {
            deptSeq: bySearch ? undefined : deptSeq,
            companyCode: !!companyCode
              ? companyCode
              : this.userStore.companyCode,
            searchWord: bySearch ? decodeURIComponent(searchWord) : undefined,
            bySearch
          };

          this._companySearchData = {
            ...this.deptSearchData
          };

          this.changeDetectorRef.markForCheck();
        }
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onChangedCompanySearch() {
    const queryParams: Params = {};
    queryParams[QueryParams.COMPANY_CODE] = this._companySearchData.companyCode;
    queryParams[QueryParams.SEARCH_WORD] = encodeURIComponent(
      this._companySearchData.searchWord
    );
    queryParams[QueryParams.BY_SEARCH] = String(true);

    this._navigate(queryParams);
  }

  onCanceledSearch() {
    const queryParams: Params = {};

    queryParams[QueryParams.DEPT_SEQ] = String(this.deptSeq);
    this._navigate(queryParams);
  }

  private _navigate(queryParams: Params = {}) {
    this.router.navigate(
      [
        'organization',
        {
          outlets: { content: 'index' }
        }
      ],
      {
        queryParams
      }
    );
  }
}
