import { Subject, of, merge } from 'rxjs';
import { takeUntil, map, catchError, take } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  ViewChild,
  ElementRef,
  EventEmitter,
  Output
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { MatDialog } from '@angular/material/dialog';

import { FileUploadItem } from '@ucap/domain-common';
import { User } from '@ucap/domain-organization';
import { UserPermission } from '@ucap/domain-authorization';
import {
  RoomInfo,
  EventType,
  MassTranslationEventJson,
  TranslationEventJson
} from '@ucap/domain-chat';

import { StickerFilesInfo } from '@ucap/ng-core';
import { StatusCode } from '@ucap/api';
import {
  TranslationSaveResponse,
  TranslationSaveRequest
} from '@ucap/api-common';

import {
  SendEventMailType,
  SendEventEmailRequest,
  SendEventEmailResponse,
  StatusCode as PiStatusCode
} from '@ucap/pi';

import { I18nService } from '@ucap/ng-i18n';
import { LogService } from '@ucap/ng-logger';
import { CommonApiService } from '@ucap/ng-api-common';
import { PiService } from '@ucap/ng-pi';

import { UserSelector } from '@ucap/ng-store-organization';
import {
  LoginSelector,
  ConfigurationSelector,
  AuthorizationSelector
} from '@ucap/ng-store-authentication';
import { ChattingSelector, RoomSelector } from '@ucap/ng-store-chat';

import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult,
  ConfirmDialogComponent,
  ConfirmDialogData,
  ConfirmDialogResult
} from '@ucap/ng-ui/core';

import { LoginSession } from '@app/models/login-session';
import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { AppFileService } from '@app/services/app-file.service';
import { FileUploadSelectorComponent } from '@app/ucap/chat/components/file-upload.selector.component';
import { AppChatService } from '@app/services/app-chat.service';

import { environment } from '@environments';
import { LoginInfo, VersionInfo } from '@ucap/domain-authentication';

export enum SelectorType {
  EMPTY = '',
  STICKER = 'STICKER',
  TRANSLATION = 'TRANSLATION',
  FILEUPLOAD = 'FILEUPLOAD',
  EMAILSENDER = 'EMAILSENDER'
}

@Component({
  selector: 'app-sections-chat-form',
  templateUrl: './form.section.component.html',
  styleUrls: ['./form.section.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class FormSectionComponent implements OnInit, OnDestroy {
  private roomIdSubject = new Subject<string>();
  private ngOnDestroySubject: Subject<void> = new Subject();
  @Input()
  set roomId(roomId: string) {
    if (!!roomId && this.roomId !== roomId) {
      this._roomId = roomId;

      this.roomIdSubject.next(roomId);

      this.initializeRoomData();
    }
  }
  get roomId(): string {
    return this._roomId;
  }
  // tslint:disable-next-line: variable-name
  _roomId: string;

  @Output()
  changeTranslationSimpleview = new EventEmitter<boolean>();

  @Output()
  openFormSelector = new EventEmitter<string>();

  @Output()
  eventSendTrigger = new EventEmitter<any>();

  versionInfo: VersionInfo;
  loginSession: LoginSession;
  loginInfo: LoginInfo;
  user: User;
  userPermission: UserPermission;

  currentRoomInfo: RoomInfo;

  selectorType: SelectorType = SelectorType.EMPTY;

  /** About Sticker */
  selectedSticker: StickerFilesInfo;

  /** About Translation */
  isTranslationProcess = false;
  translationSimpleview = false;
  translationPreview = false;
  destLocale = 'en'; // default English :: en
  translationPreviewInfo: {
    previewInfo: TranslationSaveResponse | null;
    translationType: EventType.Translation | EventType.MassTranslation;
  };

  @ViewChild('messageInput', { static: false })
  messageInput: ElementRef<HTMLTextAreaElement>;
  @ViewChild('fileInput', { static: false })
  fileInput: ElementRef<HTMLInputElement>;
  @ViewChild('fileUploadSelector', { static: false })
  fileUploadSelector: FileUploadSelectorComponent;

  SelectorType = SelectorType;

  constructor(
    private piService: PiService,
    private appFileService: AppFileService,
    private appChatService: AppChatService,
    private store: Store<any>,
    private i18nService: I18nService,
    private dialog: MatDialog,
    private logService: LogService,
    private appAuthenticationService: AppAuthenticationService,
    private commonApiService: CommonApiService,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(LoginSelector.loginInfo))
      .subscribe((loginInfo) => {
        this.loginInfo = loginInfo;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(AuthorizationSelector.userPermission)
      )
      .subscribe((userPermission) => {
        this.userPermission = userPermission;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }

    if (!!this.roomIdSubject) {
      this.roomIdSubject.next();
      this.roomIdSubject.complete();
    }
  }

  initializeRoomData() {
    if (!!this.messageInput) {
      this.messageInput.nativeElement.value = '';
    }
    this.selectorType = SelectorType.EMPTY;
    this.translationSimpleview = false;
    this.changeTranslationSimpleview.emit(false);
    this.translationPreview = false;
    this.destLocale = 'en'; // default English :: en
    this.translationPreviewInfo = null;
    this.selectedSticker = undefined;

    this.store
      .pipe(
        takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)),
        select(RoomSelector.room, this.roomId)
      )
      .subscribe((room) => {
        this.currentRoomInfo = room;
        this.changeDetectorRef.markForCheck();
      });

    this.loginSession = this.appAuthenticationService.getLoginSession();
  }
  /** About Selector */
  onOpenSelector(type: SelectorType): void {
    this.selectorType = type;

    this.selectedSticker = null;
    this.translationPreviewInfo = null;

    this.changeDetectorRef.detectChanges();
    this.openFormSelector.emit(type);
  }
  clearSelector(): void {
    this.selectorType = SelectorType.EMPTY;

    this.selectedSticker = null;
    this.translationPreviewInfo = null;

    this.changeDetectorRef.markForCheck();
  }

  /** Element Handling */
  focus(clearField: boolean = true): void {
    if (!!this.messageInput) {
      if (!!clearField) {
        if (this.selectorType !== SelectorType.TRANSLATION) {
          this.clearSelector();
        }
        this.messageInput.nativeElement.value = '';
      }
      this.messageInput.nativeElement.focus();
    }
  }

  onKeydown(event: KeyboardEvent) {
    // if (event.key === 'PageUp' || event.key === 'PageDown') {
    //   event.preventDefault();
    //   return false;
    // } else if (event.key === 'Enter' && !event.shiftKey) {
    //   event.preventDefault();
    //   this.send();
    // }
    event.preventDefault();
    this.send();
  }

  textareaResize(event: KeyboardEvent, obj) {
    const self = obj;
    setTimeout(() => {
      self.style.height = '1px';
      self.style.height = self.scrollHeight + 'px';
    });
  }

  onSelectedSticker(stickerInfo: StickerFilesInfo) {
    this.selectedSticker = stickerInfo;
    this.focus(false);
  }

  onPasteReply(event: ClipboardEvent) {
    event.preventDefault();

    // this.platform_readFromClipboard().then(async (data) => {
    //   console.log(data);
    //   console.log(data.html);
    //   console.log(data.image);
    //   console.log(data.imageDataUrl);
    //   console.log(data.text);

    //   console.log(JSON.stringify(event.clipboardData.items));
    //   if ((!!data.image && !!data.text) || !!data.image) {
    //   }
    // });
    // for (const item of items) {
    //   if (item.type.indexOf('image') === 0) {
    //     blob = item.getAsFile();
    //   }
    // }

    // console.log(JSON.stringify(event.clipboardData.items));
    // this.nativeService.platform_readFromClipboard().then(async (data) => {
    //   console.log(data);
    //   console.log(JSON.stringify(event.clipboardData.items));
    //   if ((!!data.image && !!data.text) || !!data.image) {
    //   }
    // });
  }
  async send() {
    const roomId = this.currentRoomInfo.roomId;
    const userSeq = String(this.user.info.seq);
    let message = this.messageInput.nativeElement.value;

    if (!!message || message.trim().length > 0) {
      message = message.replace(/\t/g, '    ');
    }

    // Empty Check.
    if (!this.selectedSticker) {
      try {
        if (!message || message.trim().length === 0) {
          const result = await this.dialog.open<
            AlertDialogComponent,
            AlertDialogData,
            AlertDialogResult
          >(AlertDialogComponent, {
            panelClass: 'min-create-dialog',
            data: {
              title: this.i18nService.t('chat:errors.label'),
              message: this.i18nService.t('chat:errors.inputChatMessage')
            }
          });
          return;
        }
      } catch (e) {
        this.logService.debug(e);
      }
    }

    if (
      this.selectorType === SelectorType.TRANSLATION &&
      this.destLocale.trim().length > 0
    ) {
      /** CASE : Translation */
      // 번역할 대화 없이 스티커만 전송할 경우.
      if (!message || message.trim().length === 0) {
        this.appChatService.sendMessageOfSticker(
          userSeq,
          roomId,
          this.selectedSticker,
          message
        );

        this.clearSelector();
      } else {
        this.sendMessageOfTranslate(message);
      }
    } else if (!!this.selectedSticker) {
      /** CASE : Sticker */
      this.appChatService.sendMessageOfSticker(
        userSeq,
        roomId,
        this.selectedSticker,
        message
      );

      this.clearSelector();
    } else if (
      message.trim().length > environment.productConfig.chat.masstextLength
    ) {
      /** CASE : MASS TEXT */
      this.appChatService.sendMessageOfMassText(
        this.loginInfo,
        this.user,
        this.loginSession.deviceType,
        roomId,
        message
      );
    } else {
      /** CASE : Normal Text */
      this.appChatService.sendMessageOfNormal(userSeq, roomId, message);
    }

    this.eventSendTrigger.emit(0);
    this.focus();
  }

  sendMessageOfTranslate(message: string) {
    if (!!this.isTranslationProcess) {
      return;
    }

    this.isTranslationProcess = true;

    this.commonApiService
      .translationSave({
        userSeq: String(this.user.info.seq),
        token: this.loginInfo.tokenString,
        deviceType: this.loginSession.deviceType,
        original: message,
        roomId: this.roomId,
        srcLocale: '',
        destLocale: this.destLocale
      } as TranslationSaveRequest)
      .pipe(
        take(1),
        map((res) => {
          if (res.statusCode === StatusCode.Success) {
            let sentMessage = '';
            let eventType = EventType.Translation;
            let previewObject: TranslationEventJson | MassTranslationEventJson;
            if (res.translationSeq > 0) {
              // Mass Text Translation
              previewObject = res;
              sentMessage = res.returnJson;
              eventType = EventType.MassTranslation;
            } else {
              // Normal Text Translation
              previewObject = {
                locale: this.destLocale,
                original: message,
                translation: res.translation,
                stickername: '',
                stickerfile: !!this.selectedSticker
                  ? this.selectedSticker.index
                  : ''
              };
              sentMessage = JSON.stringify(previewObject);
              eventType = EventType.Translation;
            }

            if (!!this.translationPreview) {
              this.translationPreviewInfo = {
                previewInfo: res,
                translationType: eventType
              };
              this.changeDetectorRef.markForCheck();
            } else {
              this._sendTranslationMessage(sentMessage, eventType);
            }
          } else {
            // error
            this._translationError();
          }
        }),
        catchError((error) => {
          this._translationError();
          return of(this.logService.error('error', error));
        })
      )
      .subscribe(() => {
        this.isTranslationProcess = false;
      });
  }

  onChangeTranslationSimpleView(value: boolean) {
    this.translationSimpleview = value;
    this.changeTranslationSimpleview.emit(value);
  }
  onChangeTranslationPreView(value: boolean) {
    this.translationPreview = value;
  }
  onChangeDestLocale(destLocale: string) {
    this.destLocale = destLocale;
  }
  onSendTranslationMessage(params: {
    previewInfo: TranslationSaveResponse | null;
    translationType: EventType.Translation | EventType.MassTranslation;
  }) {
    let sentMessage = '';
    if (params.translationType === EventType.MassTranslation) {
      // Mass Text Translation
      sentMessage = params.previewInfo.returnJson;
    } else {
      sentMessage = JSON.stringify({
        locale: params.previewInfo.destLocale,
        original: params.previewInfo.original,
        translation: params.previewInfo.translation,
        stickername: '',
        stickerfile: !!this.selectedSticker ? this.selectedSticker.index : ''
      });
    }

    this._sendTranslationMessage(sentMessage, params.translationType);
  }

  private _translationError() {
    this.isTranslationProcess = false;
    this.dialog.open<AlertDialogComponent, AlertDialogData, AlertDialogResult>(
      AlertDialogComponent,
      {
        data: {
          title: this.i18nService.t('chat:errors.label'),
          message: this.i18nService.t('chat:errors.translateServerError')
        },
        panelClass: 'min-create-dialog'
      }
    );
  }
  private _sendTranslationMessage(sentMessage: string, eventType: EventType) {
    this.appChatService.sendMessageOfTranslate(
      String(this.user.info.seq),
      this.roomId,
      eventType,
      sentMessage
    );
    this.translationPreviewInfo = undefined;
    this.eventSendTrigger.emit(0);
    this.focus();
  }

  onSendEventEmail(type: SendEventMailType): void {
    this.store
      .pipe(take(1), select(ChattingSelector.eventList, this._roomId))
      .subscribe((eventList) => {
        if (!!eventList && eventList.length > 0) {
          const self = this;
          const dialogRef = this.dialog.open<
            ConfirmDialogComponent,
            ConfirmDialogData,
            ConfirmDialogResult
          >(ConfirmDialogComponent, {
            panelClass: 'min-create-dialog',
            data: {
              title: this.i18nService.t('chat:label.emailSend'),
              html: this.i18nService.t(
                type === SendEventMailType.ALL
                  ? 'chat:dialog.confirmSendEventEmailAll'
                  : 'chat:dialog.confirmSendEventEmailMe'
              )
            }
          });

          dialogRef
            .afterClosed()
            .pipe(take(1))
            .subscribe((result) => {
              if (!!result && !!result.choice) {
                const req: SendEventEmailRequest = {
                  userSeq: String(this.user.info.seq),
                  deviceType: this.loginSession.deviceType,
                  tokenKey: this.loginInfo.tokenString,
                  roomSeq: this._roomId,
                  eventSeq: String(eventList[0].seq),
                  sendType: type
                };
                this.piService
                  .sendEventEmail(req)
                  .pipe(take(1))
                  .subscribe(
                    (res: SendEventEmailResponse) => {
                      if (res.intStatusCode === PiStatusCode.Success) {
                        this.dialog.open<
                          AlertDialogComponent,
                          AlertDialogData,
                          AlertDialogResult
                        >(AlertDialogComponent, {
                          panelClass: 'min-create-dialog',
                          data: {
                            title: this.i18nService.t('chat:label.emailSend'),
                            message: this.i18nService.t(
                              'chat:dialog.sendEventEmailSuccess'
                            )
                          }
                        });

                        self.selectorType = SelectorType.EMPTY;
                      } else {
                        this.logService.error(res.strErrorMessage);
                      }
                    },
                    (error) => {
                      this.logService.error(error);
                    },
                    () => {}
                  );

                this.logService.debug(
                  this.i18nService.t('chat:dialog.sendEventEmailSuccess')
                );
              }
            });
        }
      });
  }

  /**
   * About File Upload
   */
  /** FileInput change event. */
  onChangeFileInput(): void {
    const fileList = this.fileInput.nativeElement.files;
    this._fileSend(fileList);
  }

  /** DragAndDrop :: Add fileupload queue. not sending. */
  onDragAndDropStandbyFileupload(fileList: FileList) {
    const self = this;
    this.appFileService
      .validUploadFile(fileList, this.versionInfo?.fileAllowSize)
      .then((result) => {
        if (!result) {
          self.fileInput.nativeElement.value = '';
          return;
        } else {
          // selector open
          self.onOpenSelector(SelectorType.FILEUPLOAD);

          // FileuploadItem Init. & FileSelector Init.
          const fileUploadItems = FileUploadItem.fromFiles(fileList);
          if (!!self.fileUploadSelector) {
            self.fileUploadSelector.onFileSelected(fileUploadItems);
          }
          self.fileInput.nativeElement.value = '';
        }
      })
      .catch((err) => {
        self.fileInput.nativeElement.value = '';
        self.logService.error(`validUploadFile ${err}`);
      });
  }

  /** DragAndDrop :: fileUpload direct. */
  onDragAndDropFileupload(fileList: FileList): void {
    this._fileSend(fileList);
  }

  private _fileSend(fileList: FileList) {
    const self = this;
    this.appFileService
      .validUploadFile(fileList, this.versionInfo?.fileAllowSize)
      .then((result) => {
        if (!result) {
          self.fileInput.nativeElement.value = '';
          return;
        } else {
          // selector open
          self.onOpenSelector(SelectorType.FILEUPLOAD);
          self.changeDetectorRef.detectChanges();

          // FileuploadItem Init. & FileSelector Init.
          const fileUploadItems = FileUploadItem.fromFiles(fileList);
          if (!!self.fileUploadSelector) {
            self.fileUploadSelector.onFileSelected(fileUploadItems);
          }
          self.fileInput.nativeElement.value = '';

          // File Upload..
          self.appChatService
            .sendMessageOfAttachFile(
              self.loginInfo,
              self.user,
              self.loginSession.deviceType,
              self.currentRoomInfo.roomId,
              fileUploadItems
            )
            .then((success) => {
              if (!!success) {
                self.eventSendTrigger.emit(0);
                self.clearSelector();
                if (!!self.fileUploadSelector) {
                  self.fileUploadSelector.onUploadComplete();
                }
              }
            })
            .catch((err) => {
              self.clearSelector();
              if (!!self.fileUploadSelector) {
                self.fileUploadSelector.onUploadComplete();
              }

              const msg = this.i18nService.t('common:file.errors.failToUpload');
              alert(msg);
            });
        }
      })
      .catch((err) => {
        self.fileInput.nativeElement.value = '';
        self.logService.error(`validUploadFile ${err}`);
      });
  }
}
