import { combineLatest, Subject, merge } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  Input,
  Output,
  EventEmitter,
  ChangeDetectorRef
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { MatDialog } from '@angular/material/dialog';

import { LocaleCode } from '@ucap/domain-common';
import { User } from '@ucap/domain-organization';
import { RoomInfo, RoomType } from '@ucap/domain-chat';

import { I18nService } from '@ucap/ng-i18n';

import { UpdateRequest } from '@ucap/protocol-room';

import { UserSelector } from '@ucap/ng-store-organization';
import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import {
  RoomSelector,
  RoomActions,
  RoomUserMap,
  RoomUserShortMap
} from '@ucap/ng-store-chat';

import {
  TranslatePipe as OrganizationTranslate,
  TranslateService
} from '@ucap/ng-ui-organization';

import { AppChatService } from '@app/services/app-chat.service';
import { ChatDrawType } from '@app/pages/chat/types/chat-draw.type';
import { DrawInfo } from '@app/pages/chat/models/draw-info';
import {
  ProfileDialogComponent,
  ProfileDialogData,
  ProfileDialogResult
} from '@app/sections/organization/dialogs/profile.dialog.component';
import { VersionInfo } from '@ucap/domain-authentication';

@Component({
  selector: 'app-sections-chat-info',
  templateUrl: './info.section.component.html',
  styleUrls: ['./info.section.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class InfoSectionComponent implements OnInit, OnDestroy {
  private roomIdSubject = new Subject<string>();
  private ngOnDestroySubject: Subject<void> = new Subject();

  @Input()
  set roomId(roomId: string) {
    if (this._roomId !== roomId) {
      this._roomId = roomId;

      this.roomIdSubject.next(roomId);

      this.initializeRoomData();
    }

    // request selected room
    if (!!this.roomId) {
      this.store.dispatch(
        RoomActions.selectedRoom({
          roomId: this.roomId,
          localeCode: LocaleCode.Korean
        })
      );
    }
  }
  get roomId(): string {
    return this._roomId;
  }
  // tslint:disable-next-line: variable-name
  _roomId: string;

  @Output()
  openChatSearch = new EventEmitter<void>();

  @Output()
  rightDrawerToggle = new EventEmitter<DrawInfo>();

  versionInfo: VersionInfo;
  user: User;

  defaultProfileImage: string;
  defaultProfileImageMulti: string;

  currentRoomInfo: RoomInfo;
  roomList: RoomInfo[];
  roomUsersMap: RoomUserMap;
  roomUsersShortMap: RoomUserShortMap;

  roomName: string;
  roomImage: string;

  RoomType = RoomType;
  organizationTranslate: OrganizationTranslate;

  constructor(
    private store: Store<any>,
    private appChatService: AppChatService,
    private i18nService: I18nService,
    private translateService: TranslateService,
    private dialog: MatDialog,
    private changeDetectorRef: ChangeDetectorRef
  ) {
    // language setting
    this.translateService.setDefaultLang(this.i18nService.currentLng);
    this.translateService.use(this.i18nService.currentLng);
    this.organizationTranslate = new OrganizationTranslate(
      this.translateService,
      this.changeDetectorRef
    );

    // default image setting
    this.defaultProfileImage = this.appChatService.defaultProfileImage;
    this.defaultProfileImageMulti = this.appChatService.defaultProfileImageMulti;
  }

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }

    if (!!this.roomIdSubject) {
      this.roomIdSubject.next();
      this.roomIdSubject.complete();
    }
  }

  initializeRoomData() {
    combineLatest([
      this.store.pipe(select(UserSelector.user)),
      this.store.pipe(select(RoomSelector.room, this.roomId)),
      this.store.pipe(select(RoomSelector.roomUser, this.roomId)),
      this.store.pipe(select(RoomSelector.roomUserShort, this.roomId))
    ])
      .pipe(takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)))
      .subscribe(([user, room, roomUsers, roomUsersShort]) => {
        this.user = user;
        this.currentRoomInfo = room;
        this.roomUsersMap = roomUsers;
        this.roomUsersShortMap = roomUsersShort;

        this.getRoomInfo();
      });
  }

  getRoomInfo(): void {
    this.roomName = '...';
    if (!!this.currentRoomInfo) {
      // Setting RoomName.
      this.roomName = this.appChatService.getRoomName(
        this.organizationTranslate,
        this.user,
        this.currentRoomInfo,
        this.roomUsersMap,
        this.roomUsersShortMap
      );

      this.roomImage = this.appChatService.getRoomProfileImage(
        this.user,
        this.currentRoomInfo,
        this.roomUsersMap,
        this.roomUsersShortMap
      );
    }
    this.changeDetectorRef.markForCheck();
  }

  onToggleAlarm(roomInfo: RoomInfo): void {
    if (!roomInfo) {
      return;
    }

    this.store.dispatch(
      RoomActions.update({
        req: {
          roomId: roomInfo.roomId,
          roomName:
            roomInfo.roomName.trim().length === 0
              ? ' '
              : roomInfo.roomName.trim(),
          receiveAlarm: !roomInfo.receiveAlarm,
          syncAll: false
        } as UpdateRequest
      })
    );
    this.changeDetectorRef.markForCheck();
  }

  onOpenChatSearch(): void {
    this.openChatSearch.emit();
  }

  onRightDrawerToggle(): void {
    this.rightDrawerToggle.emit();
  }

  onClickRoomMenu(type: string): void {
    switch (type) {
      case 'ATTACH_IMAGE':
        this.rightDrawerToggle.emit({ chatDrawType: ChatDrawType.AttachImage });
        break;
      case 'ATTACH_VIDEO':
        this.rightDrawerToggle.emit({ chatDrawType: ChatDrawType.AttachVideo });
        break;
      case 'ATTACH_FILE':
        this.rightDrawerToggle.emit({ chatDrawType: ChatDrawType.AttachFile });
        break;
      case 'ADD_GROUP':
        this.rightDrawerToggle.emit({ chatDrawType: ChatDrawType.AddGroup });
        break;
      case 'ROOM_USERS':
        this.rightDrawerToggle.emit({ chatDrawType: ChatDrawType.RoomUsers });
        break;
      case 'CHANGE_ROOM_USERS':
        this.rightDrawerToggle.emit({ chatDrawType: ChatDrawType.Invite });
        break;
      case 'SETTING':
        this.rightDrawerToggle.emit({ chatDrawType: ChatDrawType.Setting });
        break;
      case 'EXIT':
        this.appChatService.exitRoomDialog(this.currentRoomInfo);
        break;
    }
  }

  onOpenProfile(): void {
    if (
      !!this.currentRoomInfo &&
      this.currentRoomInfo.roomType === RoomType.Single
    ) {
      const returnValue = this.appChatService.getRoomUserList01(
        this.user,
        this.roomUsersMap,
        this.roomUsersShortMap
      );
      if (
        !!returnValue &&
        !!returnValue.existUsers &&
        returnValue.users.length > 0
      ) {
        const userSeq = String(returnValue.users[0].seq);
        this.dialog.open<
          ProfileDialogComponent,
          ProfileDialogData,
          ProfileDialogResult
        >(ProfileDialogComponent, {
          panelClass: 'mid-create-dialog',
          data: {
            userSeq
          }
        });
      }
    }
  }

  getShowContextMenu(menuType: string) {
    if (
      ['EVENT', 'ROOM_USERS', 'CHANGE_ROOM_USERS', 'ADD_GROUP', 'SETTING'].some(
        (v) => v === menuType
      )
    ) {
      if (
        !this.currentRoomInfo ||
        !this.currentRoomInfo.roomType ||
        [RoomType.Mytalk, RoomType.Allim, RoomType.Bot, RoomType.Link].some(
          (v) => v === this.currentRoomInfo.roomType
        )
      ) {
        return false;
      }
    }
    // else if (['CHAT_EXPORT'].some((v) => v === menuType)) {
    //   if (
    //     !this.currentRoomInfo ||
    //     !this.currentRoomInfo.roomType ||
    //     [RoomType.Allim, RoomType.Bot, RoomType.Link].some(
    //       (v) => v === this.currentRoomInfo.roomType
    //     )
    //   ) {
    //     return false;
    //   }
    // }

    return true;
  }
}
