import moment from 'moment';

import {
  Subject,
  merge,
  combineLatest,
  Observable,
  fromEvent,
  interval
} from 'rxjs';
import {
  takeUntil,
  take,
  tap,
  withLatestFrom,
  debounce,
  debounceTime
} from 'rxjs/operators';

import { PerfectScrollbarDirective } from 'ngx-perfect-scrollbar';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  ViewChild,
  ElementRef,
  Inject
} from '@angular/core';

import { Store, select } from '@ngrx/store';
import { Dictionary } from '@ngrx/entity';

import { MatDialog, MatDialogRef } from '@angular/material/dialog';

import { DeviceType, FileDownloadItem } from '@ucap/domain-common';
import { User } from '@ucap/domain-organization';
import {
  Info,
  EventJson,
  EventType,
  FileType,
  MassTranslationEventJson,
  FileEventJson,
  UserInfo as RoomUserInfo,
  UserInfoShort as RoomUserInfoShort,
  RoomInfo,
  RoomType,
  FileInfo
} from '@ucap/domain-chat';

import { TransMassTalkDownloadRequest } from '@ucap/api-common';

import {
  ReadRequest,
  SSVC_TYPE_EVENT_SEND_NOTI,
  SendNotification
} from '@ucap/protocol-event';
import { UpdateTimerSetRequest, UpdateRequest } from '@ucap/protocol-room';

import { NativeService } from '@ucap/native';

import { I18nService } from '@ucap/ng-i18n';
import { UserSelector } from '@ucap/ng-store-organization';
import {
  LoginSelector,
  ConfigurationSelector,
  AuthorizationSelector
} from '@ucap/ng-store-authentication';
import {
  ChattingSelector,
  RoomSelector,
  ChattingActions,
  RoomActions,
  ChatUtil
} from '@ucap/ng-store-chat';
import {
  ConfirmDialogComponent,
  ConfirmDialogData,
  ConfirmDialogResult,
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult,
  ClipboardService
} from '@ucap/ng-ui/core';
import { SelectFileInfo } from '@ucap/ng-ui/viewer';
import {
  VirtualScrollViewportComponent,
  MeasureSizeVirtualScrollDirective
} from '@ucap/ng-ui/scrolling';

import { EventProtocolService } from '@ucap/ng-protocol-event';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { LogService } from '@ucap/ng-logger';

import { LoginSession } from '@app/models/login-session';
import { AppChatService } from '@app/services/app-chat.service';
import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { AppOrganizationService } from '@app/services/app-organization.service';
import { AppFileService } from '@app/services/app-file.service';

import {
  ProfileDialogComponent,
  ProfileDialogData,
  ProfileDialogResult
} from '@app/sections/organization/dialogs/profile.dialog.component';

import {
  ForwardDialogComponent,
  ForwardDialogData,
  ForwardDialogResult
} from '../dialogs/forward.dialog.component';
import { FileViewerDialogData } from '../dialogs/file-viewer.dialog.component';
import {
  SettingDialogComponent,
  SettingDialogData,
  SettingDialogResult
} from '../dialogs/setting.dialog.component';
import { ConferenceJoinRequest } from '@ucap/api-prompt';
import { LoginInfo, VersionInfo } from '@ucap/domain-authentication';
import { UserPermission } from '@ucap/domain-authorization';

@Component({
  selector: 'app-sections-chat-message',
  templateUrl: './message.section.component.html',
  styleUrls: ['./message.section.component.scss'],

  changeDetection: ChangeDetectionStrategy.OnPush
})
export class MessageSectionComponent implements OnInit, OnDestroy {
  private roomIdSubject = new Subject<string>();
  private ngOnDestroySubject: Subject<void> = new Subject();

  @Input()
  set roomId(roomId: string) {
    if (this._roomId !== roomId) {
      this._roomId = roomId;

      this.roomIdSubject.next(roomId);

      this.initializeRoomData();
    }
  }
  get roomId(): string {
    return this._roomId;
  }
  // tslint:disable-next-line: variable-name
  _roomId: string;

  @Input()
  translationSimpleview: boolean;

  @Input()
  eventSendTrigger$: Observable<any>;

  versionInfo: VersionInfo;
  loginInfo: LoginInfo;
  user: User;
  loginSession: LoginSession;
  userPermission: UserPermission;

  currentRoomInfo: RoomInfo;
  isRemainEvent = false;
  currentFileInfoList: FileInfo[] = [];
  roomUsers: RoomUserInfoShort[] = [];
  // eventList$: Observable<Info<EventJson>[]>;
  eventList: Info<EventJson>[];

  defaultProfileImage: string;
  EventType = EventType;
  FileType = FileType;

  /** 최근 메시지 */
  recentMessage: Info<EventJson>;
  recentUserInfo: RoomUserInfoShort | RoomUserInfo | undefined;

  /** Timer 대화방의 대화 삭제를 위한 interval */
  interval: any;

  /** new Chat room setting dialog */
  settingDialogRef: MatDialogRef<SettingDialogComponent, SettingDialogResult>;

  /** About Scroll */
  isInitScrollbottom = true;
  isScrollReachBottom = false;
  @ViewChild('chatMessagesContainer', { static: true })
  chatMessagesContainer: ElementRef<HTMLElement>;

  @ViewChild('vsList', { static: false })
  vsList: VirtualScrollViewportComponent;

  @ViewChild('vsList', {
    static: true,
    read: MeasureSizeVirtualScrollDirective
  })
  measureSize: MeasureSizeVirtualScrollDirective;

  @ViewChild(PerfectScrollbarDirective, {
    static: true,
    read: PerfectScrollbarDirective
  })
  psDirectiveRef: PerfectScrollbarDirective;

  String = String;

  constructor(
    private appChatService: AppChatService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private dialog: MatDialog,
    private i18nService: I18nService,
    private eventProtocolService: EventProtocolService,
    private appAuthenticationService: AppAuthenticationService,
    private appOrganizationService: AppOrganizationService,
    private appFileService: AppFileService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private logService: LogService
  ) {
    this.defaultProfileImage = this.appChatService.defaultProfileImage;

    // scroll handling when window size changing.
    fromEvent(window, 'resize')
      .pipe(takeUntil(this.ngOnDestroySubject), debounceTime(300))
      .subscribe((event: any) => {
        this.refreshAndScrollToBottom();
      });

    this.eventProtocolService.notification$
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        withLatestFrom(this.store.pipe(select(RoomSelector.rooms))),
        tap(([notiOrRes, roomList]) => {
          switch (notiOrRes.SSVC_TYPE) {
            case SSVC_TYPE_EVENT_SEND_NOTI:
              {
                const noti = notiOrRes as SendNotification;
                const roomId = noti.roomId;
                const eventInfo = noti.info;
                const trgtRoom = roomList.find(
                  (roomInfo) => roomInfo.roomId === roomId
                );

                const contents = ChatUtil.convertFinalEventMessage(
                  noti.eventType,
                  noti.info.sentMessageJson
                );

                this.logService.debug(
                  'Notification::eventProtocolService::SendNotification in message.section.components',
                  noti
                );

                if (!!roomId && roomId === this.roomId) {
                  if (!!this.isScrollReachBottom) {
                    // Fires when I enter the event
                    this.isInitScrollbottom = true;
                  } else {
                    this.recentMessage = noti.info;
                    this.recentUserInfo = this.getSenderInfo(
                      Number(noti.SENDER_SEQ)
                    );
                  }
                }
              }
              break;
            default:
              // ignore..
              break;
          }
        })
      )
      .subscribe();
  }

  ngOnInit(): void {
    this.vsList
      .elementScrolled()
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(() => {
        if (!!this.psDirectiveRef) {
          this.psDirectiveRef.update();
        }
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(AuthorizationSelector.userPermission)
      )
      .subscribe((userPermission) => {
        this.userPermission = userPermission;
      });

    this.eventSendTrigger$
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((_) => {
        // Fires when I enter the event
        this.isInitScrollbottom = true;
      });

    this.nativeService
      .window_onFocus$()
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((isFocus) => {
        if (!!this.roomId && !!this.eventList && this.eventList.length > 0) {
          const roomId = this.roomId;
          const lastReadSeq = this.eventList[this.eventList.length - 1].seq;
          this.store.dispatch(
            ChattingActions.read({
              roomId,
              lastReadSeq
            } as ReadRequest)
          );
        }
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }

    if (!!this.roomIdSubject) {
      this.roomIdSubject.next();
      this.roomIdSubject.complete();
    }

    if (!!this.interval) {
      clearInterval(this.interval);
      this.interval = undefined;
    }
  }

  initializeRoomData() {
    /** About initialize roomId */
    this.isInitScrollbottom = true;
    this.onClickInvalidateCache();

    this.loginSession = this.appAuthenticationService.getLoginSession();
    this.store
      .pipe(take(1), select(ConfigurationSelector.versionInfo))
      .subscribe((versionInfo) => (this.versionInfo = versionInfo));
    this.store
      .pipe(take(1), select(LoginSelector.loginInfo))
      .subscribe((loginInfo) => (this.loginInfo = loginInfo));
    this.store
      .pipe(take(1), select(UserSelector.user))
      .subscribe((user) => (this.user = user));

    this.store
      .pipe(
        takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)),
        select(RoomSelector.roomUserShort, this.roomId)
      )
      .subscribe((roomUserShort) => {
        if (!!roomUserShort) {
          this.roomUsers = roomUserShort.userInfos;
          this.changeDetectorRef.markForCheck();
        }
      });

    this.store
      .pipe(
        takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)),
        select(ChattingSelector.chatting, this.roomId)
      )
      .subscribe((chatting) => {
        if (!!chatting && !!chatting.remainEvent) {
          this.isRemainEvent = chatting.remainEvent;
        } else {
          this.isRemainEvent = false;
        }

        if (
          !!chatting &&
          !!chatting.fileInfoList &&
          chatting.fileInfoList.ids.length > 0
        ) {
          const tempFileInfoList: FileInfo[] = [];
          const fileInfoDic = chatting.fileInfoList.entities as Dictionary<
            FileInfo
          >;

          for (const key in fileInfoDic) {
            if (key === undefined) {
              continue;
            }

            if (fileInfoDic.hasOwnProperty(key)) {
              const fileInfo = fileInfoDic[key];
              tempFileInfoList.push(fileInfo);
            }
          }

          this.currentFileInfoList = tempFileInfoList;
        }
      });

    this.eventList = [];
    this.store
      .pipe(
        takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)),
        select(ChattingSelector.eventList, this.roomId)
      )
      .subscribe((eventList) => {
        if (!!eventList && eventList.length > 0) {
          this.eventList = eventList;
        }
      });

    /** Settings popup that opens when you first open a chat room */
    combineLatest([
      this.store.pipe(select(RoomSelector.room, this.roomId)),
      this.store.pipe(select(RoomSelector.standbyRooms))
    ])
      .pipe(takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)))
      .subscribe(([roomInfo, standbyRooms]) => {
        // get room info.
        if (!!roomInfo) {
          if (
            !!this.currentRoomInfo &&
            this.currentRoomInfo.roomId === roomInfo.roomId
          ) {
            this.currentRoomInfo = {
              ...this.currentRoomInfo,
              roomName: roomInfo.roomName,
              joinUserCount: roomInfo.joinUserCount,
              isJoinRoom: roomInfo.isJoinRoom,
              expiredFileStdSeq: roomInfo.expiredFileStdSeq,
              timeRoomInterval: roomInfo.timeRoomInterval
            };
          } else {
            // all refresh
            this.currentRoomInfo = roomInfo;
          }

          // About Interval
          if (!!this.interval) {
            clearInterval(this.interval);
            this.interval = undefined;
          }
          if (!!this.currentRoomInfo && !!this.currentRoomInfo.isTimeRoom) {
            this.interval = setInterval(() => {
              this.store.dispatch(
                ChattingActions.intervalClearEvent({ roomId: this.roomId })
              );
            }, 1000);
          }
        }

        // new room setting popup
        if (
          !!roomInfo &&
          roomInfo.roomId === this.roomId &&
          roomInfo.roomType !== RoomType.Mytalk &&
          (roomInfo.roomType !== RoomType.Single ||
            (roomInfo.roomType === RoomType.Single && !!roomInfo.isTimeRoom)) &&
          !!standbyRooms &&
          standbyRooms.length > 0 &&
          standbyRooms.findIndex((roomId) => roomId === this.roomId) > -1
        ) {
          if (!this.settingDialogRef) {
            this.settingDialogRef = this.dialog.open<
              SettingDialogComponent,
              SettingDialogData,
              SettingDialogResult
            >(SettingDialogComponent, {
              panelClass: 'min-create-dialog',
              data: {
                roomId: this.roomId
              }
            });

            this.settingDialogRef
              .afterClosed()
              .pipe(take(1))
              .subscribe((result: SettingDialogResult) => {
                if (!!result && !!result.choice) {
                  this._standByRoomSetting(result);
                }
              });

            this.changeDetectorRef.markForCheck();
          }
        }
      });
  }

  /** About scrolling */
  keyOf = (item: Info<EventJson>): string | number => {
    return item.seq;
  };

  onMeasured() {
    if (!!this.isInitScrollbottom) {
      this.gotoScrollToBottom();
      this.isInitScrollbottom = false;
    }
  }

  onContentSizeChanged(size: number) {
    if (!!this.psDirectiveRef) {
      this.psDirectiveRef.update();
    }
  }

  onClickScrollTo(i: string) {
    const index = Number(i);
    // const offset = this.vsList.offsetForIndex(index, 'start');
    // this.psDirectiveRef.scrollToY(offset);
    // console.log('offset', offset, this.psDirectiveRef.ps().element.scrollTop);

    this.vsList.scrollToIndex(index, 'center');
  }

  onClickInvalidateCache() {
    if (!!this.measureSize) {
      this.measureSize.invalidateMeasurements();
    }
  }

  gotoScrollToBottom() {
    // recent message reset;
    this.resetRecentMessage();

    // scrolling.
    if (!!this.vsList && !!this.eventList && this.eventList.length > 0) {
      this.vsList.scrollToIndex(this.eventList.length - 1, 'end');
    }
  }

  refreshAndScrollToBottom() {
    // if (!!this.vsList && !!this.isScrollReachBottom) {
    //   this.isInitScrollbottom = false;
    //   this.vsList.checkViewportSize();
    // }
    if (!!this.psDirectiveRef && !!this.isScrollReachBottom) {
      this.psDirectiveRef.update();
      this.psDirectiveRef.scrollToBottom();
    }
  }

  /** Sender Info */
  getSenderInfo(
    senderSeq: number
  ): RoomUserInfoShort | RoomUserInfo | undefined {
    if (!!this.roomUsers && this.roomUsers.length) {
      return this.roomUsers.find(
        (userInfo) => String(senderSeq) === String(userInfo.seq)
      );
    }

    return undefined;
  }

  /** Date Splitter show check */
  getDateSplitter(event: Info<EventJson>): boolean {
    const curIndex = this.eventList.findIndex((v) => v.seq === event.seq);

    if (curIndex === 0) {
      return true;
    } else if (curIndex > 0) {
      if (!this.eventList[curIndex]) {
        return false;
      }
      return !moment(this.eventList[curIndex].sendDate).isSame(
        moment(this.eventList[curIndex - 1].sendDate),
        'day'
      );
    }

    return false;
  }

  /** Unread Count calculation */
  getUnreadCount(message: Info<EventJson>): string | number {
    const unreadCnt = this.roomUsers.filter(
      (user) =>
        user.isJoinRoom &&
        String(message.senderSeq) !== String(user.seq) &&
        user.lastReadEventSeq < message.seq
    ).length;
    return unreadCnt === 0 ? '' : unreadCnt;
  }

  onClickShowPreviousEvents(event: MouseEvent) {
    event.preventDefault();
    event.stopPropagation();

    if (!!this.isRemainEvent) {
      this.store.dispatch(
        ChattingActions.moreEvents({
          roomId: this.roomId
        })
      );
    }
  }

  async onClickMessageContextMenu(params: {
    menuType: string;
    message: Info<EventJson>;
  }) {
    switch (params.menuType) {
      case 'COPY':
        {
          switch (params.message.type) {
            case EventType.Character:
              {
                this.appChatService.copyFromContentText(
                  (params.message as Info<string>).sentMessage
                );
              }
              break;
            case EventType.MassText:
              {
                this.appChatService.massTextDownload({
                  userSeq: String(this.user.info.seq),
                  deviceType: this.loginSession.deviceType,
                  token: this.loginInfo.tokenString,
                  eventMassSeq: params.message.seq
                });
              }
              break;
            case EventType.Translation:
              {
              }
              break;
            case EventType.MassTranslation:
              {
              }
              break;
            default:
              break;
          }
        }
        break;
      case 'RECALL':
        {
          const dialogRef = this.dialog.open<
            ConfirmDialogComponent,
            ConfirmDialogData,
            ConfirmDialogResult
          >(ConfirmDialogComponent, {
            panelClass: 'min-create-dialog',
            data: {
              title: this.i18nService.t('chat:label.recallEvent'),
              html: this.i18nService.t('chat:dialog.confirmRecallEvent')
            }
          });

          dialogRef
            .afterClosed()
            .pipe(take(1))
            .subscribe((result) => {
              if (!!result && !!result.choice) {
                this.store.dispatch(
                  ChattingActions.cancel({
                    roomId: this.currentRoomInfo.roomId,
                    eventSeq: params.message.seq,
                    deviceType: this.loginSession.deviceType
                  })
                );
              }
            });
        }
        break;
      case 'FORWARD':
        {
          const dialogRef = this.dialog.open<
            ForwardDialogComponent,
            ForwardDialogData,
            ForwardDialogResult
          >(ForwardDialogComponent, {
            panelClass: 'max-create-dialog'
          });
          dialogRef
            .afterOpened()
            .pipe(take(1))
            .subscribe(() => {
              dialogRef.componentInstance.psUpdate();
            });

          dialogRef
            .afterClosed()
            .pipe(take(1))
            .subscribe((result) => {
              if (!!result && !!result.choice) {
                const userSeqs: string[] = [];
                let roomSeq = '';
                if (
                  !!result.selelctUserList &&
                  result.selelctUserList.length > 0
                ) {
                  result.selelctUserList.map((user) =>
                    userSeqs.push(String(user.seq))
                  );
                }
                if (!!result.selectRoom) {
                  roomSeq = result.selectRoom.roomId;
                }
                if (userSeqs.length > 0 || roomSeq.trim().length > 0) {
                  this.store.dispatch(
                    ChattingActions.forward({
                      senderSeq: String(this.user.info.seq),
                      deviceType: this.loginSession.deviceType,
                      req: {
                        roomId: '-999',
                        eventType: params.message.type,
                        sentMessage: params.message.sentMessage
                      },
                      trgtUserSeqs: userSeqs,
                      trgtRoomId: roomSeq
                    })
                  );
                }
              }
            });
        }
        break;
      case 'FORWARD_TO_ME':
        {
          if (this.loginInfo.talkWithMeBotSeq > -1) {
            const seqs = this.user.talkWithMeBotSeq as any;
            this.store.dispatch(
              ChattingActions.forward({
                senderSeq: String(this.user.info.seq),
                deviceType: this.loginSession.deviceType,
                req: {
                  roomId: '-999',
                  eventType: params.message.type,
                  sentMessage: params.message.sentMessage
                },
                trgtUserSeqs: [seqs]
              })
            );
          }
        }
        break;
      case 'DELETE':
        {
          const dialogRef = this.dialog.open<
            ConfirmDialogComponent,
            ConfirmDialogData,
            ConfirmDialogResult
          >(ConfirmDialogComponent, {
            panelClass: 'min-create-dialog',
            data: {
              title: this.i18nService.t('chat:dialog.title.removeChat'),
              html: this.i18nService.t('chat:dialog.confirmRemoveChat')
            }
          });

          dialogRef
            .afterClosed()
            .pipe(take(1))
            .subscribe((result) => {
              if (!!result && !!result.choice) {
                this.store.dispatch(
                  ChattingActions.del({
                    roomId: this.currentRoomInfo.roomId,
                    eventSeq: params.message.seq
                  })
                );
              }
            });
        }
        break;
    }
  }

  onMassTranslationDetail(params: {
    message: Info<MassTranslationEventJson>;
    contentsType: string;
  }) {
    const req = {
      userSeq: String(this.user.info.seq),
      token: this.loginInfo.tokenString,
      deviceType: this.loginSession.deviceType,
      eventTransSeq: params.message.sentMessageJson.translationSeq.toString()
    } as TransMassTalkDownloadRequest;
    this.appChatService.massTranslationMessageDetail(
      req,
      params.message,
      params.contentsType,
      this.roomId
    );
  }

  onFileViewer(params: {
    selectFileInfo: SelectFileInfo;
    senderInfo: RoomUserInfoShort | RoomUserInfo;
  }) {
    const self = this;
    this.appOrganizationService
      .getUserInfo({ userSeq: String(params.senderInfo.seq), user: this.user })
      .then((result) => {
        let isValid = false;
        if (
          !!result &&
          !!result.userInfo &&
          !!self.userPermission &&
          self.userPermission.fileTransferAllowedCompanyList.indexOf(
            result.userInfo.companyCode
          ) > -1
        ) {
          isValid = true;
        }

        if (!!isValid) {
          const data: FileViewerDialogData = {
            fileInfos: self.currentFileInfoList,
            selectFileInfo: params.selectFileInfo,
            downloadUrl: self.versionInfo.downloadUrl,
            deviceType: self.loginSession.deviceType,
            token: self.loginInfo.tokenString,
            userSeq: String(self.user.info.seq)
          };
          self.appChatService.openFileviwer(data);
        } else {
          self._openAlert(
            self.i18nService.t('common:file.errors.disapprovalCompany')
          );
        }
      })
      .catch((reason) => {
        self._openAlert(
          self.i18nService.t('common:file.errors.disapprovalCompany')
        );
      });
  }

  onFileSave(params: {
    fileInfo: FileEventJson;
    fileDownloadItem: FileDownloadItem;
    type: string;
    senderInfo: RoomUserInfoShort | RoomUserInfo;
  }): void {
    const self = this;
    this.appOrganizationService
      .getUserInfo({ userSeq: String(params.senderInfo.seq), user: this.user })
      .then((rst) => {
        let isValid = false;
        if (
          !!rst &&
          !!rst.userInfo &&
          !!self.userPermission &&
          self.userPermission.fileTransferAllowedCompanyList.indexOf(
            rst.userInfo.companyCode
          ) > -1
        ) {
          isValid = true;
        }

        if (!!isValid) {
          if (
            params.type === 'saveAs' &&
            self.loginSession.deviceType === DeviceType.PC
          ) {
            self.nativeService
              .file_selectForSave({ defaultPath: params.fileInfo.fileName })
              .then((result) => {
                if (!!result) {
                  if (!!result.canceled) {
                    // 취소함.
                  } else {
                    self.appFileService.saveFile(
                      {
                        fileInfo: params.fileInfo,
                        fileDownloadItem: params.fileDownloadItem,
                        type: params.type,
                        fileName: params.fileInfo.fileName,
                        fileDownloadUrl: undefined,
                        savePath: result.filePath
                      },
                      self.loginInfo,
                      self.user,
                      self.loginSession
                    );
                  }
                }
              })
              .catch((reason) => {
                // self.snackBarService.open(
                //   self.translateService.instant(
                //     'common:file.errors.failToSpecifyPath'
                //   ),
                //   self.translateService.instant('common:file.errors.label')
                // );
              });
          } else {
            self.appFileService.saveFile(
              {
                fileInfo: params.fileInfo,
                fileDownloadItem: params.fileDownloadItem,
                type: params.type,
                fileName: params.fileInfo.fileName,
                fileDownloadUrl: undefined,
                savePath: undefined
              },
              self.loginInfo,
              self.user,
              self.loginSession
            );
          }
        } else {
          self._openAlert(
            self.i18nService.t('common:file.errors.disapprovalCompany')
          );
        }
      })
      .catch((reason) => {
        self._openAlert(
          self.i18nService.t('common:file.errors.disapprovalCompany')
        );
      });
  }

  private _standByRoomSetting(result: SettingDialogResult) {
    if (result.timerInterval > 0 && !!result.roomInfo.isTimeRoom) {
      this.store.dispatch(
        RoomActions.updateTimeRoomInterval({
          req: {
            roomId: result.roomInfo.roomId,
            timerInterval: result.timerInterval
          } as UpdateTimerSetRequest
        })
      );
    } else {
      this.store.dispatch(
        RoomActions.update({
          req: {
            roomId: result.roomInfo.roomId,
            roomName: result.roomName,
            receiveAlarm: result.roomInfo.receiveAlarm,
            syncAll: result.syncAll
          } as UpdateRequest
        })
      );
    }
  }

  onOpenProfile(userSeq: string): void {
    const result = this.dialog.open<
      ProfileDialogComponent,
      ProfileDialogData,
      ProfileDialogResult
    >(ProfileDialogComponent, {
      panelClass: 'mid-create-dialog',
      data: {
        userSeq
      }
    });
  }

  onJoinConference(conferenceSeq: number): void {
    const loginSession = this.appAuthenticationService.getLoginSession();
    const req: ConferenceJoinRequest = {
      userSeq: String(this.user.info.seq),
      deviceType: loginSession.deviceType,
      tokenKey: this.loginInfo.tokenString,
      conferenceSeq
    };
    this.appChatService.joinVideoConference(req);
  }

  private _openAlert(msg: string) {
    this.dialog.open<AlertDialogComponent, AlertDialogData, AlertDialogResult>(
      AlertDialogComponent,
      {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('common:file.errors.title'),
          message: msg
        }
      }
    );
  }

  resetRecentMessage() {
    this.recentMessage = undefined;
    this.recentUserInfo = undefined;
  }
}
