import { Subject, combineLatest } from 'rxjs';
import { takeUntil, debounceTime } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  Output,
  EventEmitter,
  ChangeDetectorRef,
  ChangeDetectionStrategy,
  ViewChild,
  ElementRef
} from '@angular/core';
import { FormGroup, FormBuilder } from '@angular/forms';

import { Store, select } from '@ngrx/store';

import { MatAutocompleteTrigger } from '@angular/material/autocomplete';

import { User } from '@ucap/domain-organization';

import { LogService } from '@ucap/ng-logger';

import { RoomSelector } from '@ucap/ng-store-chat';
import { UserSelector } from '@ucap/ng-store-organization';

@Component({
  selector: 'app-sections-chat-search',
  templateUrl: './search.section.component.html',
  styleUrls: ['./search.section.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SearchSectionComponent implements OnInit, OnDestroy {
  @Output()
  keyDownEnter = new EventEmitter<{
    searchWord: string;
  }>();

  @Output()
  searchCancel = new EventEmitter<any>();

  user: User;

  fgSearch: FormGroup;
  recommendedWordList: string[];
  filteredRecommendedWordList: string[];

  @ViewChild('searchWordInput', { static: true })
  searchWordInput: ElementRef<HTMLInputElement>;
  @ViewChild(MatAutocompleteTrigger) autocomplete: MatAutocompleteTrigger;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private formBuilder: FormBuilder,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {}

  ngOnInit(): void {
    this.fgSearch = this.formBuilder.group({
      searchInput: null
    });

    this.fgSearch
      .get('searchInput')
      .valueChanges.pipe(debounceTime(100))
      .subscribe((value) => {
        if (value !== null && value.length > 0) {
          this.filteredRecommendedWordList = this.recommendedWordList.filter(
            (v) => {
              return v.includes(value);
            }
          );
        } else {
          this.filteredRecommendedWordList = [];
        }

        this.changeDetectorRef.markForCheck();
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });

    combineLatest([
      this.store.pipe(select(RoomSelector.rooms)),
      this.store.pipe(select(RoomSelector.roomUsers)),
      this.store.pipe(select(RoomSelector.roomUsersShort)),
      this.store.pipe(select(RoomSelector.standbyRooms))
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([rooms, roomUsers, roomUsersShort, standbyRooms]) => {
        rooms = (rooms || []).filter((info) => {
          return (
            info.isJoinRoom &&
            !standbyRooms.find((standbyRoom) => standbyRoom === info.roomId)
          );
        });
        roomUsers = (roomUsers || []).filter(
          (userMap) =>
            rooms.findIndex((roomInfo) => roomInfo.roomId === userMap.roomId) >
            -1
        );
        roomUsersShort = (roomUsersShort || []).filter(
          (userMap) =>
            rooms.findIndex((roomInfo) => roomInfo.roomId === userMap.roomId) >
            -1
        );

        const recommendedWordList = [];
        for (const r of rooms) {
          if (!!r.roomName && '' !== r.roomName.trim()) {
            recommendedWordList.push(r.roomName);
          }
        }
        for (const ru of roomUsers) {
          for (const u of ru.userInfos) {
            if (!!this.user && String(u.seq) !== String(this.user.info.seq)) {
              if (!!u.name && '' !== u.name.trim() && u.isJoinRoom) {
                recommendedWordList.push(u.name);
              }
            }
          }
        }
        for (const ru of roomUsersShort) {
          for (const u of ru.userInfos) {
            if (!!this.user && String(u.seq) !== String(this.user.info.seq)) {
              if (!!u.name && '' !== u.name.trim() && u.isJoinRoom) {
                recommendedWordList.push(u.name);
              }
            }
          }
        }
        this.recommendedWordList = [
          ...recommendedWordList.filter(
            (item, index) => recommendedWordList.indexOf(item) === index
          )
        ];
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onKeyupBackspace(event: Event) {
    if (this.searchWordInput.nativeElement.value === '') {
      this.searchCancel.emit();
    }
    event.stopPropagation();
  }

  onKeyDownEnter(event: KeyboardEvent, searchWord: string) {
    event.preventDefault();
    event.stopPropagation();

    if (!!this.autocomplete) {
      this.autocomplete.closePanel();
    }

    this.keyDownEnter.emit({ searchWord });
  }
  onClickCancel() {
    this.searchCancel.emit();
  }
}
