import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject
} from '@angular/core';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { UserInfo, UserInfoSS } from '@ucap/domain-organization';
import { GroupInfoDetail } from '@ucap/domain-group';

import { MatStepper } from '@angular/material/stepper';
import { I18nService } from '@ucap/ng-i18n';
import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui/core';
import { environment } from '@environments';
import { Store, select } from '@ngrx/store';
import { takeUntil } from 'rxjs/operators';
import { AuthorizationSelector } from '@ucap/ng-store-authentication';
import { AppGroupService } from '@app/services/app-group.service';
import { UserInfoTypes } from '@app/types';
import { AppChatService } from '@app/services/app-chat.service';
import { UserPermission } from '@ucap/domain-authorization';

export interface CreateDialogData {}
export interface CreateDialogResult {
  userSeqs: string[];
  isTimer: boolean | undefined;
}

@Component({
  selector: 'app-dialog-chat-create',
  templateUrl: './create.dialog.component.html',
  styleUrls: ['./create.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class CreateDialogComponent implements OnInit, OnDestroy {
  currentStep = 0;
  maxChatRoomUser: number;

  isTimer: boolean | undefined;
  selectedUserList: UserInfoTypes[] = [];
  userPermission: UserPermission;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    public dialogRef: MatDialogRef<CreateDialogData, CreateDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: CreateDialogData,
    private i18nService: I18nService,
    public dialog: MatDialog,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private appGroupService: AppGroupService,
    private appChatService: AppChatService
  ) {
    this.maxChatRoomUser = environment.productConfig.chat.maxChatRoomUser;
  }

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(AuthorizationSelector.userPermission)
      )
      .subscribe((userPermission) => {
        this.userPermission = userPermission;
        if (!!this.userPermission && !this.userPermission.canTimerChat) {
          this.isTimer = false;
          this.currentStep = 1;
        }
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close();
  }
  onCancel(stepper: MatStepper) {
    if (!!this.userPermission && !this.userPermission.canTimerChat) {
      // close.
    } else if (stepper.selectedIndex > 0) {
      stepper.previous();
      return;
    }
    this.dialogRef.close();
  }
  onConfirm(stepper: MatStepper) {
    // validation.
    if (this.isTimer === undefined) {
      this.dialog.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('chat:errors.label'),
          html: this.i18nService.t('chat:errors.emptyOpenRoomType')
        }
      });
      return;
    }
    stepper.next();
  }

  onOpenRoom(stepper: MatStepper) {
    const userSeqs: string[] = [];

    this.selectedUserList.map((user) => userSeqs.push(user.seq.toString()));

    // validation and openRoom
    this.appChatService.newOpenRoom(userSeqs, this.isTimer);

    if (this.selectedUserList.length >= this.maxChatRoomUser) {
      // not close popup..
      return;
    } else {
      this.dialogRef.close({ userSeqs, isTimer: this.isTimer });
    }
  }

  onToggleCheckUser(datas: { checked: boolean; userInfo: UserInfoSS }[]) {
    const resList = this.appGroupService.getToggleCheckUser(
      datas,
      this.selectedUserList
    );
    if (!resList) {
      return;
    }
    this.selectedUserList = resList;
  }
  onToggleCheckGroup(params: {
    isChecked: boolean;
    groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfo[] };
  }) {
    const resList = this.appGroupService.getToggleCheckGroup(
      params,
      this.selectedUserList
    );

    if (!resList) {
      return;
    }

    this.selectedUserList = resList;
  }
  onRemovedProfileSelection(userInfo: UserInfo) {
    const resUserList = this.appGroupService.removedProfileSelection(
      userInfo,
      this.selectedUserList
    );

    if (!resUserList) {
    }
    this.selectedUserList = resUserList;
  }
  removableForSelection = (userInfo: UserInfo) => {
    return true;
  };
  colorForSelection = (userInfo: UserInfo) => {
    return 'accent';
  };
}
