import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject,
  ViewChild
} from '@angular/core';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { Store, select } from '@ngrx/store';

import {
  UserInfo,
  UserInfoSS,
  UserInfoF,
  UserInfoDN
} from '@ucap/domain-organization';
import { UserInfo as RoomUserInfo, RoomInfo } from '@ucap/domain-chat';

import { I18nService } from '@ucap/ng-i18n';

import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import { GroupSelector } from '@ucap/ng-store-group';

import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui/core';

import { SearchData } from '@app/ucap/organization/models/search-data';
import { Expansion02Component as AppExpansion02Component } from '@app/ucap/group/components/expansion-02.component';
import { ProfileListComponent as AppProfileListComponent } from '@app/ucap/group/components/profile-list.component';
import { environment } from '@environments';
import { AppGroupService } from '@app/services/app-group.service';
import { UserInfoTypes } from '@app/types';
import { VersionInfo } from '@ucap/domain-authentication';
import { GroupInfoDetail } from '@ucap/domain-group';

export interface ForwardDialogData {}
export interface ForwardDialogResult {
  choice: boolean;
  selelctUserList?: UserInfoTypes[];
  selectRoom?: RoomInfo;
}

@Component({
  selector: 'app-dialog-chat-forward',
  templateUrl: './forward.dialog.component.html',
  styleUrls: ['./forward.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ForwardDialogComponent implements OnInit, OnDestroy {
  @ViewChild('appGroupExpansion', { static: false })
  appGroupExpansion: AppExpansion02Component;

  @ViewChild('appProfileList', { static: false })
  appProfileList: AppProfileListComponent;

  set companySearchData(searchData: SearchData) {
    if (!!searchData && searchData.searchWord !== '') {
      this._companySearchData = { ...searchData, bySearch: true };
    } else {
      this._companySearchData = { ...searchData, bySearch: false };
    }

    this.onChangedCompanySearch();
  }
  get companySearchData() {
    return this._companySearchData;
  }
  // tslint:disable-next-line: variable-name
  _companySearchData: SearchData;

  private ngOnDestroySubject: Subject<void> = new Subject();

  maxChatRoomUser: number;

  selectedRoom: RoomInfo;
  selectedUserList: UserInfoTypes[] = [];

  versionInfo: VersionInfo;

  groupList: GroupInfoDetail[];
  isBackspaceCanceled = true;
  isSearch = false;
  searchedList: UserInfoSS[] = [];

  set currentTabIndex(idx: number) {
    this._currentTabIndex = idx;
    // this._resetSelectedObject();
  }

  get currentTabIndex() {
    return this._currentTabIndex;
  }
  // tslint:disable-next-line: variable-name
  _currentTabIndex: number;

  constructor(
    public dialogRef: MatDialogRef<ForwardDialogData, ForwardDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: ForwardDialogData,
    private store: Store<any>,
    private i18nService: I18nService,
    public dialog: MatDialog,
    private changeDetectorRef: ChangeDetectorRef,
    private appGroupService: AppGroupService
  ) {
    this.maxChatRoomUser = environment.productConfig.chat.maxChatRoomUser;
  }

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(GroupSelector.groups))
      .subscribe((groups) => {
        this.groupList = groups;
      });
    this.currentTabIndex = 0;
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  psUpdate() {
    this.appGroupExpansion.psUpdate();
  }
  onClosed(event: MouseEvent): void {
    this.dialogRef.close({ choice: false });
  }

  onSelectedIndexChange(value: number): void {
    this.currentTabIndex = value;
    this._resetSelectedObject();
  }

  private _resetSelectedObject(): void {
    this.selectedRoom = undefined;
    this.selectedUserList = [];
  }

  onSearched(searchedUserInfos: UserInfoSS[]): void {
    this.searchedList = searchedUserInfos;

    if (!!this.appProfileList) {
      this.appProfileList.psUpdate();
    }
  }

  onToggleCheckUser(datas: { checked: boolean; userInfo: UserInfoSS }[]) {
    const resList = this.appGroupService.getToggleCheckUser(
      datas,
      this.selectedUserList
    );

    if (!resList) {
      return;
    }
    this.selectedUserList = resList;
  }

  onToggleCheckGroup(params: {
    isChecked: boolean;
    groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfo[] };
  }) {
    const resList = this.appGroupService.getToggleCheckGroup(
      params,
      this.selectedUserList
    );

    if (!resList) {
      return;
    }
    this.selectedUserList = resList;
  }

  onCancel() {
    this.dialogRef.close({ choice: false });
  }
  onConfirm() {
    if (this.currentTabIndex === 0) {
      this.selectedRoom = undefined;
    } else {
      this.selectedUserList = [];
    }

    if (
      !this.selectedRoom &&
      !!this.selectedUserList &&
      this.selectedUserList.length === 0
    ) {
      const result = this.dialog.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('chat:errors.label'),
          message: this.i18nService.t('chat:errors.inputChatMessage')
        }
      });
      return;
    }

    this.dialogRef.close({
      choice: true,
      selectRoom: this.selectedRoom,
      selelctUserList: this.selectedUserList
    });
  }

  onToggleRoom(roomInfo: RoomInfo): void {
    if (!!roomInfo) {
      this.selectedRoom = roomInfo;
    } else {
      this.selectedRoom = undefined;
    }
  }

  onChangedCompanySearch() {
    if (!!this.companySearchData && this.companySearchData.bySearch) {
      this.isSearch = true;
    }
  }

  getCheckedSearchAllItem(): boolean {
    const targetRoomList = this.searchedList;

    if (
      !targetRoomList ||
      targetRoomList.length === 0 ||
      targetRoomList.filter(
        (item) =>
          !(
            this.selectedUserList.filter(
              (info) => String(info.seq) === String(item.seq)
            ).length > 0
          )
      ).length > 0
    ) {
      return false;
    } else {
      return true;
    }
  }

  onToggleSearchAllItem(value: boolean): void {
    const pushs: UserInfoSS[] = [];
    const pops: UserInfoSS[] = [];

    this.searchedList.forEach((user) => {
      const i = this.selectedUserList.findIndex((u) => u.seq === user.seq);

      if (-1 === i) {
        pushs.push(user);
      }

      if (-1 < i) {
        pops.push(user);
      }
    });

    if (!!value) {
      if (0 < pushs.length) {
        this.selectedUserList = [...this.selectedUserList, ...pushs];
      }
    } else {
      if (0 < pops.length) {
        this.selectedUserList = this.selectedUserList.filter(
          (u) => -1 === pops.findIndex((p) => p.seq === u.seq)
        );
      }
    }
    this.changeDetectorRef.markForCheck();
  }

  onCanceled() {
    this.isSearch = false;

    this.companySearchData = { ...this.companySearchData, searchWord: '' };
  }

  checkDisableBtn(): boolean {
    if (
      this.currentTabIndex === 0 &&
      !!this.selectedUserList &&
      this.selectedUserList.length > 0
    ) {
      return false;
    } else if (this.currentTabIndex === 1 && !!this.selectedRoom) {
      return false;
    }
    return true;
  }
  onRemovedProfileSelection(userInfo: UserInfo) {
    const resUserList = this.appGroupService.removedProfileSelection(
      userInfo,
      this.selectedUserList
    );

    if (!resUserList) {
    }
    this.selectedUserList = resUserList;
  }
  removableForSelection = (userInfo: UserInfo) => {
    return true;
  };
  colorForSelection = (userInfo: UserInfo) => {
    return 'accent';
  };
}
