import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog,
  MatDialogConfig
} from '@angular/material/dialog';

import { RoomInfo } from '@ucap/domain-chat';

import { I18nService } from '@ucap/ng-i18n';
import { RoomSelector } from '@ucap/ng-store-chat';

export interface SettingDialogData {
  roomId: string;
  rect?: any;
}
export interface SettingDialogResult {
  choice: boolean;
  roomInfo?: RoomInfo;
  roomName?: string;
  syncAll?: boolean;
  timerInterval?: number;
}

@Component({
  selector: 'app-dialog-chat-setting',
  templateUrl: './setting.dialog.component.html',
  styleUrls: ['./setting.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SettingDialogComponent implements OnInit, OnDestroy {
  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    public dialogRef: MatDialogRef<SettingDialogData, SettingDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: SettingDialogData,
    private store: Store<any>,
    private i18nService: I18nService,
    private dialog: MatDialog,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  currentRoomInfo: RoomInfo;
  currentParams: {
    invalid: boolean;
    roomName: string;
    checkedMe: string;
    timerInterval: number;
  };

  ngOnInit(): void {
    if (!!this.data.rect) {
      const matDialogConfig: MatDialogConfig = new MatDialogConfig();
      matDialogConfig.position = {
        left: `${this.data.rect.left}px`,
        top: '90px'
      };

      this.dialogRef.updatePosition(matDialogConfig.position);
    }

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(RoomSelector.room, this.data.roomId)
      )
      .subscribe((room) => {
        if (!!room) {
          this.currentRoomInfo = room;
          this.changeDetectorRef.markForCheck();
        }
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close({ choice: false });
  }

  onChangedRoomSetting(param: {
    invalid: boolean;
    roomName: string;
    checkedMe: string;
    timerInterval: number;
  }) {
    this.currentParams = param;
  }

  onCancel() {
    this.dialogRef.close({ choice: false });
  }
  onConfirm() {
    if (
      !!this.currentRoomInfo &&
      !!this.currentRoomInfo.isTimeRoom &&
      this.currentParams.timerInterval > 0
    ) {
      this.dialogRef.close({
        choice: true,
        roomInfo: this.currentRoomInfo,
        timerInterval: this.currentParams.timerInterval
      });
    } else if (this.currentParams.invalid) {
      console.debug(
        '유효하지 않은 이름: ',
        this.currentParams.invalid,
        this.currentParams.roomName
      );
    } else {
      this.dialogRef.close({
        choice: true,
        roomInfo: this.currentRoomInfo,
        roomName: this.currentParams.roomName,
        syncAll:
          !!this.currentParams && this.currentParams.checkedMe === 'all'
            ? true
            : false
      });
    }
  }
}
