import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject,
  ElementRef,
  ViewChild,
  AfterViewInit
} from '@angular/core';

import { Subject, of, combineLatest } from 'rxjs';
import { take, map, catchError, takeUntil } from 'rxjs/operators';
import { select, Store } from '@ngrx/store';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { DeviceType } from '@ucap/domain-common';
import {
  UserInfo as RoomUserInfo,
  UserInfoShort as RoomUserInfoShort,
  Info,
  EventJson
} from '@ucap/domain-chat';

import { StatusCode } from '@ucap/api';
import { NativeService } from '@ucap/native';

import { MassTalkDownloadRequest } from '@ucap/api-common';

import { CommonApiService } from '@ucap/ng-api-common';
import { LogService } from '@ucap/ng-logger';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { RoomSelector } from '@ucap/ng-store-chat';
import { ConfigurationSelector } from '@ucap/ng-store-authentication';

import { AppChatService } from '@app/services/app-chat.service';
import { PerfectScrollbarComponent } from 'ngx-perfect-scrollbar';
import { VersionInfo } from '@ucap/domain-authentication';

export type UserInfoTypes = RoomUserInfo | RoomUserInfoShort;

export interface TextDetailDialogData {
  message: Info<EventJson>;
  roomId: string;
  userSeq: string;
  deviceType: DeviceType;
  token: string;
  eventMassSeq?: number;
}
export interface TextDetailDialogResult {}

@Component({
  selector: 'app-dialog-chat-text-detail',
  templateUrl: './text-detail.dialog.component.html',
  styleUrls: ['./text-detail.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class TextDetailDialogComponent
  implements OnInit, OnDestroy, AfterViewInit {
  userInfo: UserInfoTypes;
  contents = '';

  defaultProfileImage: string;
  versionInfo: VersionInfo;

  @ViewChild('perfectScroll') perfectScroll: PerfectScrollbarComponent;
  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    public dialogRef: MatDialogRef<
      TextDetailDialogData,
      TextDetailDialogResult
    >,
    @Inject(MAT_DIALOG_DATA) public data: TextDetailDialogData,
    public dialog: MatDialog,
    private store: Store<any>,
    private elementRef: ElementRef,
    private commonApiService: CommonApiService,
    private appChatService: AppChatService,
    private logService: LogService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit(): void {
    this.defaultProfileImage = this.appChatService.defaultProfileImage;
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });

    this.commonApiService
      .massTalkDownload({
        userSeq: this.data.userSeq,
        deviceType: this.data.deviceType,
        eventMassSeq: this.data.eventMassSeq,
        token: this.data.token
      } as MassTalkDownloadRequest)
      .pipe(
        take(1),
        map((res) => {
          if (res.statusCode === StatusCode.Success) {
            this.contents = res.content;

            this.changeDetectorRef.markForCheck();

            setTimeout(() => {
              this.psUpdate();
              if (
                !!this.elementRef.nativeElement &&
                !!this.elementRef.nativeElement.querySelector('a')
              ) {
                const elements = this.elementRef.nativeElement.querySelectorAll(
                  'a'
                );
                elements.forEach((element) => {
                  element.addEventListener(
                    'click',
                    this.onClickEvent.bind(this)
                  );
                });
              }
            }, 500);
          } else {
            this.logService.error(
              `commonApiService] massTalkDownload ${res?.errorMessage}`
            );
          }
        }),
        catchError((error) => of({ error }))
      )
      .subscribe();

    combineLatest([
      this.store.pipe(select(RoomSelector.roomUser, this.data.roomId)),
      this.store.pipe(select(RoomSelector.roomUserShort, this.data.roomId))
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([roomUsers, roomUsersShort]) => {
        const userList: UserInfoTypes[] =
          roomUsers?.userInfos || roomUsersShort?.userInfos;
        this.userInfo = userList.find(
          (item) => item.seq + '' === this.data.message.senderSeq + ''
        );
        this.changeDetectorRef.markForCheck();
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  ngAfterViewInit(): void {
    this.psUpdate();
  }

  psUpdate() {
    if (!!this.perfectScroll) {
      this.perfectScroll.directiveRef.update();
    }
  }

  onClickEvent(event: MouseEvent) {
    this.nativeService.platform_openDefaultBrowser(
      (event.target as HTMLAnchorElement).text
    );
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close();
  }
}
