import { Subject, fromEvent } from 'rxjs';

import {
  Directive,
  ElementRef,
  EventEmitter,
  HostListener,
  Output,
  Input,
  AfterViewInit,
  NgZone,
  OnDestroy,
  OnInit
} from '@angular/core';

import { FileUploadItem } from '@ucap/domain-common';

import { FileUploadQueueComponent } from '@ucap/ng-ui/file-upload';
import { takeUntil } from 'rxjs/operators';

@Directive({
  selector: 'input[ucapFileUploadFor01], div[ucapFileUploadFor01]'
})
export class FileUploadForDirective
  implements OnInit, OnDestroy, AfterViewInit {
  @Input()
  fileUploadQueue?: FileUploadQueueComponent;

  @Input()
  validator?: (fileList: FileList) => Promise<boolean>;

  @Output()
  public fileDragEnter = new EventEmitter<DataTransferItemList>();

  @Output()
  public fileDragOver = new EventEmitter<void>();

  @Output()
  public fileDragLeave = new EventEmitter<void>();

  @Output()
  public fileSelected = new EventEmitter<FileUploadItem[] | FileList>();

  dragOver = false;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private elementRef: ElementRef,
    private readonly ngZone: NgZone
  ) {}

  ngOnInit(): void {
    fromEvent(window, 'dragenter')
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((event: DragEvent) => {
        if (!this.isFileDrag(event.dataTransfer)) {
          return;
        }

        if (!this.dragOver) {
          this.fileDragEnter.emit(event.dataTransfer.items);
          this.dragOver = true;
          if (!!this.fileUploadQueue) {
            this.fileUploadQueue.onDragEnter(event.dataTransfer.items);
          }
        }
      });

    fromEvent(window, 'dragover')
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((event: DragEvent) => {
        if (!this.isFileDrag(event.dataTransfer)) {
          return;
        }
        // if (this.fileUploadQueue.isEventInElement(event)) {
        //   event.dataTransfer.dropEffect = 'copy';
        // } else {
        //   event.dataTransfer.dropEffect = 'none';
        // }
        event.preventDefault();
      });

    fromEvent(window, 'dragleave')
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((event: DragEvent) => {
        if (!this.isFileDrag(event.dataTransfer)) {
          return;
        }

        if (event && event.pageX === 0 && event.pageY === 0) {
          this.fileDragLeave.emit();
          this.dragOver = false;
          if (!!this.fileUploadQueue) {
            this.fileUploadQueue.onDragLeave();
          }
        }
      });

    fromEvent(window, 'drop')
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((event: any) => {
        if (!this.isFileDrag(event.dataTransfer)) {
          return;
        }

        const self = this;
        const files: FileList = event.dataTransfer.files;

        event.preventDefault();
        event.stopPropagation();
        this.elementRef.nativeElement.value = ''; // Case defined directive in 'input' type.
        this.dragOver = false;

        if (!!this.validator) {
          this.validator(files)
            .then(async (result) => {
              if (!result) {
                if (!!this.fileUploadQueue) {
                  this.fileUploadQueue.onUploadComplete();
                }
                return;
              } else {
                const fileUploadItems = FileUploadItem.fromFiles(files);
                self.fileSelected.emit(fileUploadItems);

                if (!!self.fileUploadQueue) {
                  self.fileUploadQueue.onDrop(fileUploadItems);
                }
              }
            })
            .catch((err) => {
              if (!!this.fileUploadQueue) {
                this.fileUploadQueue.onUploadComplete();
              }
            });
        } else {
          const fileUploadItems = FileUploadItem.fromFiles(files);
          this.fileSelected.emit(files);

          if (!!this.fileUploadQueue) {
            this.fileUploadQueue.onDrop(fileUploadItems);
          }
        }
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  ngAfterViewInit(): void {}

  // @HostListener('change')
  // public onChange(): any {
  //   const files = this.elementRef.nativeElement.files;
  //   this.fileSelected.emit(FileUploadItem.fromFiles(files));
  //   this.elementRef.nativeElement.value = '';
  // }

  private isFileDrag(dataTransfer: DataTransfer): boolean {
    if (0 >= dataTransfer.items.length) {
      return false;
    }

    // tslint:disable-next-line: prefer-for-of
    for (let i = 0; i < dataTransfer.items.length; i++) {
      const element = dataTransfer.items[i];
      if ('file' !== element.kind) {
        return false;
      }
    }

    return true;
  }
}
