import { Subject, combineLatest, merge } from 'rxjs';
import { takeUntil, take } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  EventEmitter,
  Output,
  Input
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { MatDialog } from '@angular/material/dialog';

import { User } from '@ucap/domain-organization';
import {
  UserInfo as RoomUserInfo,
  UserInfoShort as RoomUserInfoShort
} from '@ucap/domain-chat';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';

import { UserSelector } from '@ucap/ng-store-organization';
import { GroupInfoDetail } from '@ucap/domain-group';
import { RoomSelector } from '@ucap/ng-store-chat';

import {
  ConfirmDialogComponent,
  ConfirmDialogData,
  ConfirmDialogResult,
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui/core';

import { ChatDrawType } from '@app/pages/chat/types/chat-draw.type';
import { DrawInfo } from '@app/pages/chat/models/draw-info';
import { AppGroupService } from '@app/services/app-group.service';

export type UserInfoTypes = RoomUserInfo | RoomUserInfoShort;

@Component({
  selector: 'app-drawer-chat-add-group',
  templateUrl: './add-group.drawer.component.html',
  styleUrls: ['./add-group.drawer.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class AddGroupDrawerComponent implements OnInit, OnDestroy {
  private roomIdSubject = new Subject<string>();
  private ngOnDestroySubject = new Subject<boolean>();

  @Input()
  set roomId(roomId: string) {
    if (this._roomId !== roomId) {
      this._roomId = roomId;
      this.roomIdSubject.next(roomId);
      this._initializeData();
    }
  }
  get roomId(): string {
    return this._roomId;
  }
  // tslint:disable-next-line: variable-name
  _roomId: string;

  @Input()
  returnChatDrawerType?: ChatDrawType;

  @Output()
  rightDrawerToggle = new EventEmitter<DrawInfo>();

  @Output()
  closed = new EventEmitter<void>();

  user: User;
  roomUsers: RoomUserInfo[] = [];
  roomUsersShort: RoomUserInfoShort[] = [];
  isAddBtnInvalid = true;

  groupName = '';
  selectedGroupList: GroupInfoDetail[] = [];

  constructor(
    private i18nService: I18nService,
    public dialog: MatDialog,
    private store: Store<any>,
    private logService: LogService,
    private changeDetectorRef: ChangeDetectorRef,
    private appGroupService: AppGroupService
  ) {}

  ngOnInit(): void {
    this._initializeData();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }

    if (!!this.roomIdSubject) {
      this.roomIdSubject.next();
      this.roomIdSubject.complete();
    }
  }

  private _initializeData() {
    combineLatest([
      this.store.pipe(select(UserSelector.user)),
      this.store.pipe(select(RoomSelector.roomUser, this.roomId)),
      this.store.pipe(select(RoomSelector.roomUserShort, this.roomId))
    ])
      .pipe(takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)))
      .subscribe(([user, roomUser, roomUserShort]) => {
        this.user = user;
        if (
          !!roomUser &&
          !!roomUser.userInfos &&
          roomUser.userInfos.length > 0
        ) {
          this.roomUsers = roomUser.userInfos.filter(
            (item) =>
              !!item.isJoinRoom && String(item.seq) !== String(user.info.seq)
          );
        }
        if (
          !!roomUserShort &&
          !!roomUserShort.userInfos &&
          roomUserShort.userInfos.length > 0
        ) {
          this.roomUsersShort = roomUserShort.userInfos.filter(
            (item) =>
              !!item.isJoinRoom && String(item.seq) !== String(user.info.seq)
          );
        }
      });
  }

  onChangeSelectedGroupList(data: {
    checked: boolean;
    group: GroupInfoDetail;
  }) {
    const i = this.selectedGroupList.findIndex((u) => u.seq === data.group.seq);

    if (data.checked) {
      if (-1 === i) {
        this.selectedGroupList = [...this.selectedGroupList, data.group];
      }
    } else {
      if (-1 < i) {
        this.selectedGroupList = this.selectedGroupList.filter(
          (u) => u.seq !== data.group.seq
        );
      }
    }
  }

  onChangeGroupName(data: { invalid: boolean; groupName: string }) {
    this.isAddBtnInvalid = data.invalid;
    this.groupName = data.groupName;
  }

  onClosed(event: MouseEvent): void {
    this.closed.emit();
  }

  onClickCancel(): void {
    if (!!this.returnChatDrawerType) {
      this.rightDrawerToggle.emit({
        chatDrawType: this.returnChatDrawerType
      });
    } else {
      this.closed.emit();
    }
  }

  onClickAddGroupMember(): void {
    const targetUserInfos: UserInfoTypes[] = [];

    if (!!this.roomUsers && this.roomUsers.length > 0) {
      this.roomUsers.map((item) => targetUserInfos.push(item));
    } else if (!!this.roomUsersShort && this.roomUsersShort.length > 0) {
      this.roomUsersShort.map((item) => targetUserInfos.push(item));
    } else {
      this.logService.error('chat', 'not exist selected users.');
      return;
    }

    // validations
    if (
      (!this.groupName && !this.selectedGroupList) ||
      (this.groupName.trim().length === 0 &&
        this.selectedGroupList.length === 0)
    ) {
      const result = this.dialog.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('chat:label.addGroup'),
          message: this.i18nService.t('chat:errors.addBuddyForGroup')
        }
      });
      return;
    }

    // Do Action.
    if (!!this.groupName && this.groupName.trim().length > 0) {
      // create group and update room.
      const dialogRef = this.dialog.open<
        ConfirmDialogComponent,
        ConfirmDialogData,
        ConfirmDialogResult
      >(ConfirmDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('chat:label.addGroup'),
          html: this.i18nService.t('chat:dialog.confirmAddBuddyForNewGroup', {
            targetGroups: this.groupName
          })
        }
      });
      dialogRef
        .afterClosed()
        .pipe(take(1))
        .subscribe((result) => {
          if (!!result && result.choice) {
            this.appGroupService.createGroup(this.groupName, targetUserInfos);
            this.closed.emit();
          }
        });
    } else if (!!this.selectedGroupList && this.selectedGroupList.length > 0) {
      // existed group update room.
      const dialogRef = this.dialog.open<
        ConfirmDialogComponent,
        ConfirmDialogData,
        ConfirmDialogResult
      >(ConfirmDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('chat:label.addGroup'),
          html: this.i18nService.t('chat:dialog.confirmAddBuddyForGroup', {
            targetGroups: this.selectedGroupList
              .map((group) => group.name)
              .join(',')
          })
        }
      });
      dialogRef
        .afterClosed()
        .pipe(take(1))
        .subscribe((result) => {
          if (!!result && result.choice) {
            this.selectedGroupList.forEach((group) => {
              this.appGroupService.addMemberToGroup(group, targetUserInfos);
              this.closed.emit();
            });
          }
        });
    }
  }
}
