import { Subject, combineLatest, merge } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  EventEmitter,
  Output,
  Input
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { MatDialog } from '@angular/material/dialog';

import { LocaleCode } from '@ucap/domain-common';
import {
  User,
  UserInfo,
  UserInfoSS,
  UserInfoF,
  UserInfoDN
} from '@ucap/domain-organization';
import { GroupInfoDetail } from '@ucap/domain-group';
import {
  UserInfo as RoomUserInfo,
  UserInfoShort as RoomUserInfoShort,
  RoomInfo,
  RoomType
} from '@ucap/domain-chat';

import { InviteRequest } from '@ucap/protocol-room';

import { I18nService } from '@ucap/ng-i18n';

import { UserSelector } from '@ucap/ng-store-organization';
import { RoomSelector, RoomActions } from '@ucap/ng-store-chat';

import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui/core';

import { environment } from '@environments';
import { AppChatService } from '@app/services/app-chat.service';
import { ChatDrawType } from '@app/pages/chat/types/chat-draw.type';
import { DrawInfo } from '@app/pages/chat/models/draw-info';
import { AppGroupService } from '@app/services/app-group.service';
import { UserInfoTypes } from '@app/types';

@Component({
  selector: 'app-drawer-chat-add-users',
  templateUrl: './add-users.drawer.component.html',
  styleUrls: ['./add-users.drawer.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class AddUsersDrawerComponent implements OnInit, OnDestroy {
  private roomIdSubject = new Subject<string>();
  private ngOnDestroySubject = new Subject<boolean>();

  @Input()
  set roomId(roomId: string) {
    if (this._roomId !== roomId) {
      this._roomId = roomId;
      this.roomIdSubject.next(roomId);
      this._initializeData();
    }
  }
  get roomId(): string {
    return this._roomId;
  }
  // tslint:disable-next-line: variable-name
  _roomId: string;

  @Input()
  returnChatDrawerType?: ChatDrawType;

  @Output()
  rightDrawerToggle = new EventEmitter<DrawInfo>();

  @Output()
  closed = new EventEmitter<void>();

  maxChatRoomUser: number;

  isTimer: boolean | undefined;
  selectedUserList: UserInfoTypes[] = [];

  roomInfo: RoomInfo;
  fixedUserList: (RoomUserInfo | RoomUserInfoShort)[] = [];

  user: User;

  constructor(
    private i18nService: I18nService,
    public dialog: MatDialog,
    private store: Store<any>,
    private appChatService: AppChatService,
    private changeDetectorRef: ChangeDetectorRef,
    private appGroupService: AppGroupService
  ) {
    this.maxChatRoomUser = environment.productConfig.chat.maxChatRoomUser;
  }

  ngOnInit(): void {
    this._initializeData();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }

    if (!!this.roomIdSubject) {
      this.roomIdSubject.next();
      this.roomIdSubject.complete();
    }
  }

  private _initializeData() {
    combineLatest([
      this.store.pipe(select(UserSelector.user)),
      this.store.pipe(select(RoomSelector.room, this.roomId)),
      this.store.pipe(select(RoomSelector.roomUser, this.roomId)),
      this.store.pipe(select(RoomSelector.roomUserShort, this.roomId))
    ])
      .pipe(takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)))
      .subscribe(([user, roomInfo, roomUser, roomUserShort]) => {
        this.user = user;
        this.roomInfo = roomInfo;

        this.fixedUserList = roomUserShort?.userInfos || roomUser?.userInfos;
        if (!!this.fixedUserList) {
          this.fixedUserList = this.fixedUserList.filter(
            (info) =>
              !!info.isJoinRoom && String(info.seq) !== String(user.info.seq)
          );
        }
        this.selectedUserList = this.fixedUserList.slice();

        this.changeDetectorRef.markForCheck();
      });
  }

  onClosed(event: MouseEvent): void {
    this.closed.emit();
  }
  onClickCancel(): void {
    if (!!this.returnChatDrawerType) {
      this.rightDrawerToggle.emit({
        chatDrawType: this.returnChatDrawerType
      });
    } else {
      this.closed.emit();
    }
  }
  onConfirm() {
    // No Change..
    if (
      !!this.selectedUserList &&
      this.selectedUserList.length > 0 &&
      this.fixedUserList
        .map((user) => user.seq)
        .sort()
        .join('|') ===
        this.selectedUserList
          .map((user) => user.seq)
          .sort()
          .join('|')
    ) {
      this.closed.emit();
    }

    // Invite or Open room.
    let userSeqs = this.selectedUserList.map((userInfo) => userInfo.seq + '');

    if (this.roomInfo.roomType === RoomType.Single) {
      this.appChatService.newOpenRoom(userSeqs, false);
    } else if (this.roomInfo.roomType === RoomType.Multi) {
      if (this.selectedUserList.length >= this.maxChatRoomUser) {
        this.dialog.open<
          AlertDialogComponent,
          AlertDialogData,
          AlertDialogResult
        >(AlertDialogComponent, {
          panelClass: 'min-create-dialog',
          data: {
            title: this.i18nService.t('chat:errors.label'),
            html: this.i18nService.t('chat:errors.maxCountOfRoomMemberWith', {
              maxCount: this.maxChatRoomUser
            })
          }
        });
        return;
      }

      // only target in invite users.
      userSeqs = userSeqs.filter((seq) => {
        return !this.fixedUserList.some(
          (info) => String(info.seq) === String(seq)
        );
      });

      this.store.dispatch(
        RoomActions.invite({
          req: {
            roomId: this.roomId,
            inviteUserSeqs: userSeqs
          } as InviteRequest,
          localeCode: LocaleCode.Korean
        })
      );
    }

    this.closed.emit();
  }

  onChangeUserList(datas: { checked: boolean; userInfo: UserInfoSS }[]) {
    if (!datas || 0 === datas.length) {
      return;
    }

    const pushs: UserInfoSS[] = [];
    const pops: UserInfoSS[] = [];

    datas.forEach((d) => {
      const i = this.selectedUserList.findIndex(
        (u) => String(u.seq) === String(d.userInfo.seq)
      );
      if (d.checked) {
        if (-1 === i) {
          pushs.push(d.userInfo);
        }
      } else {
        if (-1 < i) {
          pops.push(d.userInfo);
        }
      }
    });

    if (0 < pushs.length) {
      this.selectedUserList = [...this.selectedUserList, ...pushs];
    }

    if (0 < pops.length) {
      this.selectedUserList = this.selectedUserList.filter((u) => {
        let result = false;
        if (-1 === pops.findIndex((p) => String(p.seq) === String(u.seq))) {
          result = true;
        }
        if (
          !!this.fixedUserList &&
          this.fixedUserList.length > 0 &&
          this.fixedUserList.findIndex((p) => String(p.seq) === String(u.seq)) >
            -1
        ) {
          result = true;
        }

        return result;
      });
    }
  }
  onChangeGroupList(params: {
    isChecked: boolean;
    groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfo[] };
  }) {
    if (params.isChecked) {
      params.groupBuddyList.buddyList.forEach((item) => {
        if (
          this.selectedUserList.filter(
            (user) => String(user.seq) === String(item.seq)
          ).length === 0
        ) {
          this.selectedUserList = [...this.selectedUserList, item];
        }
      });
    } else {
      this.selectedUserList = this.selectedUserList.filter(
        (item) =>
          params.groupBuddyList.buddyList.filter(
            (del) => String(del.seq) === String(item.seq)
          ).length === 0
      );
    }
  }
  onRemovedProfileSelection(userInfo: UserInfo) {
    const resUserList = this.appGroupService.removedProfileSelection(
      userInfo,
      this.selectedUserList
    );

    if (!resUserList) {
    }
    this.selectedUserList = resUserList;
  }

  removableForSelection = (userInfo: UserInfo) => {
    return !this.fixedUserList.some(
      (info) => String(info.seq) === String(userInfo.seq)
    );
  };

  colorForSelection = (userInfo: UserInfo) => {
    return this.fixedUserList.some(
      (info) => String(info.seq) === String(userInfo.seq)
    )
      ? 'primary'
      : 'accent';
  };
}
