import { Subject, combineLatest } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input
} from '@angular/core';

import { Store, select } from '@ngrx/store';
import { Dictionary } from '@ngrx/entity';

import { User } from '@ucap/domain-organization';
import { RoomInfo, RoomType } from '@ucap/domain-chat';

import { I18nService } from '@ucap/ng-i18n';

import { UserSelector } from '@ucap/ng-store-organization';
import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import {
  RoomSelector,
  RoomUserMap,
  RoomUserShortMap
} from '@ucap/ng-store-chat';

import {
  TranslatePipe as OrganizationTranslate,
  TranslateService
} from '@ucap/ng-ui-organization';

import { AppChatService } from '@app/services/app-chat.service';

@Component({
  selector: 'app-sections-group-info',
  templateUrl: './info.section.component.html',
  styleUrls: ['./info.section.component.scss'],

  changeDetection: ChangeDetectionStrategy.OnPush
})
export class InfoSectionComponent implements OnInit, OnDestroy {
  @Input()
  set userSeq(seq: string) {
    this._userSeq = seq;
    this.getInfoByUser(seq);
  }
  get userSeq(): string {
    return this._userSeq;
  }
  // tslint:disable-next-line: variable-name
  _userSeq: string;

  user: User;
  isMe = false;

  defaultProfileImage: string;
  defaultProfileImageMulti: string;

  roomList: RoomInfo[];
  roomUsersDictionary: Dictionary<RoomUserMap>;
  roomUsersShortDictionary: Dictionary<RoomUserShortMap>;
  filteredRoomList: RoomInfo[] = [];
  allimRoomList: RoomInfo[] = [];

  organizationTranslate: OrganizationTranslate;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private appChatService: AppChatService,
    private i18nService: I18nService,
    private translateService: TranslateService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef
  ) {
    // language setting
    this.translateService.setDefaultLang(this.i18nService.currentLng);
    this.translateService.use(this.i18nService.currentLng);
    this.organizationTranslate = new OrganizationTranslate(
      this.translateService,
      this.changeDetectorRef
    );

    // default image setting
    this.defaultProfileImage = this.appChatService.defaultProfileImage;
    this.defaultProfileImageMulti = this.appChatService.defaultProfileImageMulti;
  }

  ngOnInit(): void {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;

        if (
          (!!this.userSeq && this.userSeq === String(user.info.seq)) ||
          !this.userSeq
        ) {
          this.isMe = true;
        } else {
          this.isMe = false;
        }
      });

    combineLatest([
      this.store.pipe(select(RoomSelector.rooms)),
      this.store.pipe(
        select(
          (state: any) =>
            state.chat.room.roomUsers.entities as Dictionary<RoomUserMap>
        )
      ),
      this.store.pipe(
        select(
          (state: any) =>
            state.chat.room.roomUsersShort.entities as Dictionary<
              RoomUserShortMap
            >
        )
      )
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([rooms, roomUsers, roomUsersShort]) => {
        rooms = (rooms || []).filter((info) => info.isJoinRoom);
        this.roomList = rooms.sort((a, b) =>
          a.finalEventDate < b.finalEventDate
            ? 1
            : a.finalEventDate > b.finalEventDate
            ? -1
            : 0
        );

        this.roomUsersDictionary = roomUsers;
        this.roomUsersShortDictionary = roomUsersShort;

        if (!!rooms && rooms.length > 0) {
          this.getInfoByUser(this.userSeq);
        }

        this.changeDetectorRef.detectChanges();
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  getInfoByUser(userSeq: string | number): void {
    // is Me check
    if (!!userSeq && !!this.user && !!this.user.info.seq) {
      if (String(this.user.info.seq) === String(userSeq)) {
        this.isMe = true;

        // Allim room list setting
        if (!!this.roomList && this.roomList.length > 0) {
          this.allimRoomList = this.roomList.filter((roomInfo) => {
            return (
              roomInfo.roomType === RoomType.Bot ||
              roomInfo.roomType === RoomType.Allim
            );
          });
        }
      } else {
        this.isMe = false;
      }
    }

    // list setting.
    if (!!this.roomList && this.roomList.length > 0) {
      this.filteredRoomList = this.roomList.filter((roomInfo) => {
        if (!!this.isMe) {
          // mine
          return roomInfo.noReadCnt > 0;
        } else {
          // others
          const roomUsers = !!this.roomUsersDictionary
            ? this.roomUsersDictionary[roomInfo.roomId]
            : undefined;
          const roomUsersShort = !!this.roomUsersShortDictionary
            ? this.roomUsersShortDictionary[roomInfo.roomId]
            : undefined;

          let users = [];
          if (!!roomUsers && roomUsers.userInfos.length > 0) {
            users = roomUsers.userInfos;
          } else if (!!roomUsersShort && roomUsersShort.userInfos.length > 0) {
            users = roomUsersShort.userInfos;
          }

          return (
            users.filter(
              (userInfo) =>
                !!userInfo.isJoinRoom &&
                Number(userInfo.seq + '') === Number(userSeq + '')
            ).length > 0
          );
        }
      });
    }

    this.changeDetectorRef.detectChanges();
  }

  onClickRoomItem(event: MouseEvent, roomInfo: RoomInfo): void {
    event.preventDefault();
    event.stopPropagation();

    this.appChatService.openRoombyRoomId(roomInfo.roomId);
  }
}
