import { Subject, of } from 'rxjs';
import { takeUntil, take, map, catchError } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  ViewChild,
  EventEmitter,
  Output,
  ElementRef
} from '@angular/core';

import { Router } from '@angular/router';

import { Store, select } from '@ngrx/store';

import { MatDialog } from '@angular/material/dialog';

import {
  User,
  UserInfo,
  UserInfoSS,
  UserInfoF,
  UserInfoDN,
  DeptSearchType
} from '@ucap/domain-organization';
import { GroupInfoDetail } from '@ucap/domain-group';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';
import { SessionStorageService } from '@ucap/ng-web-storage';
import { QueryProtocolService } from '@ucap/ng-protocol-query';

import { PresenceActions, UserSelector } from '@ucap/ng-store-organization';

import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui/core';

import { Expansion01Component as AppExpansion01Component } from '@app/ucap/group/components/expansion-01.component';
import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { SearchData } from '@app/ucap/organization/models/search-data';
import { UserStore } from '@app/models/user-store';
import { AppChatService } from '@app/services/app-chat.service';
import { GroupManageType, UserInfoTypes } from '@app/types';
import { AppGroupService } from '@app/services/app-group.service';
import { LoginSession } from '@app/models/login-session';
import { GroupOpenInfo } from '@app/models/group-open-info';

import {
  ManageDialogComponent,
  ManageDialogData,
  ManageDialogResult
} from '../dialogs/manage.dialog.component';

import {
  EditUserDialogComponent,
  EditUserDialogData,
  EditUserDialogResult
} from '../dialogs/edit-user.dialog.component';
import { SortViewType } from '@app/pages/group/types/sort-view.type';
import { BuddySelector } from '@ucap/ng-store-group';
import {
  EditInlineInputDialogComponent,
  EditInlineInputDialogData
} from '../dialogs/edit-inline-input.dialog.component';
import { ProfileListComponent as AppProfileListComponent } from '@app/ucap/group/components/profile-list.component';
import { ProfileMenuType } from '@ucap/ng-ui-organization';
import { GroupMenuType } from '@ucap/ng-ui-group';
import { ConferenceCreateRequest } from '@ucap/api-prompt';
import { LoginSelector } from '@ucap/ng-store-authentication';
import { LoginInfo } from '@ucap/domain-authentication';

@Component({
  selector: 'app-sections-group-list',
  templateUrl: './list.section.component.html',
  styleUrls: ['./list.section.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ListSectionComponent implements OnInit, OnDestroy {
  @Input()
  set searchData(searchData: SearchData) {
    this._searchData = searchData;
    if (!searchData) {
      this._searchData = {
        companyCode: this.userStore.companyCode
      };
    } else {
      if (!this._searchData.companyCode) {
        this._searchData.companyCode = this.userStore.companyCode;
      }

      if (
        searchData.bySearch &&
        searchData.searchWord.localeCompare('') !== 0
      ) {
        this.onOrganizationTenantSearch(searchData);
      } else {
        this._searchData.isShowSearch = false;
        this.searchUserInfos = [];
      }
    }
  }

  get searchData() {
    return this._searchData;
  }
  // tslint:disable-next-line: variable-name
  _searchData: any;

  @Input()
  checkable = false;

  @Input()
  showType: SortViewType;

  @Output()
  checkUser = new EventEmitter<{
    isChecked: boolean;
    userInfo: any;
  }>();

  @Output()
  clickUser = new EventEmitter<UserInfo>();

  @Output()
  openProfile = new EventEmitter<UserInfo>();

  @ViewChild('appGroupExpansion01', { static: false })
  appGroupExpansion: AppExpansion01Component;

  @ViewChild('nicknameInput', { static: false })
  nicknameInput: ElementRef<HTMLInputElement>;

  @ViewChild('appProfileList', { static: false })
  appProfileList: AppProfileListComponent;

  loginInfo: LoginInfo;
  loginSession: LoginSession;
  user: User;

  searchUserInfos: UserInfoSS[] = [];
  buddyList: UserInfoTypes[];

  private ngOnDestroySubject: Subject<void> = new Subject();
  private userStore: UserStore;

  constructor(
    private router: Router,
    private appAuthenticationService: AppAuthenticationService,
    private sessionStorageService: SessionStorageService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private i18nService: I18nService,
    private logService: LogService,
    private queryProtocolService: QueryProtocolService,
    private dialog: MatDialog,
    private appChatService: AppChatService,
    private appGroupService: AppGroupService
  ) {
    this.userStore = this.appAuthenticationService.getUserStore();
  }

  ngOnInit(): void {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(LoginSelector.loginInfo))
      .subscribe((loginInfo) => {
        this.loginInfo = loginInfo;
      });
    this.appAuthenticationService
      .getLoginSession$()
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((loginSession) => {
        this.loginSession = loginSession;
      });
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(BuddySelector.buddies))
      .subscribe((buddies) => {
        this.buddyList = buddies;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onOrganizationTenantSearch(searchData: SearchData) {
    const searchUserInfos: UserInfoSS[] = [];

    this.queryProtocolService
      .deptUser({
        divCd: 'GRP',
        companyCode: searchData.companyCode,
        searchRange: DeptSearchType.All,
        search: searchData.searchWord,
        senderCompanyCode: this.user.info.companyCode,
        senderEmployeeType: this.user.info.employeeType
      })
      .pipe(
        take(1),
        map((resObj) => {
          const userInfos = resObj.userInfos;

          searchUserInfos.push(...userInfos);
          // 검색 결과 처리.
          this.searchUserInfos = searchUserInfos.sort((a, b) =>
            a.name < b.name ? -1 : a.name > b.name ? 1 : 0
          );
          // this.searchProcessing = false;

          // 검색 결과에 따른 프레즌스 조회.
          const userSeqList: string[] = [];
          this.searchUserInfos.map((user) => userSeqList.push(user.seq));
          this.changeDetectorRef.markForCheck();

          if (userSeqList.length > 0) {
            this.store.dispatch(
              PresenceActions.bulkInfo({
                divCd: 'groupSrch',
                userSeqs: userSeqList
              })
            );
          }
        }),
        catchError((error) => {
          // this.searchProcessing = false;
          return of(error);
        })
      )
      .subscribe();
  }

  onCheckUser(params: { isChecked: boolean; userInfo: any }) {
    this.checkUser.emit(params);
  }

  onClickUser(params: { event: MouseEvent; userInfo: UserInfo }) {
    if (!!params.event) {
      params.event.preventDefault();
      params.event.stopPropagation();
    }

    this.clickUser.emit(params.userInfo);
  }

  onOpenProfile(userInfo: UserInfo) {
    this.openProfile.emit(userInfo);
  }

  onClickSearchUser(event: MouseEvent, userInfo: UserInfo) {
    this.onClickUser({ event, userInfo });
  }

  onSearched(userInfo: UserInfoSS[]) {
    if (!!this.appProfileList) {
      this.appProfileList.psUpdate();
    }
  }

  onGroupDestroy(groupOpenInfo: GroupOpenInfo) {
    this.appAuthenticationService.setLoginSession({
      ...this.loginSession,
      groupInfo: groupOpenInfo
    });
  }
  onExpandMore() {
    this.appGroupExpansion.onExpandMore();
  }
  onExpandLess() {
    this.appGroupExpansion.onExpandLess();
  }

  onProfileMenu(event) {
    console.log(event);
  }
  onSelectGroupMenu(params: {
    menuType: GroupMenuType;
    groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfo[] };
    rect: any;
  }) {
    switch (params.menuType) {
      case GroupMenuType.chat:
        {
          this.appChatService.newOpenRoom(
            params.groupBuddyList.group.userSeqs,
            false,
            this.user
          );
        }
        break;
      case GroupMenuType.message:
        {
          return;

          this.dialog.open<
            AlertDialogComponent,
            AlertDialogData,
            AlertDialogResult
          >(AlertDialogComponent, {
            data: {
              title: this.i18nService.t('group:dialog.title.messageGroup'),
              html: this.i18nService.t('group:dialog.title.messageGroup')
            }
          });
        }
        break;
      case GroupMenuType.rename:
        {
          this.renameGroup(params);
        }
        break;
      case GroupMenuType.management:
        {
          const dialogRef = this.dialog.open<
            ManageDialogComponent,
            ManageDialogData,
            ManageDialogResult
          >(ManageDialogComponent, {
            panelClass: 'max-create-dialog',
            data: {
              title: this.i18nService.t('group:dialog.title.managementGroup'),
              groupBuddyList: params.groupBuddyList
            }
          });
          dialogRef
            .afterClosed()
            .pipe(take(1))
            .subscribe((result) => {
              if (!!result && !!result.choice) {
                this.appGroupService.updateMemberToGroup(
                  result.group,
                  result.selelctUserList
                );
              }
            });
          dialogRef
            .afterOpened()
            .pipe(take(1))
            .subscribe(() => {
              dialogRef.componentInstance.psUpdate();
            });
        }
        break;
      case GroupMenuType.copyGroup:
        {
          this.editUserDialog(
            this.i18nService.t('group:dialog.title.copyGroup'),
            GroupManageType.Copy,
            params.groupBuddyList.group,
            params.groupBuddyList.buddyList
          );
        }
        break;
      case GroupMenuType.moveGroup:
        {
          this.editUserDialog(
            this.i18nService.t('group:dialog.title.moveGroup'),
            GroupManageType.Move,
            params.groupBuddyList.group,
            params.groupBuddyList.buddyList
          );
        }
        break;
      case GroupMenuType.deleteGroup:
        {
          this.appGroupService.removeGroup(params.groupBuddyList);
        }
        break;
      default:
        break;
    }
  }

  onSelectProfileMenu(params: {
    menuType: ProfileMenuType;
    userInfo: UserInfoF;
    group: GroupInfoDetail;
    rect: any;
  }) {
    switch (params.menuType) {
      case ProfileMenuType.favorite:
        this.appGroupService.updateBuddy(
          params.userInfo,
          !params.userInfo.isFavorit
        );
        break;
      case ProfileMenuType.nickname:
        {
          this.editNickname(params.userInfo, params.rect);
        }
        break;
      case ProfileMenuType.copyBuddy:
        this.editUserDialog(
          this.i18nService.t('group:contextMenu.copyBuddy'),
          GroupManageType.Copy,
          params.group,
          [params.userInfo]
        );
        break;
      case ProfileMenuType.moveBuddy:
        this.editUserDialog(
          this.i18nService.t('group:contextMenu.moveBuddy'),
          GroupManageType.Move,
          params.group,
          [params.userInfo]
        );
        break;
      case ProfileMenuType.removeBuddy:
        {
          this.removeBuddy(params.userInfo, params.group);
        }
        break;
    }
  }

  onClickProfileMenu(params: {
    menuType: ProfileMenuType;
    userInfo: UserInfoF;
  }) {
    switch (params.menuType) {
      case ProfileMenuType.chat:
        this.appChatService.newOpenRoom(
          [params.userInfo.seq as any],
          false,
          this.user
        );
        break;
      case ProfileMenuType.message:
        break;
      case ProfileMenuType.mobile:
        break;
      case ProfileMenuType.office:
        break;
      case ProfileMenuType.videoConference:
        {
          const req: ConferenceCreateRequest = {
            userSeq: String(this.user.info.seq),
            deviceType: this.loginSession.deviceType,
            tokenKey: this.loginInfo.tokenString,
            targetUserSeqs: [params.userInfo.seq]
          };
          this.appChatService.openVideoConference(req);
        }
        break;
    }
  }

  private renameGroup(params: {
    menuType: GroupMenuType;
    groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfo[] };
    rect: any;
  }) {
    this.showEditInlineInput(
      'group',
      params.rect,
      params.groupBuddyList.group,
      null
    ).then((result) => {
      this.appGroupService.updateGroupName(
        params.groupBuddyList.group,
        result.curName
      );
    });
  }

  private editUserResult(result: EditUserDialogResult) {
    if (!result) {
      return;
    }

    let isCreateGroup = false;

    if (!!result.groupName && result.groupName.trim() !== '') {
      isCreateGroup = true;
      this.appGroupService.createGroup(
        result.groupName,
        result.selelctUserList
      );
    }

    if (result.type === GroupManageType.Copy) {
      if (!!result.selectGroupList && result.selectGroupList.length > 0) {
        result.selectGroupList.map((g) => {
          this.appGroupService.copyMemberToGroup(g, result.selelctUserList);
        });
      }
    } else if (result.type === GroupManageType.Move) {
      const fromGroup = result.group;
      let toGroup: GroupInfoDetail;
      let targetUserSeqs: string[];

      if (!!result.selectGroupList && result.selectGroupList.length > 0) {
        result.selectGroupList.forEach((g, idx) => {
          toGroup = g;
          targetUserSeqs = [];

          if (!!result.selelctUserList && result.selelctUserList.length > 0) {
            if (idx === 0) {
              this.appGroupService.moveMemberToGroup(
                fromGroup,
                toGroup,
                result.selelctUserList
              );
            } else {
              this.appGroupService.copyMemberToGroup(
                toGroup,
                result.selelctUserList
              );
            }
          }
        });
      } else if (
        !!isCreateGroup &&
        !!result.selelctUserList &&
        result.selelctUserList.length > 0
      ) {
        this.appGroupService.removeMember(result.group, result.selelctUserList);
      }
    }
  }

  private editUserDialog(
    title: string,
    dialogType: GroupManageType,
    group: GroupInfoDetail,
    userInfos: UserInfoTypes[]
  ) {
    const dialogRef = this.dialog.open<
      EditUserDialogComponent,
      EditUserDialogData,
      EditUserDialogResult
    >(EditUserDialogComponent, {
      panelClass: 'max-create-dialog',
      data: {
        title,
        type: dialogType,
        group,
        userInfos
      }
    });
    dialogRef
      .afterClosed()
      .pipe(
        take(1),
        map((result: EditUserDialogResult) => {
          if (!!result) {
            this.editUserResult(result);
          }
        }),
        catchError((err) => {
          return of(err);
        })
      )
      .subscribe();
  }

  private removeBuddy(userInfo: UserInfoF, group: GroupInfoDetail) {
    this.appGroupService.removeMemberToGroup(
      this.i18nService.t('group:dialog.removeBuddyConfirm', {
        targetMember: `${userInfo.name}`
      }),
      [userInfo],
      group
    );
  }
  private editNickname(userInfo: UserInfoF, rect: any) {
    const isBuddy = this.appGroupService.checkBuddy(this.buddyList, userInfo);

    if (!isBuddy) {
      return;
    }

    this.showEditInlineInput('nickname', rect, null, userInfo).then(
      (result) => {
        this.appGroupService.updateNickname(result.userInfo, result.curName);
      }
    );
  }

  private showEditInlineInput(
    type: string,
    rect: any,
    group?: GroupInfoDetail,
    userInfo?: UserInfoF
  ): Promise<{
    type: string;
    group?: GroupInfoDetail;
    userInfo?: UserInfoF;
    curName: string;
  }> {
    return new Promise<{
      type: string;
      group?: GroupInfoDetail;
      userInfo?: UserInfoF;
      curName: string;
    }>((resolve, rejects) => {
      try {
        let dialogWidth: string;
        let dialogHeight: string;
        let dialogData: EditInlineInputDialogData;

        if (type === 'group') {
          dialogWidth = rect.width;
          dialogHeight = rect.height;
          dialogData = {
            type: 'GROUP_NAME',
            curValue: group.name,
            placeholder: this.i18nService.t('group:dialog.newGroupName'),
            maxLength: 20,
            left: rect.left,
            top: rect.top
          };
        } else {
          dialogWidth = rect.width - 30 + '';
          dialogHeight = rect.height;
          dialogData = {
            type: 'NICKNAME',
            curValue: userInfo.nickName,
            placeholder: this.i18nService.t('group:dialog.newNickname'),
            maxLength: 20,
            left: rect.left + 70,
            top: rect.top
          };
        }
        const dialogRef = this.dialog.open(EditInlineInputDialogComponent, {
          width: dialogWidth,
          height: dialogHeight,
          panelClass: 'ucap-edit-group-name-dialog',
          data: dialogData
        });

        dialogRef
          .afterClosed()
          .pipe(
            take(1),
            map((result) => {
              if (
                !!result &&
                result.choice &&
                result.curValue.localeCompare(name) !== 0
              ) {
                resolve({ type, group, userInfo, curName: result.curValue });
              }
            })
          )
          .subscribe();
      } catch (error) {
        rejects(error);
      }
    });
  }
}
