import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject,
  ViewChild
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { UserInfo, UserInfoSS } from '@ucap/domain-organization';
import { GroupInfoDetail } from '@ucap/domain-group';

import { MatStepper } from '@angular/material/stepper';
import { I18nService } from '@ucap/ng-i18n';

import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui/core';

import { AppGroupService } from '@app/services/app-group.service';

import { SelectUserSectionComponent } from '../components/select-user.section.component';
import { UserInfoTypes } from '@app/types';
import { GroupSelector } from '@ucap/ng-store-group';
import { takeUntil } from 'rxjs/operators';

export interface AddUserDialogData {}
export interface AddUserDialogResult {}
enum GroupNameType {
  none = 'NONE',
  empty = 'EMPTY',
  exist = 'EXIST'
}
@Component({
  selector: 'app-dialog-group-add-user',
  templateUrl: './add-user.dialog.component.html',
  styleUrls: ['./add-user.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class AddUserDialogComponent implements OnInit, OnDestroy {
  @ViewChild('groupSelectUser', { static: false })
  groupSelectUser: SelectUserSectionComponent;

  constructor(
    public dialogRef: MatDialogRef<AddUserDialogData, AddUserDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: AddUserDialogData,
    private changeDetectorRef: ChangeDetectorRef,
    private store: Store<any>,
    private i18nService: I18nService,
    public dialog: MatDialog,
    private appGroupService: AppGroupService
  ) {}

  private ngOnDestroySubject: Subject<void> = new Subject();
  private groupName: string;
  private defaultGroup: GroupInfoDetail;

  currentStep = 0;
  isDisable = true;

  selectedUserList: UserInfoTypes[] = [];
  selectedGroupList: GroupInfoDetail[] = [];

  ngOnInit(): void {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(GroupSelector.groups))
      .subscribe((groups) => {
        this.defaultGroup = groups.filter((group) => group.seq === 0)[0];
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  psUpdate() {
    this.groupSelectUser.psUpdate();
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close();
  }

  onCancel(stepper: MatStepper) {
    if (stepper.selectedIndex > 0) {
      this.currentStep--;
      stepper.previous();
      return;
    }
    this.dialogRef.close();
  }
  onConfirm(stepper: MatStepper) {
    let contents: string;

    if (
      this.currentStep === 1 &&
      !!this.isDisable &&
      this.isGroupName() === GroupNameType.exist
    ) {
      contents = this.i18nService.t('group:error.invalidGroupName');
    }

    if (!!this.selectedUserList && this.selectedUserList.length === 0) {
      contents = this.i18nService.t('group:error.notSelectedUser');
    }

    if (!!contents && contents.localeCompare('') !== 0) {
      this.dialog.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('group:error.title.default'),
          html: contents
        }
      });
      return;
    }

    const userSeqs: string[] = [];
    this.selectedUserList.map((user) => userSeqs.push(String(user.seq)));

    if (this.currentStep === 1) {
      if (this.isGroupName() === GroupNameType.exist) {
        this.appGroupService.createGroup(this.groupName, this.selectedUserList);
      }

      if (
        !!this.selectedGroupList &&
        this.selectedGroupList.length > 0 &&
        !!this.selectedUserList &&
        this.selectedUserList.length > 0
      ) {
        this.selectedGroupList.map((g) => {
          this.appGroupService.copyMemberToGroup(g, this.selectedUserList);
        });
      } else if (
        this.isGroupName() !== GroupNameType.exist &&
        this.selectedUserList.length > 0 &&
        this.selectedGroupList.length === 0
      ) {
        this.appGroupService.addMemberToGroup(
          this.defaultGroup,
          this.selectedUserList
        );
      }
    } else {
      this.appGroupService.addMemberToGroup(
        this.defaultGroup,
        this.selectedUserList
      );
    }

    this.dialogRef.close();
  }
  onCompleteConfirm(stepper: MatStepper) {
    this.currentStep++;
    stepper.next();
  }

  disabledConfirmBtn(): boolean {
    if (this.currentStep === 0) {
      return this.disabledCompleateBtn();
    } else {
      if (this.selectedUserList.length === 0) {
        return true;
      }

      if (!!this.isDisable && this.isGroupName() === GroupNameType.exist) {
        return true;
      }

      return false;
    }
  }

  disabledCompleateBtn(): boolean {
    if (!!this.selectedUserList && this.selectedUserList.length > 0) {
      return false;
    }
    return true;
  }
  onToggleCheckUser(datas: { checked: boolean; userInfo: UserInfoSS }[]) {
    const resList = this.appGroupService.getToggleCheckUser(
      datas,
      this.selectedUserList
    );

    if (!resList) {
      return;
    }

    this.selectedUserList = resList;
  }

  onToggleCheckGroup(params: {
    isChecked: boolean;
    groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfo[] };
  }) {
    const resList = this.appGroupService.getToggleCheckGroup(
      params,
      this.selectedUserList
    );

    if (!resList) {
      return;
    }

    this.selectedUserList = resList;
  }

  onToggleCheckForGroupList(datas: {
    checked: boolean;
    group: GroupInfoDetail;
  }) {
    this.selectedGroupList = this.appGroupService.getToggleCheckForGroupList(
      datas,
      this.selectedGroupList
    );
  }

  onChangeGroupName(data: { invalid: boolean; groupName: string }) {
    this.isDisable = data.invalid;
    this.groupName = data.groupName;
  }

  onRemovedProfileSelection(userInfo: UserInfo) {
    const resUserList = this.appGroupService.removedProfileSelection(
      userInfo,
      this.selectedUserList
    );

    if (!resUserList) {
    }
    this.selectedUserList = resUserList;
  }

  removableForSelection = (userInfo: UserInfo) => {
    return true;
  };

  colorForSelection = (userInfo: UserInfo) => {
    return 'accent';
  };

  private isGroupName(): GroupNameType {
    if (!this.groupName) {
      return GroupNameType.none;
    } else if (!!this.groupName && this.groupName.trim() === '') {
      return GroupNameType.empty;
    }

    return GroupNameType.exist;
  }
}
