import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject,
  Input,
  ViewChild
} from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';

import { Store, select } from '@ngrx/store';

import { MatStepper } from '@angular/material/stepper';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { UserInfo, UserInfoSS } from '@ucap/domain-organization';
import { GroupInfoDetail } from '@ucap/domain-group';

import { I18nService } from '@ucap/ng-i18n';
import { GroupSelector } from '@ucap/ng-store-group';
import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui/core';

import { AppGroupService } from '@app/services/app-group.service';
import { UserInfoTypes } from '@app/types';

import { SelectUserSectionComponent } from '../components/select-user.section.component';

export interface CreateDialogData {
  title: string;
}
export interface CreateDialogResult {}

@Component({
  selector: 'app-dialog-group-create',
  templateUrl: './create.dialog.component.html',
  styleUrls: ['./create.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class CreateDialogComponent implements OnInit, OnDestroy {
  @ViewChild('groupSelectUser', { static: false })
  groupSelectUser: SelectUserSectionComponent;

  constructor(
    public dialogRef: MatDialogRef<CreateDialogData, CreateDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: CreateDialogData,
    private changeDetectorRef: ChangeDetectorRef,
    private store: Store<any>,
    private formBuilder: FormBuilder,
    private i18nService: I18nService,
    public dialog: MatDialog,
    private appGroupService: AppGroupService
  ) {}

  private ngOnDestroySubject: Subject<void> = new Subject();
  private groupName: string;
  private defaultGroup: GroupInfoDetail;

  currentStep = 0;
  isDisable = true;

  selectedUserList: UserInfoTypes[] = [];

  ngOnInit(): void {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(GroupSelector.groups))
      .subscribe((groups) => {
        this.defaultGroup = groups.filter((group) => group.seq === 0)[0];
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close();
  }

  onCancel(stepper: MatStepper) {
    if (stepper.selectedIndex > 0) {
      this.currentStep--;
      stepper.previous();
      return;
    }
    this.dialogRef.close();
  }
  onConfirm(stepper: MatStepper) {
    let contents: string;
    let isDefaultGroup = false;

    if (
      !this.groupName ||
      this.groupName.localeCompare('') === 0 ||
      this.groupName.trim().localeCompare('') === 0
    ) {
      contents = this.i18nService.t('group:error.requireGroupName');
    } else {
      if (this.currentStep === 0) {
        if (this.isDisable) {
          contents = this.i18nService.t('group:error.invalidGroupName');
        }
      } else {
        if (
          !this.groupName ||
          this.groupName.localeCompare('') === 0 ||
          this.groupName.trim().localeCompare('') === 0
        ) {
          isDefaultGroup = true;
        }
        if (!!this.selectedUserList && this.selectedUserList.length === 0) {
          contents = this.i18nService.t('group:error.notSelectedUser');
        }
      }
    }

    if (!!contents && contents.localeCompare('') !== 0) {
      this.dialog.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('group:error.title.default'),
          html: contents
        }
      });
      return;
    }

    const userSeqs: string[] = [];
    this.selectedUserList.map((user) => userSeqs.push(String(user.seq)));

    if (!isDefaultGroup) {
      this.appGroupService.createGroup(this.groupName, this.selectedUserList);
    } else {
      this.appGroupService.addMemberToGroup(
        this.defaultGroup,
        this.selectedUserList
      );
    }

    this.dialogRef.close();
  }
  onCompleteConfirm(stepper: MatStepper) {
    this.currentStep++;
    stepper.next();
    this.groupSelectUser.psUpdate();
  }

  checkDisableConfirmBtn() {
    if (!!this.groupName && this.groupName.trim().localeCompare('') === 0) {
      return true;
    } else if (
      this.isDisable &&
      !!this.groupName &&
      this.groupName.trim().localeCompare('') !== 0
    ) {
      return true;
    }

    return false;
  }

  onChangeGroupName(data: { invalid: boolean; groupName: string }) {
    this.isDisable = data.invalid;
    this.groupName = data.groupName;
  }

  onToggleCheckUser(datas: { checked: boolean; userInfo: UserInfoSS }[]) {
    const resList = this.appGroupService.getToggleCheckUser(
      datas,
      this.selectedUserList
    );

    if (!resList) {
      return;
    }
    this.selectedUserList = resList;
  }

  onToggleCheckGroup(params: {
    isChecked: boolean;
    groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfo[] };
  }) {
    const resList = this.appGroupService.getToggleCheckGroup(
      params,
      this.selectedUserList
    );

    if (!resList) {
      return;
    }
    this.selectedUserList = resList;
  }

  onRemovedProfileSelection(userInfo: UserInfo) {
    const resUserList = this.appGroupService.removedProfileSelection(
      userInfo,
      this.selectedUserList
    );

    if (!resUserList) {
    }
    this.selectedUserList = resUserList;
  }

  removableForSelection = (userInfo: UserInfo) => {
    return true;
  };

  colorForSelection = (userInfo: UserInfo) => {
    return 'accent';
  };
}
