import { Subject, of } from 'rxjs';
import { take, map, catchError, takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  Inject
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog,
  MatDialogConfig
} from '@angular/material/dialog';

import { I18nService } from '@ucap/ng-i18n';

import { GroupInfoDetail } from '@ucap/domain-group';

import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui/core';
import { GroupSelector } from '@ucap/ng-store-group';

import { AppGroupService } from '@app/services/app-group.service';

export interface EditInlineInputDialogData {
  type: string;
  curValue?: string;
  placeholder: string;
  maxLength: number;
  left?: number;
  top?: number;
}
export interface EditInlineInputDialogResult {
  choice: boolean;
  curValue?: string;
}

@Component({
  selector: 'app-dialog-group-edit-inline-input',
  templateUrl: './edit-inline-input.dialog.component.html',
  styleUrls: ['./edit-inline-input.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class EditInlineInputDialogComponent implements OnInit, OnDestroy {
  constructor(
    public dialogRef: MatDialogRef<
      EditInlineInputDialogData,
      EditInlineInputDialogResult
    >,
    @Inject(MAT_DIALOG_DATA) public data: EditInlineInputDialogData,

    private store: Store<any>,
    private i18nService: I18nService,
    public dialog: MatDialog,
    private appGroupService: AppGroupService
  ) {}

  private ngOnDestroySubject: Subject<void>;
  groupList: GroupInfoDetail[] = [];

  ngOnInit(): void {
    const matDialogConfig: MatDialogConfig = new MatDialogConfig();

    matDialogConfig.position = {
      left: `${this.data.left}px`,
      top: `${this.data.top}px`
    };

    this.dialogRef.updatePosition(matDialogConfig.position);

    this.ngOnDestroySubject = new Subject();

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(GroupSelector.groups))
      .subscribe((groups) => {
        this.groupList = groups;
      });

    if (
      !!this.data &&
      !!this.data.curValue &&
      this.data.curValue.trim() === ''
    ) {
      this.data.curValue = this.data.curValue.trim();
    }
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClosed(event: MouseEvent): void {
    this.doAction(false);
  }

  onApply(inputValue: string) {
    if (!!this.data && this.data.type.localeCompare('GROUP_NAME') === 0) {
      const errMsg = this.appGroupService.validateGroupName(
        inputValue,
        this.groupList
      );

      if (!!errMsg && errMsg !== '') {
        return this._alertErr(errMsg);
      }
    }

    this.doAction(true, inputValue);
  }

  doAction(choice: boolean, curValue?: string): void {
    if (!choice) {
      this.dialogRef.close({ choice: false });
      return;
    }

    this.dialogRef.close({ choice, curValue });
  }

  private _alertErr(msg: string) {
    if (msg.localeCompare('') !== 0) {
      const dialogRef = this.dialog.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        data: {
          title: this.i18nService.t('group:error.title.default'),
          html: msg
        }
      });
      dialogRef
        .afterClosed()
        .pipe(
          take(1),
          map((result) => {}),
          catchError((err) => {
            return of(err);
          })
        )
        .subscribe();
      return;
    }
  }
}
