import { Subject } from 'rxjs';
import { takeUntil, take } from 'rxjs/operators';

import {
  Component,
  OnInit,
  ChangeDetectorRef,
  Input,
  OnDestroy,
  ChangeDetectionStrategy,
  ViewChild
} from '@angular/core';

import { MediaObserver } from '@angular/flex-layout';

import { MatCheckboxChange, MatCheckbox } from '@angular/material/checkbox';
import { MatDialog } from '@angular/material/dialog';

import { select, Store } from '@ngrx/store';

import { SortOrder } from '@ucap/domain-common';
import {
  DeptInfo,
  User,
  UserInfo,
  UserInfoSS,
  Company
} from '@ucap/domain-organization';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';

import {
  DepartmentSelector,
  CompanySelector,
  UserSelector
} from '@ucap/ng-store-organization';
import {
  AuthorizationSelector,
  LoginSelector
} from '@ucap/ng-store-authentication';

import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui/core';

import { SearchData } from '@app/ucap/organization/models/search-data';
import { ProfileListComponent as AppProfileListComponent } from '@app/ucap/organization/components/profile-list.component';
import { AppChatService } from '@app/services/app-chat.service';

import {
  AddGroupDialogComponent,
  AddGroupDialogData,
  AddGroupDialogResult
} from '../dialogs/add-group.dialog.component';
import {
  ProfileDialogComponent,
  ProfileDialogData,
  ProfileDialogResult
} from '../dialogs/profile.dialog.component';

import { ProfileMenuType } from '@ucap/ng-ui-organization';
import { ConferenceCreateRequest } from '@ucap/api-prompt';

import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { LoginInfo } from '@ucap/domain-authentication';
import { UserPermission } from '@ucap/domain-authorization';

@Component({
  selector: 'app-sections-organization-member-list',
  templateUrl: './member-list.component.html',
  styleUrls: ['./member-list.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class MemberListComponent implements OnInit, OnDestroy {
  @Input()
  set searchData(searchData: SearchData) {
    this._searchData = searchData;

    this.selectedDeptInfo = undefined;
    this.selectedCompanyInfo = undefined;

    this.sortOrderForProfileList = {
      ...this.sortOrderForProfileList,
      ascending: undefined
    };

    if (!!searchData && !searchData.bySearch && !!searchData.deptSeq) {
      this.setDeptInfo(searchData.deptSeq);
    } else {
      // if (!!this.loginInfo) {
      //   this.setDeptInfo(this.loginInfo.departmentCode + '');
      // }
      // this.setCompanyInfo(searchData.companyCode);
    }
  }

  @Input()
  cacheSize = 0;

  @ViewChild('profileList', { static: false })
  profileList: AppProfileListComponent;

  @ViewChild('checkboxAll', { static: false })
  checkboxAll: MatCheckbox;

  // tslint:disable-next-line: variable-name
  _searchData: SearchData;
  user: User;
  selectedDeptInfo: DeptInfo;
  selectedCompanyInfo: Company;
  searchedProfileLength: number;
  searchedUserInfos: UserInfoSS[] = [];
  selectedUserInfos: UserInfoSS[] = [];
  isExpanded = false;
  isAllCheck = false;
  userPermission: UserPermission;
  loginInfo: LoginInfo;

  sortOrderForProfileList: SortOrder = {
    property: 'name',
    ascending: undefined
  };

  layoutMode: 'min' | 'mid' | 'max';

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private appAuthenticationService: AppAuthenticationService,
    private appChatService: AppChatService,
    public mediaObserver: MediaObserver,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService,
    private dialog: MatDialog,
    private i18nService: I18nService
  ) {}

  ngOnInit() {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
        if (
          !!this._searchData &&
          !this._searchData.bySearch &&
          !!this._searchData.deptSeq
        ) {
          // ignore..
        } else {
          if (!!user) {
            this.setDeptInfo(user.departmentCode + '');
          }
        }
      });

    this.mediaObserver
      .asObservable()
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((changes) => {
        if (!changes || 0 === changes.length) {
          return;
        }
        for (const change of changes) {
          switch (change.mqAlias) {
            case 'lt-sm':
              this.layoutMode = 'min';
              this.changeDetectorRef.detectChanges();
              return;
            case 'sm':
            case 'md':
              this.layoutMode = 'mid';
              this.changeDetectorRef.detectChanges();
              return;
            case 'gt-md':
              this.layoutMode = 'max';
              this.changeDetectorRef.detectChanges();
              return;
            default:
              break;
          }
        }
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(LoginSelector.loginInfo))
      .subscribe((loginInfo) => {
        this.loginInfo = loginInfo;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(AuthorizationSelector.userPermission)
      )
      .subscribe((userPermission) => {
        this.userPermission = userPermission;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onChangedSearch(data: { deptSeq: string; searchWord: string }) {
    this._searchData = {
      bySearch: true,
      ...data
    };

    this.setDeptInfo(data.deptSeq);
  }

  onSearchedProfileList(userInfos: UserInfoSS[]) {
    if (!!this.checkboxAll) {
      this.checkboxAll.checked = false;
    }

    this.searchedUserInfos = userInfos;
    this.searchedProfileLength = !!userInfos ? userInfos.length : 0;

    this._changeAllCheck();
  }

  onChangedCheckProfileList(
    datas: { checked: boolean; userInfo: UserInfoSS }[]
  ) {
    if (!datas || 0 === datas.length) {
      return;
    }

    const pushs: UserInfoSS[] = [];
    const pops: UserInfoSS[] = [];

    datas.forEach((d) => {
      const i = this.selectedUserInfos.findIndex(
        (u) => u.seq === d.userInfo.seq
      );
      if (d.checked) {
        if (-1 === i) {
          pushs.push(d.userInfo);
        }
      } else {
        if (-1 < i) {
          pops.push(d.userInfo);
        }
      }
    });

    if (0 < pushs.length) {
      this.selectedUserInfos = [...this.selectedUserInfos, ...pushs];
    }

    if (0 < pops.length) {
      this.selectedUserInfos = this.selectedUserInfos.filter(
        (u) => -1 === pops.findIndex((p) => p.seq === u.seq)
      );
    }
    this._changeAllCheck();
    this.changeDetectorRef.markForCheck();
  }

  private _changeAllCheck() {
    const findUser: UserInfoSS[] = [];
    let isChecked = false;

    if (
      !!this.searchedUserInfos &&
      !!this.selectedUserInfos &&
      this.searchedUserInfos.length > 0 &&
      this.selectedUserInfos.length > 0
    ) {
      this.selectedUserInfos.forEach((selUser) => {
        this.searchedUserInfos.every((searchUser) => {
          if (selUser.seq === searchUser.seq) {
            findUser.push(selUser);
            return false;
          }
          return true;
        });
      });
    }

    if (
      findUser.length > 0 &&
      findUser.length === this.searchedUserInfos.length
    ) {
      isChecked = true;
    }
    if (!!this.checkboxAll) {
      this.checkboxAll.checked = isChecked;
    }
  }

  onRemovedProfileSelection(userInfo: UserInfo) {
    const i = this.selectedUserInfos.findIndex(
      (u) => u.seq === String(userInfo.seq)
    );

    if (-1 < i) {
      this.selectedUserInfos = this.selectedUserInfos.filter(
        (u) => u.seq !== String(userInfo.seq)
      );
    }
    this._changeAllCheck();
    this.changeDetectorRef.markForCheck();
  }

  onOpenProfile(userInfo: UserInfoSS) {
    const result = this.dialog.open<
      ProfileDialogComponent,
      ProfileDialogData,
      ProfileDialogResult
    >(ProfileDialogComponent, {
      panelClass: 'mid-create-dialog',
      data: {
        userSeq: userInfo.seq
      }
    });
  }
  removableForSelection = (userInfo: UserInfo) => {
    return true;
  };

  colorForSelection = (userInfo: UserInfo) => {
    return 'accent';
  };

  onClickToggleSort() {
    this.sortOrderForProfileList = {
      ...this.sortOrderForProfileList,
      ascending:
        undefined === this.sortOrderForProfileList.ascending
          ? true
          : true === this.sortOrderForProfileList.ascending
          ? false
          : undefined
    };
  }

  onChangeSelectAll(event: MatCheckboxChange) {
    if (event.checked) {
      this.profileList.checkAll();
    } else {
      this.profileList.uncheckAll();
    }
  }

  getContainerLayout(type: 'max' | 'mid'): string {
    switch (type) {
      case 'max':
        return 'row';
      case 'mid':
        return 'column';
      default:
        break;
    }
  }

  onAfterExpandForSelection() {
    this.isExpanded = true;
  }

  onAfterCollapseForSelection() {
    this.isExpanded = false;
  }

  onClearSelected(): void {
    if (!!this.selectedUserInfos && this.selectedUserInfos.length > 0) {
      const tempSelectUser: UserInfoSS[] = [];

      this.selectedUserInfos.forEach((selectUser) => {
        this.searchedUserInfos.every((searchUser) => {
          if (selectUser.seq === searchUser.seq) {
            tempSelectUser.push(searchUser);
            return false;
          }
          return true;
        });
      });

      this.selectedUserInfos = [...tempSelectUser];
    }

    this.profileList.uncheckAll();
  }

  private setCompanyInfo(companyCode: string) {
    const destroySubject: Subject<void> = new Subject();
    this.store
      .pipe(takeUntil(destroySubject), select(CompanySelector.companyList))
      .subscribe((companyList) => {
        if (!companyList) {
          return;
        }
        this.selectedCompanyInfo = companyList.find(
          (c) => c.companyCode === companyCode
        );
        this.changeDetectorRef.markForCheck();
        destroySubject.next();
        destroySubject.complete();
      });
  }

  private setDeptInfo(seq: string) {
    const destroySubject: Subject<void> = new Subject();
    this.store
      .pipe(
        takeUntil(destroySubject),
        select(DepartmentSelector.departmentInfoList)
      )
      .subscribe((departmentInfoList) => {
        if (!departmentInfoList) {
          return;
        }
        this.selectedDeptInfo = departmentInfoList.find(
          (d) => String(d.seq) === seq
        );

        this.changeDetectorRef.markForCheck();
        destroySubject.next();
        destroySubject.complete();
      });
  }

  openVideoConference() {
    if (!!this.selectedUserInfos && this.selectedUserInfos.length > 0) {
      this._openVideoConference(
        this.selectedUserInfos.map((item) => Number(item.seq))
      );
    }
  }

  openChatRoom() {
    if (!!this.selectedUserInfos && this.selectedUserInfos.length > 0) {
      this.appChatService.newOpenRoom(
        this.selectedUserInfos.map((item) => item.seq),
        false,
        this.user
      );
    }
  }

  addGroup(): void {
    const dialogRef = this.dialog.open<
      AddGroupDialogComponent,
      AddGroupDialogData,
      AddGroupDialogResult
    >(AddGroupDialogComponent, {
      panelClass: 'mid-create-dialog',
      data: {
        selectedUserList: this.selectedUserInfos
      }
    });
    dialogRef
      .afterClosed()
      .pipe(take(1))
      .subscribe((result) => {
        if (!!result && !!result.choice) {
          // this.selectedUserInfos = [];
        }
      });
  }

  onClickProfileMenu(params: {
    menuType: ProfileMenuType;
    userInfo: UserInfoSS;
  }) {
    switch (params.menuType) {
      case ProfileMenuType.chat:
        this.appChatService.newOpenRoom(
          [String(params.userInfo.seq)],
          false,
          this.user
        );
        break;
      case ProfileMenuType.message:
        break;
      case ProfileMenuType.mobile:
        break;
      case ProfileMenuType.office:
        break;
      case ProfileMenuType.videoConference:
        this._openVideoConference([Number(params.userInfo.seq)]);
        break;
    }
  }

  private _openVideoConference(targetUserSeqs: number[]) {
    const loginSession = this.appAuthenticationService.getLoginSession();
    const req: ConferenceCreateRequest = {
      userSeq: String(this.user.info.seq),
      deviceType: loginSession.deviceType,
      tokenKey: this.loginInfo.tokenString,
      targetUserSeqs
    };
    this.appChatService.openVideoConference(req);
  }
}
