import { Subject, of } from 'rxjs';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject,
  ViewChild,
  ViewContainerRef
} from '@angular/core';

import { Store } from '@ngrx/store';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import {
  UserInfo,
  UserInfoSS,
  UserInfoF,
  UserInfoDN
} from '@ucap/domain-organization';
import { GroupInfoDetail } from '@ucap/domain-group';
import { UserInfo as RoomUserInfo } from '@ucap/domain-chat';

import { MatStepper } from '@angular/material/stepper';
import { I18nService } from '@ucap/ng-i18n';
import { GroupActions } from '@ucap/ng-store-group';
import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult,
  ConfirmDialogComponent,
  ConfirmDialogResult,
  ConfirmDialogData
} from '@ucap/ng-ui/core';

import { take, map, catchError } from 'rxjs/operators';

import { SelectUserDialogType, GroupManageType } from '@app/types';
import { LogService } from '@ucap/ng-logger';
import { AppGroupService } from '@app/services/app-group.service';

export type UserInfoTypes =
  | UserInfo
  | UserInfoSS
  | UserInfoF
  | UserInfoDN
  | RoomUserInfo;

export interface AddGroupDialogData {
  selectedUserList: UserInfoTypes[];
}
export interface AddGroupDialogResult {
  choice: boolean;
}

@Component({
  selector: 'app-dialog-organization-add-group',
  templateUrl: './add-group.dialog.component.html',
  styleUrls: ['./add-group.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class AddGroupDialogComponent implements OnInit, OnDestroy {
  constructor(
    public dialogRef: MatDialogRef<AddGroupDialogData, AddGroupDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: AddGroupDialogData,
    private changeDetectorRef: ChangeDetectorRef,
    private store: Store<any>,
    private i18nService: I18nService,
    private logService: LogService,
    public dialog: MatDialog,
    private appGroupService: AppGroupService
  ) {}

  private ngOnDestroySubject: Subject<void>;

  groupName = '';
  selectedUserList: UserInfoTypes[] = [];
  selectedGroupList: GroupInfoDetail[] = [];
  isDisableBtn = false;

  ngOnInit(): void {
    this.ngOnDestroySubject = new Subject();
    this.selectedUserList = this.data.selectedUserList;
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close({ choice: false });
  }

  checkDisableBtnConfirm(): boolean {
    return false;
  }

  onChangeGroupName(data: { invalid: boolean; groupName: string }) {
    this.groupName = data.groupName;

    if (
      !!this.groupName &&
      this.groupName.trim().localeCompare('') !== 0 &&
      !!data.invalid
    ) {
      this.isDisableBtn = data.invalid;
    } else {
      this.isDisableBtn = false;
    }
  }

  onToggleCheckGroup(data: { checked: boolean; group: GroupInfoDetail }) {
    const i = this.selectedGroupList.findIndex((u) => u.seq === data.group.seq);

    if (data.checked) {
      if (-1 === i) {
        this.selectedGroupList = [...this.selectedGroupList, data.group];
      }
    } else {
      if (-1 < i) {
        this.selectedGroupList = this.selectedGroupList.filter(
          (u) => u.seq !== data.group.seq
        );
      }
    }
  }

  onCancel(): void {
    this.dialogRef.close({ choice: false });
  }
  onConfirm(): void {
    const targetUserSeqs = this.selectedUserList.map((item) => item.seq + '');
    if (!targetUserSeqs || targetUserSeqs.length === 0) {
      this.logService.error('organization', 'not exist selected users.');
      return;
    }

    // validations
    if (
      (!this.groupName && !this.selectedGroupList) ||
      (this.groupName.trim().length === 0 &&
        this.selectedGroupList.length === 0)
    ) {
      const result = this.dialog.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('organization:dialog.title.addGroup'),
          message: this.i18nService.t(
            'organization:dialog.errorAddBuddyForGroup'
          )
        }
      });
      return;
    }

    // Do Action.
    if (!!this.groupName && this.groupName.trim().length > 0) {
      // create group and update room.
      const dialogRef = this.dialog.open<
        ConfirmDialogComponent,
        ConfirmDialogData,
        ConfirmDialogResult
      >(ConfirmDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('organization:label.addGroup'),
          html: this.i18nService.t(
            'organization:dialog.confirmAddBuddyForNewGroup',
            {
              targetGroups: this.groupName
            }
          )
        }
      });
      dialogRef
        .afterClosed()
        .pipe(take(1))
        .subscribe((result) => {
          if (!!result && result.choice) {
            this.appGroupService.createGroup(
              this.groupName,
              this.selectedUserList
            );
            this.dialogRef.close({ choice: true });
          }
        });
    } else if (!!this.selectedGroupList && this.selectedGroupList.length > 0) {
      // existed group update room.
      const dialogRef = this.dialog.open<
        ConfirmDialogComponent,
        ConfirmDialogData,
        ConfirmDialogResult
      >(ConfirmDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('organization:label.addGroup'),
          html: this.i18nService.t(
            'organization:dialog.confirmAddBuddyForGroup',
            {
              targetGroups: this.selectedGroupList
                .map((group) => group.name)
                .join(',')
            }
          )
        }
      });
      dialogRef
        .afterClosed()
        .pipe(take(1))
        .subscribe((result) => {
          if (!!result && result.choice) {
            this.selectedGroupList.forEach((group) => {
              this.appGroupService.addMemberToGroup(
                group,
                this.selectedUserList
              );
              this.dialogRef.close({ choice: true });
            });
          }
        });
    }
  }
}
