import { Subject } from 'rxjs';
import { takeUntil, take, map } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Inject
} from '@angular/core';

import {
  MatDialogRef,
  MAT_DIALOG_DATA,
  MatDialog
} from '@angular/material/dialog';

import { select, Store } from '@ngrx/store';

import { FileUploadItem } from '@ucap/domain-common';
import {
  UserInfoUpdateType,
  User,
  UserInfoSS,
  UserInfoF,
  Company
} from '@ucap/domain-organization';

import { FileProfileSaveRequest } from '@ucap/api-common';

import { LogService } from '@ucap/ng-logger';

import { UserSelector, CompanySelector } from '@ucap/ng-store-organization';
import {
  LoginSelector,
  ConfigurationSelector
} from '@ucap/ng-store-authentication';

import { UserInfoTypes, GroupManageType } from '@app/types';
import { AppGroupService } from '@app/services/app-group.service';
import { AppFileService } from '@app/services/app-file.service';
import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { AppChatService } from '@app/services/app-chat.service';
import { BuddySelector } from '@ucap/ng-store-group';
import {
  EditUserDialogComponent,
  EditUserDialogData,
  EditUserDialogResult
} from '@app/sections/group/dialogs/edit-user.dialog.component';
import { I18nService } from '@ucap/ng-i18n';

import { ConferenceCreateRequest } from '@ucap/api-prompt';
import { LoginInfo, VersionInfo } from '@ucap/domain-authentication';

export interface ProfileDialogData {
  userSeq: string;
}
export interface ProfileDialogResult {}

@Component({
  selector: 'app-dialog-organization-profile',
  templateUrl: './profile.dialog.component.html',
  styleUrls: ['./profile.dialog.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileDialogComponent implements OnInit, OnDestroy {
  constructor(
    private store: Store<any>,
    public dialogRef: MatDialogRef<ProfileDialogData, ProfileDialogResult>,
    @Inject(MAT_DIALOG_DATA) public data: ProfileDialogData,
    private logService: LogService,
    private changeDetectorRef: ChangeDetectorRef,
    public dialog: MatDialog,
    private i18nService: I18nService,
    private appAuthenticationService: AppAuthenticationService,
    private appGroupService: AppGroupService,
    private appChatService: AppChatService,
    private appFileServie: AppFileService
  ) {}

  private ngOnDestroySubject: Subject<void> = new Subject();
  private loginInfo: LoginInfo;
  private user: User;
  private versionInfo: VersionInfo;
  buddyList: UserInfoTypes[];
  companyList: Company[];

  ngOnInit(): void {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(LoginSelector.loginInfo))
      .subscribe((loginInfo) => {
        this.loginInfo = loginInfo;
      });
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(BuddySelector.buddies))
      .subscribe((buddies) => {
        this.buddyList = buddies;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(CompanySelector.companyList)
      )
      .subscribe((companyList) => {
        if (!companyList) {
          return;
        }
        this.companyList = companyList;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClosed(event: MouseEvent): void {
    this.dialogRef.close();
  }

  onOpenChat(userInfo: UserInfoSS) {
    this.appChatService.newOpenRoom(
      [Number(userInfo.seq) as any],
      false,
      this.user
    );
    this.dialogRef.close();
  }
  onSendMessage(userInfo: UserInfoSS) {}
  onSendCall(call: string) {}
  onSendSms(employeeNum: string) {}
  onCreateConference(userSeq: number) {
    const loginSession = this.appAuthenticationService.getLoginSession();
    const req: ConferenceCreateRequest = {
      userSeq: String(this.user.info.seq),
      deviceType: loginSession.deviceType,
      tokenKey: this.loginInfo.tokenString,
      targetUserSeqs: [userSeq]
    };
    this.appChatService.openVideoConference(req);
  }
  onToggleFavorit(params: { userInfo: UserInfoSS; isFavorite: boolean }) {
    this.appGroupService.updateBuddy(params.userInfo, params.isFavorite);
  }
  onToggleBuddy(params: { userInfo: UserInfoSS; isBuddy: boolean }) {
    if (params.isBuddy) {
      // 동료추가
      const dialogRef = this.dialog.open<
        EditUserDialogComponent,
        EditUserDialogData,
        EditUserDialogResult
      >(EditUserDialogComponent, {
        panelClass: 'max-create-dialog',
        data: {
          title: this.i18nService.t('group:dialog.title.addBuddy'),
          type: GroupManageType.Add,
          userInfos: [params.userInfo]
        }
      });
      dialogRef
        .afterClosed()
        .pipe(
          take(1),
          map((result: EditUserDialogResult) => {
            if (!!result) {
              this.appGroupService
                .addBuddy(result)
                .then(() => this.changeDetectorRef.markForCheck());
            }
          })
        )
        .subscribe();
    } else {
      // 동료삭제
      this.appGroupService
        .removeBuddy(params.userInfo)
        .then(() => this.changeDetectorRef.markForCheck());
    }
  }
  onUploadProfileImage(profileImageFileUploadItem: FileUploadItem) {
    const loginSession = this.appAuthenticationService.getLoginSession();

    const profile = {
      userSeq: String(this.user.info.seq),
      deviceType: loginSession.deviceType,
      token: this.loginInfo.tokenString,
      file: profileImageFileUploadItem.file,
      fileUploadItem: profileImageFileUploadItem
    } as FileProfileSaveRequest;

    this.appFileServie.fileProfileSave(
      profile,
      this.versionInfo.profileUploadUrl
    );
  }
  onUpdateIntro(intro: string) {
    this.appGroupService.updateIntro(intro, UserInfoUpdateType.Intro);
  }
  onUpdateNickname(params: { userInfo: UserInfoTypes; nickname: string }) {
    const isBuddy = this.appGroupService.checkBuddy(
      this.buddyList,
      params.userInfo
    );

    if (!isBuddy) {
      return;
    }

    this.appGroupService.updateNickname(
      params.userInfo as UserInfoF,
      params.nickname
    );
  }
}
