import { Observable } from 'rxjs';
import { take, concatMap, map } from 'rxjs/operators';

import { Injectable, Inject } from '@angular/core';

import { Store } from '@ngrx/store';

import { MatDialog } from '@angular/material/dialog';

import { LocaleCode } from '@ucap/domain-common';
import { SSOMode, LoginInfo } from '@ucap/domain-authentication';
import { StatusBulkInfo } from '@ucap/domain-status';

import { PasswordUtil } from '@ucap/pi';

import { NativeService } from '@ucap/native';

import {
  SessionStorageService,
  LocalStorageService
} from '@ucap/ng-web-storage';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { InnerProtocolService } from '@ucap/ng-protocol-inner';
import { AuthenticationProtocolService } from '@ucap/ng-protocol-authentication';
import { LoginActions } from '@ucap/ng-store-authentication';
import { PresenceActions } from '@ucap/ng-store-organization';

import {
  ConfirmDialogComponent,
  ConfirmDialogData,
  ConfirmDialogResult
} from '@ucap/ng-ui/core';

import { LoginSession } from '@app/models/login-session';
import { UserStore } from '@app/models/user-store';

import { AppKey } from '@app/types/app-key.type';

import { environment } from '@environments';

@Injectable({
  providedIn: 'root'
})
export class AppAuthenticationService {
  constructor(
    private innerProtocolService: InnerProtocolService,
    private authenticationProtocolService: AuthenticationProtocolService,
    private sessionStorageService: SessionStorageService,
    private localStorageService: LocalStorageService,
    private store: Store<any>,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private matDialog: MatDialog
  ) {}

  getUserStore(): UserStore {
    return this.localStorageService.encGet<UserStore>(
      AppKey.UserStore,
      environment.productConfig.localEncriptionKey
    );
  }

  getUserStore$(): Observable<UserStore> {
    return this.localStorageService.encGet$<UserStore>(
      AppKey.UserStore,
      environment.productConfig.localEncriptionKey
    );
  }

  setUserStore(userStore: UserStore) {
    const oldUserStore = this.getUserStore();

    this.localStorageService.encSet<UserStore>(
      AppKey.UserStore,
      {
        ...oldUserStore,
        ...userStore
      },
      environment.productConfig.localEncriptionKey
    );
  }

  getLoginSession(): LoginSession {
    return this.sessionStorageService.get<LoginSession>(AppKey.LoginSession);
  }

  getLoginSession$(): Observable<LoginSession> {
    return this.sessionStorageService.get$<LoginSession>(AppKey.LoginSession);
  }

  setLoginSession(loginSession: LoginSession) {
    const oldLoginSession = this.getLoginSession();
    this.sessionStorageService.set<LoginSession>(AppKey.LoginSession, {
      ...oldLoginSession,
      ...loginSession
    });
  }

  loggedIn(): boolean {
    const loginSession = this.getLoginSession();
    return !!loginSession && !!loginSession.loginId;
  }

  loggedIn$(): Observable<boolean> {
    return this.getLoginSession$().pipe(
      map((loginSession) => !!loginSession && !!loginSession.loginId)
    );
  }

  useAutoLogin(): UserStore {
    const userStore = this.getUserStore();
    return !!userStore &&
      !!userStore.settings &&
      !!userStore.settings.general &&
      userStore.settings.general.autoLogin
      ? userStore
      : null;
  }

  useAutoLogin$(): Observable<UserStore> {
    return this.getUserStore$().pipe(
      map((userStore) =>
        !!userStore &&
        !!userStore.settings &&
        !!userStore.settings.general &&
        userStore.settings.general.autoLogin
          ? userStore
          : null
      )
    );
  }

  async postWebLogin(
    loginSession: LoginSession,
    rememberMe: boolean,
    autoLogin: boolean
  ) {
    const prevLoginSession = this.getLoginSession();

    const newLoginSession = {
      ...prevLoginSession,
      ...loginSession,
      localeCode: LocaleCode.Korean
    };
    const encLoginPw = PasswordUtil.encrypt(newLoginSession.loginPw);

    this.setLoginSession({
      ...newLoginSession,
      initPw: newLoginSession.loginId === newLoginSession.loginPw,
      loginPw: encLoginPw
    });

    let userStore = this.getUserStore();

    if (!userStore) {
      userStore = {
        idleCheckTime: 10,
        settings: {
          ...environment.productConfig.defaultSettings,
          chat: {
            ...environment.productConfig.defaultSettings.chat,
            downloadPath: await this.nativeService.file_path(
              'documents',
              environment.productConfig.file.defaultDownloadFolder
            )
          }
        }
      };

      if (!!environment.productConfig.defaultSettings.general.autoLaunch) {
        // this.nativeService.changeAutoLaunch(
        //   environment.productConfig.defaultSettings.general.autoLaunch
        // );
      }
    }

    userStore = {
      ...userStore,
      companyGroupType: loginSession.companyGroupType,
      companyCode: loginSession.companyCode,
      loginId: loginSession.loginId,
      loginPw: loginSession.loginPw
    };

    if (rememberMe || autoLogin) {
      userStore = {
        ...userStore,
        rememberMe
      };
      userStore.settings.general.autoLogin = autoLogin;
    }

    this.setUserStore(userStore);

    this.sessionStorageService.remove(AppKey.LogoutSession);
  }

  createSession(): Promise<{
    loginInfo: LoginInfo;
    loginSession: LoginSession;
  }> {
    return new Promise<{
      loginInfo: LoginInfo;
      loginSession: LoginSession;
    }>(async (resolve, reject) => {
      const loginSession = this.getLoginSession();

      const networkInfo = await this.nativeService.platform_networkInfo();
      const localIp =
        !!networkInfo && 0 < networkInfo.length && !!networkInfo[0].ip
          ? networkInfo[0].ip
          : '';
      const localMac =
        !!networkInfo && 0 < networkInfo.length && !!networkInfo[0].mac
          ? networkInfo[0].mac
          : '';

      this.innerProtocolService
        .conn({})
        .pipe(
          take(1),
          concatMap((connRes) => {
            return this.authenticationProtocolService
              .login({
                loginId: loginSession.loginId,
                loginPw: loginSession.loginPw,
                deviceType: loginSession.deviceType,
                deviceId: ' ',
                token: '',
                localeCode: loginSession.localeCode,
                pushId: ' ',
                companyCode: loginSession.companyCode,
                passwordEncodingType: 1,
                clientVersion: '',
                reconnect: false,
                ip: localIp,
                hostName: localMac,
                ssoMode: SSOMode.AUTH,
                userSpecificInformation: 'PRO_000482',
                andId: '',
                andPushRefreshYn: ''
              })
              .pipe(take(1));
          })
        )
        .subscribe(
          (result) => {
            this.store.dispatch(
              LoginActions.loginSuccess({
                loginInfo: result.loginInfo,
                loginSession
              })
            );
            // this.store.dispatch(
            //   PresenceActions.bulkInfo({
            //     divCd: 'myBulk',
            //     userSeqs: [loginInfo.userSeq]
            //   })
            // );
            // 임시 코드
            this.store.dispatch(
              PresenceActions.bulkInfoSuccess({
                statusBulkInfoList: [
                  {
                    conferenceStatus: 'X',
                    imessengerStatus: 'X',
                    mobileConferenceStatus: 'X',
                    mobileStatus: 'X',
                    pcStatus: 'O',
                    phoneStatus: 'X',
                    statusMessage: '.',
                    terminalStatus: 'TERMINAL_STATE_UNKNOWN',
                    terminalStatusNumber: 0,
                    userSeq: String(result.loginInfo.userSeq)
                  } as StatusBulkInfo
                ]
              })
            );
            resolve({
              loginSession,
              loginInfo: result.loginInfo
            });
          },
          (error) => {
            reject(error);
          }
        );
    });
  }

  exitConfirm(): Promise<boolean> {
    return new Promise<boolean>((resolve, reject) => {
      const dialogRef = this.matDialog.open<
        ConfirmDialogComponent,
        ConfirmDialogData,
        ConfirmDialogResult
      >(ConfirmDialogComponent, {
        panelClass: '',
        data: {
          title: '',
          message: ''
        }
      });

      dialogRef
        .afterClosed()
        .pipe(take(1))
        .subscribe((result) => {
          resolve(result.choice);
        });
    });
  }

  logoutConfirm(): Promise<boolean> {
    return new Promise<boolean>((resolve, reject) => {
      const dialogRef = this.matDialog.open<
        ConfirmDialogComponent,
        ConfirmDialogData,
        ConfirmDialogResult
      >(ConfirmDialogComponent, {
        panelClass: '',
        data: {
          title: '',
          message: ''
        }
      });

      dialogRef
        .afterClosed()
        .pipe(take(1))
        .subscribe((result) => {
          resolve(result.choice);
        });
    });
  }

  logout() {
    this.sessionStorageService.remove(AppKey.LoginSession);

    let userStore = this.getUserStore();
    if (!!userStore) {
      userStore = {
        ...userStore,
        settings: {
          ...userStore.settings,
          general: {
            ...userStore.settings.general,
            autoLogin: false
          }
        }
      };

      this.setUserStore(userStore);
    }
  }
}
