import { Observable, of, forkJoin, Subject } from 'rxjs';
import { take, map, catchError, takeUntil } from 'rxjs/operators';

import { Injectable, NgZone, Inject, OnDestroy } from '@angular/core';
import { Router } from '@angular/router';

import { Store } from '@ngrx/store';
import { Dictionary } from '@ngrx/entity';

import { MatDialog } from '@angular/material/dialog';

import { FileUtil } from '@ucap/core';
import { DeviceType, FileUploadItem } from '@ucap/domain-common';
import { User } from '@ucap/domain-organization';
import {
  RoomInfo,
  RoomType,
  EventType,
  Info,
  MassTranslationEventJson
} from '@ucap/domain-chat';

import { StatusCode } from '@ucap/api';
import {
  MassTalkSaveRequest,
  FileTalkSaveResponse,
  FileTalkSaveRequest,
  MassTalkDownloadRequest,
  TransMassTalkDownloadRequest,
  TransMassTalkDownloadResponse
} from '@ucap/api-common';
import {
  ConferenceCreateRequest,
  ConferenceJoinRequest,
  ModuleConfig
} from '@ucap/api-prompt';
import {
  ConferenceCreateRequest as WebexConferenceCreateRequest,
  ConferenceJoinRequest as WebexConferenceJoinRequest,
  ModuleConfig as WebexModuleConfig
} from '@ucap/api-webex';

import { OpenRequest, Open3Request, ExitRequest } from '@ucap/protocol-room';
import { SendRequest as SendEventRequest } from '@ucap/protocol-event';
import { NativeService } from '@ucap/native';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';
import { LocalStorageService } from '@ucap/ng-web-storage';
import { CommonApiService } from '@ucap/ng-api-common';
import { ConferenceService } from '@ucap/ng-api-prompt';
import { ConferenceService as WebexConferenceService } from '@ucap/ng-api-webex';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';

import {
  RoomUserMap,
  RoomUserShortMap
} from '@ucap/ng-store-chat/lib/store/room/state';
import { ChattingActions, RoomActions } from '@ucap/ng-store-chat';

import {
  TranslatePipe as OrganizationTranslate,
  TranslateService
} from '@ucap/ng-ui-organization';

import { environment } from '@environments';

import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult,
  ConfirmDialogComponent,
  ConfirmDialogData,
  ConfirmDialogResult,
  ClipboardService
} from '@ucap/ng-ui/core';

import { StickerFilesInfo, KEY_STICKER_HISTORY } from '@ucap/ng-core';

import {
  TransDetailDialogComponent,
  TransDetailDialogData,
  TransDetailDialogResult
} from '@app/sections/chat/dialogs/trans-detail.dialog.component';
import {
  FileViewerDialogComponent,
  FileViewerDialogData,
  FileViewerDialogResult
} from '@app/sections/chat/dialogs/file-viewer.dialog.component';
import { AppRoomActions } from '@app/store/actions';

import { AppUiService } from './app-ui.service';
import { LoginInfo } from '@ucap/domain-authentication';

@Injectable({
  providedIn: 'root'
})
export class AppChatService implements OnDestroy {
  defaultProfileImage = 'assets/images/ico/img_nophoto.svg';
  defaultProfileImageMulti = 'assets/images/ico/img_nophoto_multiple.svg';

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private i18nService: I18nService,
    private dialog: MatDialog,
    private localStorageService: LocalStorageService,
    private store: Store<any>,
    private appUiService: AppUiService,
    private commonApiService: CommonApiService,
    private conferenceService: ConferenceService,
    private webexConferenceService: WebexConferenceService,
    private clipboardService: ClipboardService,
    private router: Router,
    private logService: LogService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private ngZone: NgZone
  ) {}

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  subscribe(): void {
    this.nativeService
      .chat_onOpen$()
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((roomId) => {
        this.ngZone.run(() => {
          this.openRoombyRoomId(roomId);
        });
      });
  }

  /**
   * 방이름 생성.
   * cf) 방이름이 지정되어 있다면 방이름 리턴, 지정되어 있지 않으면 방참여인원의 이름 조합.
   */
  getRoomNameByDic(
    organizationTranslate: OrganizationTranslate,
    user: User,
    roomInfo: RoomInfo,
    roomUsersDictionary?: Dictionary<RoomUserMap>,
    roomUsersShortDictionary?: Dictionary<RoomUserShortMap>
  ): string {
    if (!roomInfo) {
      return '';
    }

    let roomName = '...';

    switch (roomInfo.roomType) {
      case RoomType.Mytalk:
        roomName = 'MyTalk';
        break;
      case RoomType.Single:
        {
          const roomUsers = this.getRoomUserList(
            user,
            roomInfo.roomId,
            roomUsersDictionary,
            roomUsersShortDictionary
          );

          if (roomUsers.users.length > 0) {
            roomName = roomUsers.users
              .map((item) => organizationTranslate.transform(item, 'name'))
              .sort((a, b) => (a < b ? -1 : a > b ? 1 : 0))
              .join(',');
          } else {
            roomName = this.i18nService.t('chat:room.noRoomUser');
          }
        }
        break;
      default:
        {
          const roomUsers = this.getRoomUserList(
            user,
            roomInfo.roomId,
            roomUsersDictionary,
            roomUsersShortDictionary
          );
          const users = roomUsers.users.filter(
            (userInfo) => !!userInfo.isJoinRoom
          );
          const existUsers = roomUsers.existUsers;

          const curRoomName = roomInfo.roomName;
          if (!!curRoomName && curRoomName.trim().length > 0) {
            // Exist RoomName.
            roomName = curRoomName;
          } else {
            if (users.length > 0) {
              roomName = users
                .map((item) => organizationTranslate.transform(item, 'name'))
                .sort((a, b) => (a < b ? -1 : a > b ? 1 : 0))
                .join(',');
              // roomName = organizationTranslate.transform(users, 'name', ',');
            } else {
              if (existUsers) {
                roomName = this.i18nService.t('chat:room.noRoomUser');
              }
            }
          }
        }
        break;
    }

    return roomName;
  }
  getRoomName(
    organizationTranslate: OrganizationTranslate,
    user: User,
    roomInfo: RoomInfo,
    roomUsersMap?: RoomUserMap,
    roomUsersShortMap?: RoomUserShortMap
  ): string {
    if (!roomInfo) {
      return '';
    }

    let roomName = '...';

    switch (roomInfo.roomType) {
      case RoomType.Mytalk:
        roomName = 'MyTalk';
        break;
      case RoomType.Single:
        {
          const roomUsers = this.getRoomUserList01(
            user,
            roomUsersMap,
            roomUsersShortMap
          );
          const curRoomName = roomInfo.roomName;
          if (!!curRoomName && curRoomName.trim().length > 0) {
            // Exist RoomName.
            roomName = curRoomName;
          } else {
            if (roomUsers.users.length > 0) {
              roomName = roomUsers.users
                .map((item) => organizationTranslate.transform(item, 'name'))
                .sort((a, b) => (a < b ? -1 : a > b ? 1 : 0))
                .join(',');
            } else {
              roomName = this.i18nService.t('chat:room.noRoomUser');
            }
          }
        }
        break;
      default:
        {
          const roomUsers = this.getRoomUserList01(
            user,
            roomUsersMap,
            roomUsersShortMap
          );
          const users = roomUsers.users.filter(
            (userInfo) => !!userInfo.isJoinRoom
          );
          const existUsers = roomUsers.existUsers;

          const curRoomName = roomInfo.roomName;
          if (!!curRoomName && curRoomName.trim().length > 0) {
            // Exist RoomName.
            roomName = curRoomName;
          } else {
            if (users.length > 0) {
              roomName = users
                .map((item) => organizationTranslate.transform(item, 'name'))
                .sort((a, b) => (a < b ? -1 : a > b ? 1 : 0))
                .join(',');
              // roomName = organizationTranslate.transform(users, 'name', ',');
            } else {
              if (existUsers) {
                roomName = this.i18nService.t('chat:room.noRoomUser');
              }
            }
          }
        }
        break;
    }

    return roomName;
  }

  /**
   * 방 프로필 이미지 생성.
   * cf) 방 참여인원의 프로필을 리턴.
   * 없으면, '' 빈값을 리턴하고 ucapImage pipe 에서 default Image 로 처리.
   */
  getRoomProfileImageByDic(
    user: User,
    roomInfo: RoomInfo,
    roomUsersDictionary?: Dictionary<RoomUserMap>,
    roomUsersShortDictionary?: Dictionary<RoomUserShortMap>
  ): string {
    let roomImage = '';

    if (!!roomInfo) {
      switch (roomInfo.roomType) {
        case RoomType.Mytalk:
          {
            if (!!user && !!user.info) {
              roomImage = user.info.profileImageFile;
            }
          }
          break;
        case RoomType.Multi:
          {
            // default image
          }
          break;
        default:
          {
            const roomUsers = this.getRoomUserList(
              user,
              roomInfo.roomId,
              roomUsersDictionary,
              roomUsersShortDictionary
            );
            if (
              !!roomUsers &&
              !!roomUsers.existUsers &&
              roomUsers.users.length > 0
            ) {
              roomImage = roomUsers.users[0].profileImageFile;
            }
          }
          break;
      }
    }

    return roomImage;
  }
  getRoomProfileImage(
    user: User,
    roomInfo: RoomInfo,
    roomUsersMap?: RoomUserMap,
    roomUsersShortMap?: RoomUserShortMap
  ): string {
    let roomImage = '';

    if (!!roomInfo) {
      switch (roomInfo.roomType) {
        case RoomType.Mytalk:
          {
            if (!!user && !!user.info) {
              roomImage = user.info.profileImageFile;
            }
          }
          break;
        case RoomType.Multi:
          {
            // default image
          }
          break;
        default:
          {
            const roomUsers = this.getRoomUserList01(
              user,
              roomUsersMap,
              roomUsersShortMap
            );
            if (
              !!roomUsers &&
              !!roomUsers.existUsers &&
              roomUsers.users.length > 0
            ) {
              roomImage = roomUsers.users[0].profileImageFile;
            }
          }
          break;
      }
    }

    return roomImage;
  }

  /**
   * 방 인원 정보 수집.
   * cf) roomUserShort, roomUser 데이터가 쌍을 이루는데 수집방법에 따라 short 만 존재할 수 있어, 수집할 수 있는 방인원을 리턴.
   * roomUser 가 detail 정보라 우선함.
   */
  getRoomUserList(
    user: User,
    roomId: string,
    roomUsersDictionary?: Dictionary<RoomUserMap>,
    roomUsersShortDictionary?: Dictionary<RoomUserShortMap>
  ) {
    let users = [];
    let existUsers = false;

    const roomUsers = !!roomUsersDictionary
      ? roomUsersDictionary[roomId]
      : undefined;
    const roomUsersShort = !!roomUsersShortDictionary
      ? roomUsersShortDictionary[roomId]
      : undefined;

    if (!!roomUsers && roomUsers.userInfos.length > 0) {
      existUsers = true;
      users = roomUsers.userInfos.filter(
        (userInfo) => String(userInfo.seq) !== String(user.info.seq)
      );
    } else if (!!roomUsersShort && roomUsersShort.userInfos.length > 0) {
      existUsers = true;
      users = roomUsersShort.userInfos.filter(
        (userInfo) => String(userInfo.seq) !== String(user.info.seq)
      );
    }

    return {
      existUsers,
      users
    };
  }
  getRoomUserList01(
    user: User,
    roomUsersMap?: RoomUserMap,
    roomUsersShortMap?: RoomUserShortMap
  ) {
    let users = [];
    let existUsers = false;

    if (!!roomUsersMap && roomUsersMap.userInfos.length > 0) {
      existUsers = true;
      users = roomUsersMap.userInfos.filter(
        (userInfo) => String(userInfo.seq) !== String(user.info.seq)
      );
    } else if (!!roomUsersShortMap && roomUsersShortMap.userInfos.length > 0) {
      existUsers = true;
      users = roomUsersShortMap.userInfos.filter(
        (userInfo) => String(userInfo.seq) !== String(user.info.seq)
      );
    }

    return {
      existUsers,
      users
    };
  }

  /**
   * 스티커 히스토리 관리 in localstorage.
   * cf) 스티커 전송 성공분만 처리.
   */
  setStickerHistory(sticker: StickerFilesInfo) {
    const history = this.localStorageService.get<string[]>(KEY_STICKER_HISTORY);

    if (!!history && history.length > 0) {
      const stickers: string[] = [];
      [sticker.index, ...history.filter((hist) => hist !== sticker.index)].map(
        (s, i) => {
          if (i < 10) {
            stickers.push(s);
          }
        }
      );

      this.localStorageService.set<string[]>(KEY_STICKER_HISTORY, stickers);
    } else {
      this.localStorageService.set<string[]>(KEY_STICKER_HISTORY, [
        sticker.index
      ]);
    }
  }

  /**
   * Event Send Protocol Service.
   */
  protected sendEvent(
    senderSeq: string,
    roomId: string,
    eventType: EventType,
    sentMessage: string
  ) {
    this.store.dispatch(
      ChattingActions.send({
        senderSeq,
        req: {
          roomId,
          eventType,
          sentMessage
        } as SendEventRequest
      })
    );
  }

  /** Send Normal message */
  sendMessageOfNormal(senderSeq: string, roomId: string, sentMessage: string) {
    this.sendEvent(senderSeq, roomId, EventType.Character, sentMessage);
  }

  /** Send Masstext message */
  sendMessageOfMassText(
    loginInfo: LoginInfo,
    user: User,
    deviceType: DeviceType,
    roomId: string,
    sentMessage: string
  ) {
    const req: MassTalkSaveRequest = {
      userSeq: String(user.info.seq),
      deviceType,
      token: loginInfo.tokenString,
      content: sentMessage.replace(/"/g, '\\"'),
      roomId
    };

    this.commonApiService
      .massTalkSave(req)
      .pipe(
        take(1),
        map((res) => {
          if (res.statusCode === StatusCode.Success) {
            this.sendEvent(
              String(user.info.seq),
              roomId,
              EventType.MassText,
              res.returnJson
            );
          } else {
            this.logService.error(
              `commonApiService] massTalkSave ${res?.errorMessage}`
            );
          }
        }),
        catchError((error) => of({ error }))
      )
      .subscribe();
  }

  copyFromContentText(text: string) {
    if (this.clipboardService.copyFromContent(text)) {
      alert('복사완료');
      // this.snackBarService.open(
      //   this.translateService.instant('common:clipboard.results.copied'),
      //   '',
      //   {
      //     duration: 3000,
      //     verticalPosition: 'top',
      //     horizontalPosition: 'center'
      //   }
      // );
    }
  }

  massTextDownload(massTalkDownloadReq: MassTalkDownloadRequest) {
    this.commonApiService
      .massTalkDownload(massTalkDownloadReq)
      .pipe(take(1))
      .subscribe((res) => {
        this.copyFromContentText(res.content);
      });
  }

  async sendMessageOfSticker(
    senderSeq: string,
    roomId: string,
    selectedSticker: StickerFilesInfo,
    sentMessage: string
  ) {
    // Validation
    if (
      !!sentMessage &&
      sentMessage.trim().length > environment.productConfig.chat.masstextLength
    ) {
      const result = await this.dialog.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('chat:errors.label'),
          message: this.i18nService.t('chat:errors.maxLengthOfMassText', {
            maxLength: environment.productConfig.chat.masstextLength
          })
        }
      });
      return;
    }

    // Send
    this.sendEvent(
      senderSeq,
      roomId,
      EventType.Sticker,
      JSON.stringify({
        name: '스티커',
        file: selectedSticker.index,
        chat: !!sentMessage ? sentMessage.trim() : ''
      })
    );

    // set sticker's history in localstorage
    this.setStickerHistory(selectedSticker);
  }

  /** Send Translation message */
  sendMessageOfTranslate(
    senderSeq: string,
    roomId: string,
    eventType: EventType,
    sentMessage: string
  ) {
    this.sendEvent(senderSeq, roomId, eventType, sentMessage);
  }

  /**
   * Mass Translation message detail
   */
  massTranslationMessageDetail(
    req: TransMassTalkDownloadRequest,
    message: Info<MassTranslationEventJson>,
    contentsType: string,
    roomId: string
  ) {
    this.commonApiService
      .transMassTalkDownload(req)
      .pipe(take(1))
      .subscribe((res: TransMassTalkDownloadResponse) => {
        let contents = '';

        if (res.statusCode === StatusCode.Success) {
          contents = contentsType === 'T' ? res.translation : res.original;
        } else {
          contents =
            contentsType === 'T'
              ? message.sentMessageJson.translation
              : message.sentMessageJson.original;
        }

        this.dialog.open<
          TransDetailDialogComponent,
          TransDetailDialogData,
          TransDetailDialogResult
        >(TransDetailDialogComponent, {
          panelClass: 'mid-create-dialog',
          data: {
            contents,
            message,
            roomId,
            defaultProfileImage: this.defaultProfileImage
          }
        });
        return;
      });
  }
  /** Send AttachFile message */
  sendMessageOfAttachFile(
    loginInfo: LoginInfo,
    user: User,
    deviceType: DeviceType,
    roomId: string,
    fileUploadItems: FileUploadItem[]
  ): Promise<boolean> {
    const executor = async (
      resolve: (value?: boolean | PromiseLike<boolean>) => void,
      reject: (reason?: any) => void
    ) => {
      const allObservables: Observable<FileTalkSaveResponse>[] = [];

      for (const fileUploadItem of fileUploadItems) {
        let thumbnail: File;
        if (
          -1 !==
          [
            '3gp',
            'avi',
            'm4v',
            'mkv',
            'mov',
            'mp4',
            'mpeg',
            'mpg',
            'rv',
            'ts',
            'webm',
            'wmv'
          ].indexOf(FileUtil.getExtension(fileUploadItem.file.name))
        ) {
          try {
            thumbnail = await FileUtil.thumbnail(fileUploadItem.file);
          } catch (err) {
            this.logService.error('video thumbnail error.', err);
          }
          this.logService.debug('thumbnail', thumbnail);
        }

        const req: FileTalkSaveRequest = {
          userSeq: String(user.info.seq),
          deviceType,
          token: loginInfo.tokenString,
          roomId,
          file: fileUploadItem.file,
          fileName: fileUploadItem.file.name,
          thumb: thumbnail,
          fileUploadItem
        };

        allObservables.push(
          this.commonApiService.fileTalkSave(req).pipe(
            map((res) => {
              if (!res) {
                return;
              }
              if (StatusCode.Success === res.statusCode) {
                return res;
              } else {
                throw res;
              }
            })
          )
        );
      }

      this.ngZone.run(() => {
        forkJoin(allObservables)
          .pipe(take(1))
          .subscribe(
            (resList) => {
              for (const res of resList) {
                this.store.dispatch(
                  ChattingActions.send({
                    senderSeq: String(user.info.seq),
                    req: {
                      roomId,
                      eventType: EventType.File,
                      sentMessage: JSON.stringify(res.returnJson)
                    } as SendEventRequest
                  })
                );
              }
            },
            (error) => {
              this.logService.debug('onFileSelected error', error);
              reject(error);
            },
            () => {
              resolve(true);
            }
          );
      });
    };

    return new Promise<boolean>(executor);
  }

  /**
   * Open Dialog For 'Exit Room'.
   */
  exitRoomDialog(roomInfo: RoomInfo): void {
    if (!roomInfo) {
      return;
    }

    const dialogRef = this.dialog.open<
      ConfirmDialogComponent,
      ConfirmDialogData,
      ConfirmDialogResult
    >(ConfirmDialogComponent, {
      panelClass: 'min-create-dialog',
      data: {
        title: this.i18nService.t('chat:label.exitFromRoom'),
        html: this.i18nService.t('chat:dialog.confirmExitFromRoom')
      }
    });

    dialogRef
      .afterClosed()
      .pipe(take(1))
      .subscribe((result) => {
        if (!!result && !!result.choice) {
          this.store.dispatch(
            RoomActions.del({
              req: {
                roomId: roomInfo.roomId
              } as ExitRequest
            })
          );
        }
      });
  }

  openRoombyRoomId(roomId: string): void {
    this.appUiService.closeLeftSidenavOnNarrowMode();
    this.router.navigate(
      [
        'chat',
        {
          outlets: { content: 'chatroom' }
        }
      ],
      {
        queryParams: { roomId }
      }
    );
  }

  newOpenRoom(userSeqs: string[], isTimerRoom: boolean, user?: User) {
    // maxChatRoomUser check.
    const maxChatRoomUser = environment.productConfig.chat.maxChatRoomUser;
    if (!!userSeqs && userSeqs.length >= (maxChatRoomUser || 300)) {
      this.dialog.open<
        AlertDialogComponent,
        AlertDialogData,
        AlertDialogResult
      >(AlertDialogComponent, {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('chat:errors.label'),
          html: this.i18nService.t('chat:errors.maxCountOfRoomMemberWith', {
            maxCount: maxChatRoomUser
          })
        }
      });
      return;
    }
    if (!userSeqs || userSeqs.length === 0) {
      this.logService.error(
        `error open room user size by ${
          environment.productConfig.chat.maxChatRoomUser || 300
        }`
      );
      return;
    }

    // popup all close.
    this.dialog.closeAll();
    this.appUiService.closeLeftSidenavOnNarrowMode();
    isTimerRoom = isTimerRoom || false;

    if (!!isTimerRoom) {
      /** Timer Room Open. */
      const req: Open3Request = {
        divCd: 'OPROOMT',
        roomName: '',
        isTimerRoom,
        timerRoomInterval:
          environment.productConfig.chat.timerRoomDefaultInterval,
        userSeqs
      };
      this.store.dispatch(RoomActions.createTimer({ req }));
    } else {
      /** Normal Room Open */
      let req: OpenRequest;
      if (
        userSeqs.length === 1 &&
        !!user &&
        String(userSeqs[0]) === String(user.info.seq)
      ) {
        // MyTalk Open.
        req = {
          divCd: 'OPMYTALK',
          userSeqs: [user.talkWithMeBotSeq + '']
        };
      } else {
        req = {
          divCd: 'OPROOM',
          userSeqs
        };
      }
      this.store.dispatch(RoomActions.create({ req }));
    }
  }

  openFileviwer(data: FileViewerDialogData): void {
    this.dialog.open<
      FileViewerDialogComponent,
      FileViewerDialogData,
      FileViewerDialogResult
    >(FileViewerDialogComponent, {
      // position: {
      //   top: '50px'
      // },
      maxWidth: '100vw',
      maxHeight: '100vh',
      // height: 'calc(100% - 50px)',
      height: '100%',
      width: '100%',
      hasBackdrop: false,
      panelClass: 'app-dialog-full',
      data
    });
  }

  setHistoryRoomId(historyRoomId: string | null) {
    this.store.dispatch(AppRoomActions.setHistoryRoomId({ historyRoomId }));
  }

  openVideoConference(
    req: ConferenceCreateRequest | WebexConferenceCreateRequest
  ) {
    /** CASE :: others */
    // const urlCreateVideoConference = this.conferenceService.urlForWebexConferenceCreate(req);

    /** CASE :: webex */
    const urlCreateVideoConference = this.webexConferenceService.urlForConferenceCreate(
      req
    );
    if (
      !!urlCreateVideoConference &&
      urlCreateVideoConference.trim().length > 0
    ) {
      this.nativeService.platform_openDefaultBrowser(urlCreateVideoConference);
    }
  }

  joinVideoConference(req: ConferenceJoinRequest | WebexConferenceJoinRequest) {
    /** CASE :: others */
    // const urlJoinVideoConference = this.conferenceService.urlForWebexConferencejoin(req);

    /** CASE :: webex */
    const urlJoinVideoConference = this.webexConferenceService.urlForConferencejoin(
      req
    );
    if (!!urlJoinVideoConference && urlJoinVideoConference.trim().length > 0) {
      this.nativeService.platform_openDefaultBrowser(urlJoinVideoConference);
    }
  }
}
