import { of } from 'rxjs';
import { take, map, finalize, catchError } from 'rxjs/operators';

import { Injectable, Inject, NgZone } from '@angular/core';

import { Store } from '@ngrx/store';

import { MatDialog } from '@angular/material/dialog';

import { FileUtil, MimeUtil } from '@ucap/core';

import { FileDownloadItem } from '@ucap/domain-common';
import { UserInfoUpdateType, User } from '@ucap/domain-organization';
import { LoginSession, LoginInfo } from '@ucap/domain-authentication';
import { FileEventJson } from '@ucap/domain-chat';

import { NativeService } from '@ucap/native';
import { StatusCode } from '@ucap/api';
import { FileProfileSaveRequest } from '@ucap/api-common';

import { DownCheckRequest } from '@ucap/protocol-file';

import { StringUtil } from '@ucap/ng-core';
import { I18nService } from '@ucap/ng-i18n';
import { CommonApiService } from '@ucap/ng-api-common';
import { LogService } from '@ucap/ng-logger';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { FileProtocolService } from '@ucap/ng-protocol-file';

import { UserActions } from '@ucap/ng-store-organization';
import { ChattingActions } from '@ucap/ng-store-chat';

import {
  AlertDialogComponent,
  AlertDialogData,
  AlertDialogResult
} from '@ucap/ng-ui/core';

import { UserStore } from '@app/models/user-store';

import { AppAuthenticationService } from './app-authentication.service';

@Injectable({
  providedIn: 'root'
})
export class AppFileService {
  constructor(
    private dialog: MatDialog,
    private store: Store<any>,
    private i18nService: I18nService,
    private commonApiService: CommonApiService,
    private fileProtocolService: FileProtocolService,
    private appAuthenticationService: AppAuthenticationService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private ngZone: NgZone,
    private logService: LogService
  ) {}

  async validUploadFile(
    fileList: FileList,
    fileAllowSize: number = 50
  ): Promise<boolean> {
    let valid = true;

    // File size check.
    // tslint:disable-next-line: prefer-for-of
    for (let i = 0; i < fileList.length; i++) {
      const file = fileList[i];
      if (file.size > fileAllowSize * 1024 * 1024) {
        const msg = this.i18nService.t('common:file.errors.oversize', {
          maxSize: fileAllowSize
        });
        this._openAlert(msg);

        valid = false;
        return valid;
      }
    }

    // Extention check.
    const checkExt = this.commonApiService.acceptableExtensionForFileTalk(
      FileUtil.getExtensions(fileList)
    );
    if (!!checkExt) {
      const msg = this.i18nService.t('common:file.errors.notSupporedType', {
        supporedType: checkExt.join(',')
      });
      this._openAlert(msg);
      // this.snackBarService.openFromComponent<
      //   AlertSnackbarComponent,
      //   AlertSnackbarData
      // >(AlertSnackbarComponent, {
      //   duration: 1000,
      //   verticalPosition: 'bottom',
      //   horizontalPosition: 'center',
      //   data: {
      //     html: this.translateService.instant(
      //       'common:file.errors.notSupporedType',
      //       {
      //         supporedType: checkExt.join(',')
      //       }
      //     )
      //   }
      // });
      valid = false;
      return valid;
    }

    // Fake media file check.
    const fakeMedia = await this.commonApiService.checkInvalidMediaMimeForFileTalkForFileList(
      fileList
    );
    if (!!fakeMedia) {
      const msg = this.i18nService.t('common:file.errors.notAcceptableMime', {
        supporedType: fakeMedia.join(',')
      });
      this._openAlert(msg);

      valid = false;
      return valid;
    }

    return valid;
  }

  validUploadProfileFile(
    file: File,
    fileAllowSize: number = 50
  ): Promise<boolean> {
    return new Promise<boolean>(async (resolve, reject) => {
      try {
        if (file.size > fileAllowSize * 1024 * 1024) {
          const msg = this.i18nService.t('common:file.errors.oversize', {
            maxSize: fileAllowSize
          });
          this._openAlert(msg);

          resolve(false);
        }
        const fakeMedia = await this.commonApiService.mediaFiles([file]);
        if (!fakeMedia) {
          const extension = FileUtil.getExtension(
            file.name
          ).toLocaleLowerCase();
          const msg = this.i18nService.t(
            'common:file.errors.notAcceptableMime',
            {
              supporedType: extension
            }
          );
          this._openAlert(msg);

          resolve(false);
        }
        resolve(true);
      } catch (error) {
        reject(error);
      }
    });
  }

  fileProfileSave(request: FileProfileSaveRequest, profileUploadUrl: string) {
    this.validUploadProfileFile(request.fileUploadItem.file)
      .then((res) => {
        if (!!res) {
          this._fileProfileSave(request, profileUploadUrl);
        }
      })
      .catch((resone) => {
        this.logService.error(resone);
      });
  }
  private _fileProfileSave(
    request: FileProfileSaveRequest,
    profileUploadUrl: string
  ) {
    this.commonApiService
      .fileProfileSave(request, profileUploadUrl)
      .pipe(
        take(1),
        map((res) => {
          console.log(res);
          if (!res) {
            return;
          }
          if (StatusCode.Success === res.statusCode) {
            // this.onUpdateProfile(res.profileURL);
            return res;
          } else {
            throw res;
          }
        }),
        finalize(() => {
          setTimeout(() => {
            request.fileUploadItem.uploadingProgress$ = undefined;
          }, 1000);
        })
      )
      .subscribe(
        (res) => {
          let infoImg: string;

          if (!!res.profileSubDir && res.profileSubDir.trim() !== '') {
            infoImg = res.profileSubDir;
          } else {
            infoImg = StringUtil.getProfileSubDir(res.profileURL);
          }
          this.store.dispatch(
            UserActions.modifyInfo({
              req: {
                type: UserInfoUpdateType.Image,
                info: infoImg
              }
            })
          );
        },
        (error) => {
          this.logService.error(error);
          // this.snackBarService.open(
          //   this.translateService.instant(
          //     'profile.errors.failToChangeProfileImage'
          //   ),
          //   '',
          //   {
          //     duration: 3000,
          //     verticalPosition: 'bottom'
          //   }
          // );
        }
      );
  }
  saveFile(
    fileParams: {
      fileInfo: FileEventJson;
      fileDownloadItem: FileDownloadItem;
      type: string;
      fileName: string;
      fileDownloadUrl?: string;
      savePath?: string;
    },
    loginInfo: LoginInfo,
    user: User,
    loginSession: LoginSession
  ) {
    const fileName = fileParams.fileInfo.fileName;
    const fileDownloadItem = fileParams.fileDownloadItem;
    const fileDownloadUrl = fileParams.fileDownloadUrl;
    const attachmentsSeq = fileParams.fileInfo.attachmentSeq;
    const savePath = fileParams.savePath;
    let downloadPath;

    const userStore: UserStore = this.appAuthenticationService.getUserStore();

    if (!!userStore) {
      downloadPath = userStore.settings.chat.downloadPath;
    }

    const fileTalkDownloadError = (reason: any) => {
      this.logService.warn(reason);
      // this.snackBarService.openFromComponent<
      //   AlertSnackbarComponent,
      //   AlertSnackbarData
      // >(AlertSnackbarComponent, {
      //   data: {
      //     html: this.translateService.instant('common:file.errors.failToSave'),
      //     buttonText: this.translateService.instant('common:file.errors.label')
      //   }
      // });
    };

    this.ngZone.run(() => {
      this.commonApiService
        .fileTalkDownload(
          {
            userSeq: String(user.info.seq),
            deviceType: loginSession.deviceType,
            token: loginInfo.tokenString,
            attachmentsSeq: attachmentsSeq + '',
            fileDownloadItem
          },
          fileDownloadUrl
        )
        .pipe(
          take(1),
          map((rawBlob) => {
            const mimeType = MimeUtil.getMimeFromExtension(
              FileUtil.getExtension(fileName)
            );
            const blob = rawBlob.slice(0, rawBlob.size, mimeType);

            FileUtil.fromBlobToBuffer(blob)
              .then((buffer) => {
                /** download check */
                this.store.dispatch(
                  ChattingActions.fileDownCheck({
                    req: { seq: attachmentsSeq } as DownCheckRequest
                  })
                );

                this.nativeService
                  .file_save(buffer, fileName, mimeType, savePath)
                  .then((filePath) => {
                    if (!!filePath) {
                      //                 const snackBarRef = this.snackBarService.open(
                      //                   this.translateService.instant(
                      //                     'common:file.results.savedToPath',
                      //                     {
                      //                       path: filePath
                      //                     }
                      //                   ),
                      //                   this.translateService.instant('common:file.open'),
                      //                   {
                      //                     duration: 3000,
                      //                     verticalPosition: 'bottom',
                      //                     horizontalPosition: 'center'
                      //                   }
                      //                 );
                      //                 snackBarRef.onAction().subscribe(() => {
                      //                   snackBarRef.dismiss();
                      //                   this.ngZone.runOutsideAngular(() => {
                      //                     this.nativeService
                      //                       .openTargetItem(filePath)
                      //                       .catch((reason) => {
                      //                         this.logger.warn(reason);
                      //                       });
                      //                   });
                      //                 });
                    } else {
                      fileTalkDownloadError('fail');
                    }
                  })
                  .catch((reason) => {
                    fileTalkDownloadError(reason);
                  });
              })
              .catch((reason) => {
                fileTalkDownloadError(reason);
              });
          }),
          finalize(() => {
            if (!!fileDownloadItem) {
              setTimeout(() => {
                fileDownloadItem.downloadingProgress$ = undefined;
              }, 1000);
            }
          }),
          catchError((error) => of(error))
        )
        .subscribe();
    });
  }

  private _openAlert(msg: string) {
    this.dialog.open<AlertDialogComponent, AlertDialogData, AlertDialogResult>(
      AlertDialogComponent,
      {
        panelClass: 'min-create-dialog',
        data: {
          title: this.i18nService.t('common:file.errors.title'),
          html: msg
        }
      }
    );
  }
}
