import { of, Subject } from 'rxjs';
import {
  tap,
  withLatestFrom,
  catchError,
  finalize,
  take,
  map,
  takeUntil
} from 'rxjs/operators';

import { Injectable, Inject, NgZone, OnDestroy } from '@angular/core';

import { MatDialog } from '@angular/material/dialog';

import { Store, select } from '@ngrx/store';
import { Dictionary } from '@ngrx/entity';

import { LocaleCode, NotificationMethod } from '@ucap/domain-common';
import { UserInfo } from '@ucap/domain-organization';
import { EventType } from '@ucap/domain-chat';

import {
  NativeService,
  WindowState,
  NotificationType,
  NotificationRequest
} from '@ucap/native';
import {
  SSVC_TYPE_LOGOUT_RES,
  LogoutNotification,
  SSVC_TYPE_LOGOUT_REMOTE_NOTI,
  LogoutRemoteNotification
} from '@ucap/protocol-authentication';
import {
  SSVC_TYPE_EVENT_SEND_RES,
  SSVC_TYPE_EVENT_SEND_NOTI,
  SSVC_TYPE_EVENT_READ_RES,
  SSVC_TYPE_EVENT_READ_NOTI,
  SSVC_TYPE_EVENT_CANCEL_NOTI,
  SSVC_TYPE_EVENT_DEL_RES,
  SendNotification,
  ReadNotification,
  CancelNotification,
  DelNotification as EventDelNotification,
  ReadRequest as EventReadRequest
} from '@ucap/protocol-event';
import {
  SSVC_TYPE_INFO_USER_NOTI,
  UserNotification
} from '@ucap/protocol-info';
import {
  SSVC_TYPE_GROUP_UPD_RES2,
  SSVC_TYPE_GROUP_ADD_RES,
  SSVC_TYPE_GROUP_DEL_RES,
  UpdateNotification as GroupUpdateNotification,
  AddNotification as GroupAddNotification,
  DelNotification as GroupDelNotification
} from '@ucap/protocol-group';
import {
  SSVC_TYPE_BUDDY_UPD_RES,
  SSVC_TYPE_BUDDY_ADD_RES,
  SSVC_TYPE_BUDDY_DEL_RES,
  UpdateNotification as BuddyUpdateNotification,
  AddNotification as BuddyAddNotification,
  DelNotification as BuddyDelNotification
} from '@ucap/protocol-buddy';
import {
  SSVC_TYPE_ROOM_INVITE_RES,
  SSVC_TYPE_ROOM_INVITE_NOTI,
  SSVC_TYPE_ROOM_UPD_RES,
  SSVC_TYPE_ROOM_EXIT_RES,
  SSVC_TYPE_ROOM_EXIT_NOTI,
  SSVC_TYPE_ROOM_EXIT_FORCING_RES,
  SSVC_TYPE_ROOM_EXIT_FORCING_NOTI,
  SSVC_TYPE_ROOM_FONT_UPD_NOTI,
  InviteNotification,
  UpdateNotification as RoomUpdateNotification,
  ExitNotification,
  ExitForcingResponse,
  ExitForcingNotification,
  UpdateFontNotification
} from '@ucap/protocol-room';
import {
  SSVC_TYPE_STATUS_NOTI,
  StatusNotification
} from '@ucap/protocol-status';
import {
  SSVC_TYPE_OPTION_REG_UPD_RES,
  RegUpdateNotification
} from '@ucap/protocol-option';
import {
  SSVC_TYPE_UMG_NOTI,
  SSVC_TYPE_UMG_DELETE_NOTI,
  UmgNotiNotification,
  UmgDeleteNotiNotification
} from '@ucap/protocol-umg';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';
import {
  LocalStorageService,
  SessionStorageService
} from '@ucap/ng-web-storage';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { AuthenticationProtocolService } from '@ucap/ng-protocol-authentication';
import { EventProtocolService } from '@ucap/ng-protocol-event';
import { InfoProtocolService } from '@ucap/ng-protocol-info';
import { RoomProtocolService } from '@ucap/ng-protocol-room';
import { GroupProtocolService } from '@ucap/ng-protocol-group';
import { BuddyProtocolService } from '@ucap/ng-protocol-buddy';
import { QueryProtocolService } from '@ucap/ng-protocol-query';
import { StatusProtocolService } from '@ucap/ng-protocol-status';
import { OptionProtocolService } from '@ucap/ng-protocol-option';
import { UmgProtocolService } from '@ucap/ng-protocol-umg';

import {
  RoomSelector,
  RoomActions,
  ChattingActions,
  ChattingSelector,
  ChatUtil
} from '@ucap/ng-store-chat';
import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import {
  CommonActions,
  UserSelector,
  PresenceActions
} from '@ucap/ng-store-organization';

import { UserStore } from '@app/models/user-store';

import { AppAuthenticationService } from './app-authentication.service';

@Injectable()
export class AppNotificationService implements OnDestroy {
  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private authenticationProtocolService: AuthenticationProtocolService,
    private eventProtocolService: EventProtocolService,
    private infoProtocolService: InfoProtocolService,
    private roomProtocolService: RoomProtocolService,
    private groupProtocolService: GroupProtocolService,
    private buddyProtocolService: BuddyProtocolService,
    private queryProtocolService: QueryProtocolService,
    private statusProtocolService: StatusProtocolService,
    private optionProtocolService: OptionProtocolService,
    private umgProtocolService: UmgProtocolService,

    private localStorageService: LocalStorageService,
    private sessionStorageService: SessionStorageService,
    private appAuthenticationService: AppAuthenticationService,

    private store: Store<any>,
    private ngZone: NgZone,
    private dialog: MatDialog,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private i18nService: I18nService,
    private logService: LogService
  ) {}

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  public subscribe(): void {
    this.authenticationProtocolService.logoutNotification$
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        tap((notiOrRes) => {
          switch (notiOrRes.SSVC_TYPE) {
            case SSVC_TYPE_LOGOUT_RES:
              {
                const res = notiOrRes as LogoutNotification;
                this.logService.debug(
                  'Notification::authenticationProtocolService::LogoutResponse',
                  res
                );

                // this.sessionStorageService.set<LogoutInfo>(KEY_LOGOUT_INFO, {
                //   personLogout: true,
                //   reasonCode: res.reasonCode,
                //   ip: res.ip,
                //   mac: res.mac
                // } as LogoutInfo);
              }
              break;
            case SSVC_TYPE_LOGOUT_REMOTE_NOTI:
              {
                const noti = notiOrRes as LogoutRemoteNotification;
                this.logService.debug(
                  'Notification::authenticationProtocolService::LogoutRemoteNotification',
                  noti
                );

                // this.sessionStorageService.set<LogoutInfo>(KEY_LOGOUT_INFO, {
                //   personLogout: true,
                //   reasonCode: ServerErrorCode.ERRCD_FORCE_INIT,
                //   forceType: noti.requestDeviceType
                // } as LogoutInfo);
              }
              break;
            default:
              break;
          }
          this.dialog.closeAll();

          // this.store.dispatch(AuthenticationStore.loginRedirect());
        })
      )
      .subscribe();

    this.eventProtocolService.notification$
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        withLatestFrom(
          this.store.pipe(select(UserSelector.user)),
          this.store.pipe(select(ConfigurationSelector.versionInfo)),
          this.store.pipe(select(ChattingSelector.activeRoomId)),
          this.store.pipe(select(RoomSelector.rooms)),
          this.store.pipe(select(RoomSelector.roomUsers)),
          this.store.pipe(
            select(
              (state: any) =>
                state.group.buddy.buddies.entities as Dictionary<UserInfo>
            )
          )
        ),
        tap(
          ([
            notiOrRes,
            user,
            versionInfo,
            activeRoomId,
            roomList,
            roomUsers,
            buddyList
          ]) => {
            switch (notiOrRes.SSVC_TYPE) {
              case SSVC_TYPE_EVENT_SEND_RES:
              case SSVC_TYPE_EVENT_SEND_NOTI:
                {
                  const noti = notiOrRes as SendNotification;
                  const roomId = noti.roomId;
                  const eventInfo = noti.info;
                  const trgtRoom = roomList.find(
                    (roomInfo) => roomInfo.roomId === roomId
                  );

                  this.logService.debug(
                    'Notification::eventProtocolService::SendNotification',
                    noti
                  );

                  // Add Event..
                  this.store.dispatch(
                    ChattingActions.addEvent({
                      roomId,
                      info: eventInfo
                    })
                  );

                  // Unread Count..
                  let doReadRequest = false;
                  // Get Window state.
                  const windowState = this.nativeService.window_onState$()
                    .value;
                  const isWindowFocusState = this.nativeService.window_onFocus$()
                    .value;

                  // Is activate event read request.
                  if (isWindowFocusState !== false) {
                    // Exclude window unfocused..
                    if (
                      windowState !== WindowState.Minimized &&
                      windowState !== WindowState.Hidden
                    ) {
                      if (!!activeRoomId && activeRoomId === roomId) {
                        // Only Active room.
                        doReadRequest = true;
                      }
                    }
                  }

                  if (doReadRequest) {
                    this.store.dispatch(
                      ChattingActions.read({
                        roomId,
                        lastReadSeq: noti.info.seq
                      } as EventReadRequest)
                    );
                  } else {
                    //   // not opened room :: unread count increased
                    if (notiOrRes.SSVC_TYPE === SSVC_TYPE_EVENT_SEND_RES) {
                      /**
                       * 다른 디바이스에서 대화를 송신 할경우 RES 가 noti 로 유입될 수 있다.
                       * 이때 unread count 를 중가하지 않는다.
                       */
                    } else {
                      if (
                        !!trgtRoom &&
                        noti.info.type !== EventType.Join &&
                        noti.info.type !== EventType.Exit &&
                        noti.info.type !== EventType.RenameRoom &&
                        noti.info.type !== EventType.ForcedExit
                      ) {
                        this.store.dispatch(
                          RoomActions.updateUnreadCount({
                            roomId,
                            noReadCnt: trgtRoom.noReadCnt + 1
                          })
                        );
                      }
                    }
                  }

                  // notification..
                  if (notiOrRes.SSVC_TYPE === SSVC_TYPE_EVENT_SEND_NOTI) {
                    let doNoti = true;

                    // 방별 알림이 꺼져 있으면 노티 안함. > 우선순위 최상위.
                    if (!!trgtRoom && !trgtRoom.receiveAlarm) {
                      doNoti = false;
                    }

                    // Is activate event read request.
                    if (isWindowFocusState !== false) {
                      // Exclude window unfocused..
                      if (
                        windowState !== WindowState.Minimized &&
                        windowState !== WindowState.Hidden
                      ) {
                        if (!!activeRoomId && activeRoomId === roomId) {
                          // Only Active room.
                          doNoti = false;
                        }
                      }
                    }
                    // // 포커스 아웃일때 무조건 노티.
                    // // Case 1 : 단순 포커스 아웃.
                    // // Case 2 : hidden 시 포커스 인 상태이지만 위에서 필터링 됨.
                    // if (!isWindowFocusState) {
                    //   doNoti = true;
                    // }

                    if (doNoti) {
                      const userStore: UserStore = this.appAuthenticationService.getUserStore();
                      if (userStore.settings.notification.use) {
                        if (
                          userStore.settings.notification.method ===
                          NotificationMethod.Sound
                        ) {
                          const audio = new Audio(
                            'assets/sounds/messageAlarm.mp3'
                          );
                          audio.play();
                        } else {
                          const contents = ChatUtil.convertFinalEventMessage(
                            noti.eventType,
                            noti.info.sentMessageJson
                          );
                          if (!!contents) {
                            const notiReq: NotificationRequest = {
                              type: NotificationType.Event,
                              seq: roomId,
                              title: this.i18nService.t(
                                'common:notification.titleChatEventArrived'
                              ),
                              contents,
                              image: '',
                              useSound: [
                                NotificationMethod.Sound,
                                NotificationMethod.SoundAndAlert
                              ].some(
                                (n) =>
                                  n === userStore.settings.notification.method
                              )
                                ? true
                                : false,
                              displayTime:
                                (userStore.settings.notification
                                  .alertExposureTime || 5) * 1000
                            };
                            // Sender Info setting
                            // STEP 1 >> In buddy group.
                            let senderInfo: any = buddyList[noti.SENDER_SEQ];
                            // STEP 2 >> In Current Room Users.
                            if (!senderInfo) {
                              if (!!roomUsers && roomUsers.length > 0) {
                                const userMap = roomUsers.find(
                                  (item) => item.roomId === roomId
                                );
                                if (!!userMap) {
                                  senderInfo = userMap.userInfos.find(
                                    (item) =>
                                      item.seq + '' === noti.SENDER_SEQ + ''
                                  );
                                }
                              }
                            }
                            // STEP 3 >> user protocol
                            if (!senderInfo) {
                              this.queryProtocolService
                                .dataUser({
                                  divCd: 'OPENNOTI',
                                  seq: Number(noti.SENDER_SEQ),
                                  senderCompanyCode: user.info.companyCode,
                                  senderEmployeeType: user.info.employeeType
                                })
                                .pipe(
                                  take(1),
                                  map((res) => {
                                    if (!!res && !!res.userInfo) {
                                      senderInfo = res.userInfo;
                                      let name = senderInfo.name;
                                      let grade = senderInfo.grade;
                                      switch (
                                        this.i18nService.currentLng.toUpperCase()
                                      ) {
                                        case LocaleCode.English:
                                          name = senderInfo.nameEn;
                                          grade = senderInfo.gradeEn;
                                          break;
                                        case LocaleCode.Chinese:
                                          name = senderInfo.nameCn;
                                          grade = senderInfo.gradeCn;
                                          break;
                                      }
                                      notiReq.title = this.i18nService.t(
                                        'common:notification.titleChatEventArrivedByUser',
                                        {
                                          userInfo: !!grade
                                            ? `${name} ${grade}`
                                            : name
                                        }
                                      );
                                      // Image set.
                                      if (!!senderInfo.profileImageFile) {
                                        notiReq.image = `${versionInfo.profileRoot}${senderInfo.profileImageFile}`;
                                      }
                                    }
                                  }),
                                  catchError((error) => {
                                    return of();
                                  }),
                                  finalize(() => {
                                    this.nativeService.app_showNotify(notiReq);
                                  })
                                )
                                .subscribe();
                            } else {
                              // Sender Info setting.
                              // name set
                              let name = senderInfo.name;
                              let grade = senderInfo.grade;
                              switch (
                                this.i18nService.currentLng.toUpperCase()
                              ) {
                                case LocaleCode.English:
                                  name = senderInfo.nameEn;
                                  grade = senderInfo.gradeEn;
                                  break;
                                case LocaleCode.Chinese:
                                  name = senderInfo.nameCn;
                                  grade = senderInfo.gradeCn;
                                  break;
                              }
                              notiReq.title = this.i18nService.t(
                                'common:notification.titleChatEventArrivedByUser',
                                {
                                  userInfo: !!grade ? `${name} ${grade}` : name
                                }
                              );
                              // Image set.
                              if (!!senderInfo.profileImageFile) {
                                notiReq.image = `${versionInfo.profileRoot}${senderInfo.profileImageFile}`;
                              }
                              // express noti popup
                              this.nativeService.app_showNotify(notiReq);
                            }
                          }
                        }
                      }
                    }
                  }
                }
                break;
              case SSVC_TYPE_EVENT_READ_RES:
              case SSVC_TYPE_EVENT_READ_NOTI:
                {
                  // 대화방 unread count 처리.
                  const noti = notiOrRes as ReadNotification;
                  this.logService.debug(
                    'Notification::eventProtocolService::ReadNotification',
                    noti
                  );

                  // roomUsers lastReadEventSeq update.
                  this.store.dispatch(ChattingActions.readSuccess(noti));

                  // roomInfo noReadCount update.
                  if (notiOrRes.SSVC_TYPE === SSVC_TYPE_EVENT_READ_RES) {
                    this.store.dispatch(
                      RoomActions.updateUnreadCount({
                        roomId: noti.roomId
                      })
                    );
                  }
                }
                break;
              case SSVC_TYPE_EVENT_CANCEL_NOTI:
                {
                  const noti = notiOrRes as CancelNotification;
                  this.logService.debug(
                    'Notification::eventProtocolService::CancelNotification',
                    noti
                  );
                  this.store.dispatch(
                    ChattingActions.cancelNotification({
                      noti
                    })
                  );
                }
                break;
              case SSVC_TYPE_EVENT_DEL_RES:
                {
                  const noti = notiOrRes as EventDelNotification;
                  this.logService.debug(
                    'Notification::eventProtocolService::DelNotification',
                    noti
                  );
                  this.store.dispatch(
                    ChattingActions.delNotification({
                      noti
                    })
                  );
                }
                break;
              default:
                break;
            }
          }
        )
      )
      .subscribe();

    this.infoProtocolService.notification$
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        tap((notiOrRes) => {
          switch (notiOrRes.SSVC_TYPE) {
            case SSVC_TYPE_INFO_USER_NOTI:
              {
                const noti = notiOrRes as UserNotification;
                this.logService.debug(
                  'Notification::infoProtocolService::UserNotification',
                  noti
                );
                this.store.dispatch(
                  CommonActions.userNotification({
                    noti
                  })
                );
              }
              break;
            default:
              break;
          }
        })
      )
      .subscribe();

    this.groupProtocolService.notification$
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        tap((notiOrRes) => {
          switch (notiOrRes.SSVC_TYPE) {
            case SSVC_TYPE_GROUP_UPD_RES2:
              {
                const noti = notiOrRes as GroupUpdateNotification;
                this.logService.debug(
                  'Notification::groupProtocolService::GroupUpdateNotification',
                  noti
                );
                // this.store.dispatch(
                //   SyncStore.group2({
                //     syncDate
                //   })
                // );
              }
              break;
            case SSVC_TYPE_GROUP_ADD_RES:
              {
                const noti = notiOrRes as GroupAddNotification;
                this.logService.debug(
                  'Notification::groupProtocolService::GroupAddNotification',
                  noti
                );
                // this.store.dispatch(SyncStore.createGroupSuccess(noti));
              }
              break;
            case SSVC_TYPE_GROUP_DEL_RES:
              {
                const noti = notiOrRes as GroupDelNotification;
                this.logService.debug(
                  'Notification::groupProtocolService::GroupDelNotification',
                  noti
                );
                // this.store.dispatch(SyncStore.delGroupSuccess(noti));
              }
              break;

            default:
              break;
          }
        })
      )
      .subscribe();

    this.buddyProtocolService.notification$
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        tap((notiOrRes) => {
          switch (notiOrRes.SSVC_TYPE) {
            case SSVC_TYPE_BUDDY_UPD_RES:
              {
                const noti = notiOrRes as BuddyUpdateNotification;
                this.logService.debug(
                  'Notification::groupProtocolService::BuddyUpdateNotification',
                  noti
                );
                // this.store.dispatch(SyncStore.updateBuddySuccess(noti));
              }
              break;
            case SSVC_TYPE_BUDDY_ADD_RES:
              {
                const noti = notiOrRes as BuddyAddNotification;
                this.logService.debug(
                  'Notification::groupProtocolService::BuddyAddNotification',
                  noti
                );
                // this.store.dispatch(SyncStore.buddy2({ syncDate }));
              }
              break;
            case SSVC_TYPE_BUDDY_DEL_RES:
              {
                const noti = notiOrRes as BuddyDelNotification;
                this.logService.debug(
                  'Notification::groupProtocolService::BuddyDelNotification',
                  noti
                );
                // this.store.dispatch(SyncStore.delBuddySuccess(noti));
              }
              break;

            default:
              break;
          }
        })
      )
      .subscribe();

    this.roomProtocolService.notification$
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        withLatestFrom(this.store.pipe(select(UserSelector.user))),
        tap(([notiOrRes, user]) => {
          switch (notiOrRes.SSVC_TYPE) {
            case SSVC_TYPE_ROOM_INVITE_RES:
            case SSVC_TYPE_ROOM_INVITE_NOTI:
              {
                const noti = notiOrRes as InviteNotification;
                this.logService.debug(
                  'Notification::roomProtocolService::InviteNotification',
                  noti
                );
                this.store.dispatch(
                  RoomActions.inviteNotification({
                    noti,
                    localeCode: LocaleCode.Korean
                  })
                );
              }
              break;
            case SSVC_TYPE_ROOM_UPD_RES:
              {
                const noti = notiOrRes as RoomUpdateNotification;
                this.logService.debug(
                  'Notification::roomProtocolService::RoomUpdateNotification',
                  noti
                );

                if (noti.SENDER_SEQ === String(user.info.seq)) {
                  // Update Res.
                  this.store.dispatch(
                    RoomActions.updateSuccess({
                      res: {
                        roomId: noti.roomId,
                        roomName:
                          noti.roomName.trim().length === 0
                            ? ' '
                            : noti.roomName.trim(),
                        receiveAlarm: noti.receiveAlarm,
                        syncAll: false // ignore params
                      }
                    })
                  );
                } else {
                  // Update Room Name Noti.
                  this.store.dispatch(
                    RoomActions.updateRoomName({
                      res: {
                        roomId: noti.roomId,
                        roomName:
                          noti.roomName.trim().length === 0
                            ? ' '
                            : noti.roomName.trim(),
                        receiveAlarm: noti.receiveAlarm,
                        syncAll: false // ignore params
                      }
                    })
                  );
                }
              }
              break;
            case SSVC_TYPE_ROOM_EXIT_RES:
            case SSVC_TYPE_ROOM_EXIT_NOTI:
              {
                const noti = notiOrRes as ExitNotification;
                this.logService.debug(
                  'Notification::roomProtocolService::ExitNotification',
                  noti
                );

                this.store.dispatch(
                  RoomActions.exitNotification({
                    roomId: noti.roomId,
                    senderSeq: noti.SENDER_SEQ
                  })
                );
              }
              break;
            case SSVC_TYPE_ROOM_EXIT_FORCING_RES:
            case SSVC_TYPE_ROOM_EXIT_FORCING_NOTI:
              {
                const res = notiOrRes as ExitForcingResponse;
                this.logService.debug(
                  'Notification::roomProtocolService::ExitForcingNotification RES',
                  res
                );

                this.store.dispatch(RoomActions.expelNotification({ res }));
              }
              break;
            case SSVC_TYPE_ROOM_FONT_UPD_NOTI:
              {
                const noti = notiOrRes as UpdateFontNotification;
                this.logService.debug(
                  'Notification::roomProtocolService::UpdateFontNotification',
                  noti
                );
                // this.store.dispatch(
                //   RoomStore.updateFontNotification({
                //     noti
                //   })
                // );
              }
              break;
            default:
              break;
          }
        })
      )
      .subscribe();

    this.statusProtocolService.notification$
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        tap((notiOrRes) => {
          switch (notiOrRes.SSVC_TYPE) {
            case SSVC_TYPE_STATUS_NOTI:
              {
                const noti = notiOrRes as StatusNotification;
                this.logService.debug(
                  'Notification::statusProtocolService::StatusNotification',
                  noti
                );
                this.store.dispatch(
                  PresenceActions.statusNotification({
                    noti
                  })
                );
              }
              break;
            default:
              break;
          }
        })
      )
      .subscribe();

    this.optionProtocolService.notification$
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        tap((notiOrRes) => {
          switch (notiOrRes.SSVC_TYPE) {
            case SSVC_TYPE_OPTION_REG_UPD_RES:
              {
                const noti = notiOrRes as RegUpdateNotification;
                this.logService.debug(
                  'Notification::optionProtocolService::RegUpdateNotification',
                  noti
                );

                // const appUserInfo: AppUserInfo = this.localStorageService.encGet<
                //   AppUserInfo
                // >(KEY_APP_USER_INFO, environment.customConfig.appKey);

                // const modifiedSettings: Settings = clone(appUserInfo.settings);
                // // 모바일에서 해당 값만 수정함.
                // modifiedSettings.notification.receiveForMobile =
                //   noti.mobileNotification;
                // appUserInfo.settings = modifiedSettings;

                // this.localStorageService.encSet<AppUserInfo>(
                //   KEY_APP_USER_INFO,
                //   appUserInfo,
                //   environment.customConfig.appKey
                // );
              }
              break;
            default:
              break;
          }
        })
      )
      .subscribe();

    this.umgProtocolService.notification$
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        tap((notiOrRes) => {
          switch (notiOrRes.SSVC_TYPE) {
            case SSVC_TYPE_UMG_NOTI:
              {
                const noti = notiOrRes as UmgNotiNotification;
                this.logService.debug(
                  'Notification::umgProtocolService::UmgNotiNotification',
                  noti
                );

                // // unreadCount refresh..
                // this.store.dispatch(MessageStore.retrieveUnreadCount({}));

                // // Receive Message List refresh..
                // this.store.dispatch(
                //   MessageStore.retrieveMessage({
                //     messageType: MessageType.Receive
                //   })
                // );

                // // notification..
                // const appUserInfo = this.localStorageService.encGet<
                //   AppUserInfo
                // >(KEY_APP_USER_INFO, environment.customConfig.appKey);

                // if (appUserInfo.settings.notification.use) {
                //   if (
                //     appUserInfo.settings.notification.method ===
                //     NotificationMethod.Sound
                //   ) {
                //     const audio = new Audio('assets/sounds/messageAlarm.mp3');
                //     audio.play();
                //   } else {
                //     const notiReq: NotificationRequest = {
                //       type: NotificationType.Message,
                //       seq: noti.keyId,
                //       title: this.translateService.instant(
                //         'notification.titleMessageArrived'
                //       ),
                //       contents: noti.text,
                //       image: noti.senderInfo.profileImageFile,
                //       useSound: [
                //         NotificationMethod.Sound,
                //         NotificationMethod.SoundAndAlert
                //       ].some(
                //         (n) => n === appUserInfo.settings.notification.method
                //       )
                //         ? true
                //         : false,
                //       displayTime:
                //         appUserInfo.settings.notification.alertExposureTime *
                //         1000
                //     };
                //     this.nativeService.notify(notiReq);
                //   }
                // }

                // // direct open detail
                // if (appUserInfo.settings.notification.receiveForMessage) {
                //   this.store.dispatch(
                //     MessageStore.detailMessage({
                //       messageType: MessageType.Receive,
                //       msgId: Number(noti.keyId)
                //     })
                //   );
                // }
              }
              break;
            case SSVC_TYPE_UMG_DELETE_NOTI:
              {
                const noti = notiOrRes as UmgDeleteNotiNotification;
                this.logService.debug(
                  'Notification::umgProtocolService::UmgDeleteNotiNotification',
                  noti
                );

                // // Remove one Receive Message
                // if (!!noti && !!noti.keyId) {
                //   // clear badge in left navi
                //   this.store.dispatch(MessageStore.retrieveUnreadCount({}));

                //   // delete message in receive message list
                //   this.store.dispatch(
                //     deleteMessageSuccess({
                //       messageType: MessageType.Receive,
                //       msgList: [
                //         {
                //           msgId: Number(noti.keyId)
                //         }
                //       ]
                //     })
                //   );
                // }
              }
              break;

            default:
              break;
          }
        })
      )
      .subscribe();
  }
}
