import { Injectable, Inject } from '@angular/core';

import { Store } from '@ngrx/store';

import * as detectBrowser from 'detect-browser';

import { DeviceType, DesktopType } from '@ucap/domain-common';
import { NativeService, NativeType } from '@ucap/native';

import { I18nService } from '@ucap/ng-i18n';
import { LogService } from '@ucap/ng-logger';
import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { PingProtocolService } from '@ucap/ng-protocol-ping';
import { DateService } from '@ucap/ng-ui/date';
import { TranslateService } from '@ucap/ng-ui-organization';
import { ProtocolService } from '@ucap/ng-protocol';
import { LoginActions } from '@ucap/ng-store-authentication';

import { environment } from '@environments';

import { AppAuthenticationService } from './app-authentication.service';
import { AppNotificationService } from './app-notification.service';
import { GroupOpenInfo } from '@app/models/group-open-info';
import { AppNativeService } from './app-native.service';
import { AppChatService } from './app-chat.service';

@Injectable()
export class AppService {
  readonly companyGroupCode = environment.companyConfig.companyGroupCode;
  bannedGroupNames: string[] = [];

  constructor(
    private i18nService: I18nService,
    private translateService: TranslateService,
    private dateService: DateService,
    private protocolService: ProtocolService,
    private pingProtocolService: PingProtocolService,
    private appAuthenticationService: AppAuthenticationService,
    private appNativeService: AppNativeService,
    private appNotificationService: AppNotificationService,
    private appChatService: AppChatService,
    private logService: LogService,
    private store: Store<any>,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService
  ) {}

  initialize(): Promise<void[]> {
    const initApp = new Promise<void>(async (resolve, reject) => {
      try {
        this.nativeService.window_onFocus$().subscribe((focus) => {
          console.log('window_onFocus$', focus);
        });

        resolve();
      } catch (error) {
        reject(error);
      }
    });

    const initSession = new Promise<void>(async (resolve, reject) => {
      try {
        let loginSession = this.appAuthenticationService.getLoginSession();

        loginSession = loginSession || {};

        let deviceType: DeviceType;
        let desktopType: DesktopType;

        switch (await this.nativeService.platform_nativeType()) {
          case NativeType.Browser:
            deviceType = DeviceType.Web;
            break;
          case NativeType.Electron:
            deviceType = DeviceType.PC;
            break;
          default:
            break;
        }

        const groupInfo = {
          lastGroupSeq: 0,
          groupSeqs: []
        } as GroupOpenInfo;

        const info = detectBrowser.detect();
        if (info.os.startsWith('Windows')) {
          desktopType = DesktopType.Windows;
        } else if (info.os.startsWith('Mac OS')) {
          desktopType = DesktopType.MacOS;
        } else if (info.os.startsWith('Linux')) {
          desktopType = DesktopType.Linux;
        } else {
        }

        this.appAuthenticationService.setLoginSession({
          ...loginSession,
          deviceType,
          desktopType,
          companyGroupCode: this.companyGroupCode,
          alive: false,
          groupInfo
        });

        this.dateService.setDefaultTimezone('Asia/Seoul');
        this.dateService.use('Asia/Seoul');

        resolve();
      } catch (error) {
        reject(error);
      }
    });

    const initI18n = new Promise<void>(async (resolve, reject) => {
      const backends = environment.productConfig.i18next.useBackends;
      if (!!backends && 0 < backends.length) {
        for (const backend of backends) {
          this.i18nService.use(backend);
        }
      }

      this.i18nService
        .init(environment.productConfig.i18next.options)
        .then(() => {
          const langs = ['en', 'ko'];
          const bannedNameKeys = [
            'category.favorite',
            'category.default'
            // 'category.myDept'
          ];

          langs.map((lang) => {
            this.i18nService.changeLanguage(lang);
            bannedNameKeys.map((key) => {
              const tempResource = this.i18nService.getResource(
                lang,
                'group',
                key
              );

              this.bannedGroupNames.push(tempResource);
            });
          });

          this.dateService.setLocale(this.i18nService.currentLng);

          resolve();
        })
        .catch((reason) => {
          reject(reason);
        });
    });

    const initOrganization = new Promise<void>(async (resolve, reject) => {
      try {
        this.translateService.setDefaultLang('ko');
        this.translateService.use('ko');
        resolve();
      } catch (error) {
        reject(error);
      }
    });

    const initChat = new Promise<void>(async (resolve, reject) => {
      try {
        this.appChatService.subscribe();
        resolve();
      } catch (error) {
        reject(error);
      }
    });

    const initProtocol = new Promise<void>(async (resolve, reject) => {
      try {
        this.protocolService.connected$.subscribe((connected) => {
          if (undefined === connected) {
            return;
          }
          if (connected) {
            this.appAuthenticationService
              .createSession()
              .then((result) => {
                this.store.dispatch(
                  LoginActions.sessionCreated({
                    loginInfo: result.loginInfo,
                    loginSession: result.loginSession
                  })
                );
              })
              .catch((reason) => {
                this.logService.error('createSession failed', reason);
              });
          } else {
            this.store.dispatch(
              LoginActions.sessionDestroyed({ error: 'disconnected' })
            );
          }
        });
        resolve();
      } catch (error) {
        reject(error);
      }
    });

    const initNative = new Promise<void>(async (resolve, reject) => {
      try {
        this.appNativeService.subscribe();
        resolve();
      } catch (error) {
        reject(error);
      }
    });

    const initNotification = new Promise<void>(async (resolve, reject) => {
      try {
        this.appNotificationService.subscribe();
        resolve();
      } catch (error) {
        reject(error);
      }
    });

    return Promise.all([
      initApp,
      initI18n,
      initNative,
      initProtocol,
      initNotification,
      initSession,
      initOrganization,
      initChat
    ]);
  }
}
