import { Subject } from 'rxjs';
import { take, takeUntil } from 'rxjs/operators';

import { Component, OnInit, OnDestroy, ViewChild, Inject } from '@angular/core';

import { MatCheckbox } from '@angular/material/checkbox';

import { Store, select } from '@ngrx/store';

import { Company } from '@ucap/domain-organization';
import { LoginTry } from '@ucap/domain-authentication';

import { NativeService } from '@ucap/native';

import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';
import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';
import {
  SessionStorageService,
  LocalStorageService
} from '@ucap/ng-web-storage';
import { PiService } from '@ucap/ng-pi';
import { ProtocolService } from '@ucap/ng-protocol';
import { CompanyActions, CompanySelector } from '@ucap/ng-store-organization';
import { LoginActions } from '@ucap/ng-store-authentication';

import { UserStore } from '@app/models/user-store';
import { LoginSession } from '@app/models/login-session';
import { AppKey } from '@app/types/app-key.type';
import { AppAuthenticationService } from '@app/services/app-authentication.service';

import { environment } from '@environments';
import { UrlConfig } from '@ucap/core';

@Component({
  selector: 'app-authentication-login',
  templateUrl: './login.component.html',
  styleUrls: ['./login.component.scss']
})
export class LoginComponent implements OnInit, OnDestroy {
  companyGroupCode = environment.companyConfig.companyGroupCode;
  userStore: UserStore;

  readonly useRememberMe =
    environment.productConfig.authentication.useRememberMe;

  readonly useAutoLogin = environment.productConfig.authentication.useAutoLogin;

  readonly fixedCompanyCode = environment.companyConfig.fixedCompanyCode;

  @ViewChild('chkUseRememberMe', { static: false })
  chkUseRememberMe: MatCheckbox;

  @ViewChild('chkUseAutoLogin', { static: false })
  chkUseAutoLogin: MatCheckbox;

  loginSession: LoginSession;
  companyList: Company[];
  disableLoginForm = false;
  loginProcessing = false;
  loginTry: LoginTry;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private piService: PiService,
    private protocolService: ProtocolService,
    private sessionStorageService: SessionStorageService,
    private localStorageService: LocalStorageService,
    private i18nService: I18nService,

    private store: Store<any>,
    private appAuthenticationService: AppAuthenticationService,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private logService: LogService
  ) {}

  ngOnInit(): void {
    this.localStorageService
      .encGet$<UserStore>(
        AppKey.UserStore,
        environment.productConfig.localEncriptionKey
      )
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((userStore) => (this.userStore = userStore));

    this.loginSession = this.appAuthenticationService.getLoginSession();

    this.sessionStorageService
      .get$<LoginTry>(AppKey.LoginTry)
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((loginTry) => (this.loginTry = loginTry));

    this.protocolService.disconnect();

    this.store.dispatch(
      CompanyActions.companies({
        req: { companyGroupCode: this.companyGroupCode }
      })
    );

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(CompanySelector.companyList)
      )
      .subscribe((companyList) => {
        this.companyList = companyList;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onLogin(event: {
    companyCode: string;
    loginId: string;
    loginPw: string;
    notValid: () => void;
  }) {
    const useRememberMe: boolean = this.chkUseRememberMe.checked;
    const useAutoLogin: boolean = this.chkUseAutoLogin.checked;

    this.disableLoginForm = true;
    this.loginProcessing = true;

    this.piService
      .login2({
        companyCode: event.companyCode,
        loginId: event.loginId,
        loginPw: event.loginPw,
        deviceType: this.loginSession.deviceType
      })
      .pipe(take(1))
      .subscribe(
        (res) => {
          if ('success' !== res.status.toLowerCase()) {
            this.onWebLoginFailure(event, res.status);
            return;
          } else {
            this.store.dispatch(
              LoginActions.webLoginSuccess({
                companyCode: event.companyCode,
                loginId: event.loginId,
                loginPw: event.loginPw,
                autoLogin: useAutoLogin,
                rememberMe: useRememberMe,
                login2Response: res
              })
            );
            return;
          }
        },
        (error) => {
          this.onWebLoginFailure(event, error);
        },
        () => {
          this.disableLoginForm = false;
          this.loginProcessing = false;
        }
      );
  }

  onClickForgotPassword() {
    const piUrls = UrlConfig.getUrls(
      environment.piModuleConfig.hostConfig,
      environment.piModuleConfig.urls
    );
    if (
      !!piUrls &&
      !!piUrls.passwordInitStep1 &&
      piUrls.passwordInitStep1.length > 0
    ) {
      this.nativeService.platform_openDefaultBrowser(
        piUrls.passwordInitStep1 + `?locale=${this.i18nService.currentLng}`,
        {
          name: this.i18nService.t(
            'authentication:login.labels.changePassword'
          ),
          features:
            'menubar=no,location=no,resizable=yes,scrollbars=yes,status=no,width=400,height=510'
        }
      );
    }
  }

  onClickPrivacy() {
    const piUrls = UrlConfig.getUrls(
      environment.piModuleConfig.hostConfig,
      environment.piModuleConfig.urls
    );

    if (!!piUrls && !!piUrls.policyPrivacy && piUrls.policyPrivacy.length > 0) {
      this.nativeService.platform_openDefaultBrowser(
        piUrls.policyPrivacy + `?locale=${this.i18nService.currentLng}`,
        {
          name: this.i18nService.t('authentication:login.labels.privacyPolicy')
        }
      );
    }
  }

  private onWebLoginFailure(
    event: {
      companyCode: string;
      loginId: string;
      loginPw: string;
      notValid: () => void;
    },
    error: any
  ) {
    this.store.dispatch(LoginActions.webLoginFailure({ error }));

    event.notValid();
  }
}
