import { Subject, combineLatest, merge } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  EventEmitter,
  Output,
  Input
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { User } from '@ucap/domain-organization';
import {
  UserInfo as RoomUserInfo,
  UserInfoShort as RoomUserInfoShort,
  FileDownloadInfo
} from '@ucap/domain-chat';

import { RoomSelector, ChattingSelector } from '@ucap/ng-store-chat';
import { UserSelector } from '@ucap/ng-store-organization';
import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import { VersionInfo } from '@ucap/domain-authentication';

export type UserInfoTypes = RoomUserInfo | RoomUserInfoShort;
export type FilterType = 'ALL' | 'DOWN' | 'NOTDOWN';

@Component({
  selector: 'app-chat-attach-downcheck',
  templateUrl: './attach-downcheck.component.html',
  styleUrls: ['./attach-downcheck.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class AttachDowncheckComponent implements OnInit, OnDestroy {
  private roomIdSubject = new Subject<string>();
  private ngOnDestroySubject = new Subject<boolean>();

  @Input()
  set roomId(roomId: string) {
    if (this._roomId !== roomId) {
      this._roomId = roomId;
      this.roomIdSubject.next(roomId);
      this._initializeData();
    }
  }
  get roomId(): string {
    return this._roomId;
  }
  // tslint:disable-next-line: variable-name
  _roomId: string;

  @Input()
  set attachmentSeq(value: number) {
    this._attachmentSeq = value;
    this._initializeData();
  }
  // tslint:disable-next-line: variable-name
  _attachmentSeq: number;

  filterType: FilterType = 'ALL';

  @Output()
  closed = new EventEmitter<void>();

  roomUsers: RoomUserInfo[] = [];
  roomUsersShort: RoomUserInfoShort[] = [];
  currentRoomUsers: (RoomUserInfo | RoomUserInfoShort)[];
  filteredRoomUsers: (RoomUserInfo | RoomUserInfoShort)[];
  fileInfoCheckList: FileDownloadInfo[];

  user: User;
  versionInfo: VersionInfo;

  constructor(
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });

    this._initializeData();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }

    if (!!this.roomIdSubject) {
      this.roomIdSubject.next();
      this.roomIdSubject.complete();
    }
  }

  private _initializeData() {
    combineLatest([
      this.store.pipe(select(UserSelector.user)),
      this.store.pipe(select(RoomSelector.roomUser, this.roomId)),
      this.store.pipe(select(RoomSelector.roomUserShort, this.roomId)),
      this.store.pipe(select(ChattingSelector.fileInfoCheckList, this.roomId))
    ])
      .pipe(takeUntil(merge(this.ngOnDestroySubject, this.roomIdSubject)))
      .subscribe(([user, roomUser, roomUserShort, fileInfoCheckList]) => {
        this.user = user;

        // retrieve room users.
        if (!!user) {
          if (
            !!roomUser &&
            !!roomUser.userInfos &&
            roomUser.userInfos.length > 0
          ) {
            this.currentRoomUsers = roomUser.userInfos.filter(
              (item) =>
                !!item.isJoinRoom && String(item.seq) !== String(user.info.seq)
            );
          } else if (
            !!roomUserShort &&
            !!roomUserShort.userInfos &&
            roomUserShort.userInfos.length > 0
          ) {
            this.currentRoomUsers = roomUserShort.userInfos.filter(
              (item) =>
                !!item.isJoinRoom && String(item.seq) !== String(user.info.seq)
            );
          }
        }

        // fileInfoDown check.
        if (!!this._attachmentSeq) {
          this.fileInfoCheckList = fileInfoCheckList.filter(
            (item) => item.seq === this._attachmentSeq
          );
        }

        this.getFiltered();
      });
  }

  onChangeFilterType(type: FilterType) {
    this.filterType = type;
    this.getFiltered();
  }

  onClosed(event: MouseEvent): void {
    this.closed.emit();
  }

  getFiltered() {
    switch (this.filterType) {
      case 'ALL':
        {
          this.filteredRoomUsers = this.currentRoomUsers.slice();
        }
        break;
      case 'DOWN':
        {
          this.filteredRoomUsers = this.currentRoomUsers.filter((curUser) => {
            return (
              this.fileInfoCheckList.findIndex(
                (downUser) =>
                  !!downUser.isDownload &&
                  String(downUser.userSeq) === String(curUser.seq)
              ) > -1
            );
          });
        }
        break;
      case 'NOTDOWN':
        {
          this.filteredRoomUsers = this.currentRoomUsers.filter((curUser) => {
            return (
              this.fileInfoCheckList.findIndex(
                (downUser) =>
                  !downUser.isDownload &&
                  String(downUser.userSeq) === String(curUser.seq)
              ) > -1 ||
              !this.fileInfoCheckList.some(
                (downUser) => String(downUser.userSeq) === String(curUser.seq)
              )
            );
          });
        }
        break;
    }

    this.changeDetectorRef.markForCheck();
  }

  isMe(userInfo: UserInfoTypes) {
    if (!!this.user) {
      return String(this.user.info.seq) === String(userInfo.seq);
    } else {
      return false;
    }
  }

  isDownload(userInfo: UserInfoTypes): boolean {
    if (!!this.fileInfoCheckList && this.fileInfoCheckList.length > 0) {
      return this.fileInfoCheckList.some(
        (item) =>
          !!item.isDownload && String(item.userSeq) === String(userInfo.seq)
      );
    }
    return false;
  }
  getDownloadDateString(userInfo: UserInfoTypes): string {
    if (!!this.fileInfoCheckList && this.fileInfoCheckList.length > 0) {
      const obj = this.fileInfoCheckList.find(
        (item) =>
          !!item.isDownload && String(item.userSeq) === String(userInfo.seq)
      );
      if (!!obj) {
        return obj.downloadDate;
      }
    }
  }
}
