import { Subject, combineLatest } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter,
  ViewChild
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { FileDownloadItem } from '@ucap/domain-common';
import {
  UserInfo as RoomUserInfo,
  UserInfoShort as RoomUserInfoShort,
  FileInfo,
  FileDownloadInfo,
  FileType
} from '@ucap/domain-chat';

import { InfoRequest } from '@ucap/protocol-file';

import { LogService } from '@ucap/ng-logger';
import {
  RoomUserMap,
  RoomUserShortMap
} from '@ucap/ng-store-chat/lib/store/room/state';
import {
  RoomSelector,
  ChattingActions,
  ChattingSelector
} from '@ucap/ng-store-chat';

import { FileListComponent } from '@ucap/ng-ui-chat';

@Component({
  selector: 'app-chat-attach-file-list',
  templateUrl: './attach-file-list.component.html',
  styleUrls: ['./attach-file-list.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class AttachFileListComponent implements OnInit, OnDestroy {
  @Input()
  roomId: string;

  @Input()
  fileInfoList: {
    fileInfo: FileInfo;
    fileDownloadItem: FileDownloadItem;
  }[] = [];

  @Input()
  selectedList: {
    fileInfo: FileInfo;
    fileDownloadItem: FileDownloadItem;
  }[] = [];

  @Output()
  changeCheck = new EventEmitter<{
    checked: boolean;
    fileInfo: FileInfo;
    fileDownloadItem: FileDownloadItem;
  }>();

  @Output()
  showDownCheck = new EventEmitter<FileInfo>();

  @ViewChild('ucapChatList', { static: false })
  ucapChatList: FileListComponent;

  roomUserMap: RoomUserMap;
  roomUserShortMap: RoomUserShortMap;
  downloadTotalCount: number;
  downloadCheckList: FileDownloadInfo[] = [];

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {}

  ngOnInit(): void {
    if (!!this.roomId) {
      this.store.dispatch(
        ChattingActions.fileInfos({
          req: { roomId: this.roomId, type: FileType.File } as InfoRequest
        })
      );
    }

    combineLatest([
      this.store.pipe(select(RoomSelector.roomUser, this.roomId)),
      this.store.pipe(select(RoomSelector.roomUserShort, this.roomId))
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([roomUsers, roomUsersShort]) => {
        this.roomUserMap = roomUsers;
        this.roomUserShortMap = roomUsersShort;

        if (
          !!this.roomUserMap &&
          !!this.roomUserMap.userInfos &&
          this.roomUserMap.userInfos.length > 0
        ) {
          this.downloadTotalCount = this.roomUserMap.userInfos.length;
        } else if (
          !!this.roomUserShortMap &&
          !!this.roomUserShortMap.userInfos &&
          this.roomUserShortMap.userInfos.length > 0
        ) {
          this.downloadTotalCount = this.roomUserShortMap.userInfos.length;
        }
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ChattingSelector.fileInfoCheckList, this.roomId)
      )
      .subscribe((downList) => {
        this.downloadCheckList = downList;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  getDownloadedCount(fileInfo: FileInfo): number {
    if (!!this.downloadCheckList && this.downloadCheckList.length > 0) {
      return this.downloadCheckList.filter((item) => item.seq === fileInfo.seq)
        .length;
    }
    return 0;
  }

  getItemChecked(fileInfo: FileInfo): boolean {
    if (!!this.selectedList && this.selectedList.length > 0) {
      return this.selectedList.some(
        (info) => info.fileInfo.seq === fileInfo.seq
      );
    }
    return false;
  }

  getDownloadItem(fileInfo: FileInfo): FileDownloadItem {
    const obj = this.selectedList.find(
      (item) => item.fileInfo.seq === fileInfo.seq
    );

    if (!!obj) {
      return obj.fileDownloadItem;
    } else {
      return new FileDownloadItem();
    }
  }

  onShowDownCheck(fileInfo: FileInfo): void {
    this.showDownCheck.emit(fileInfo);
  }

  onChangeCheck(param: {
    checked: boolean;
    fileInfo: FileInfo;
    fileDownloadItem: FileDownloadItem;
  }) {
    this.changeCheck.emit(param);
  }

  getSenderInfo(userSeq: string): RoomUserInfo | RoomUserInfoShort {
    if (
      !!this.roomUserShortMap &&
      !!this.roomUserShortMap.userInfos &&
      this.roomUserShortMap.userInfos.length > 0
    ) {
      return this.roomUserShortMap.userInfos.find(
        (item) => item.seq + '' === userSeq + ''
      );
    }

    if (
      !!this.roomUserMap &&
      !!this.roomUserMap.userInfos &&
      this.roomUserMap.userInfos.length > 0
    ) {
      return this.roomUserMap.userInfos.find(
        (item) => item.seq + '' === userSeq + ''
      );
    }
  }

  updatePsDirectiveRef(): void {
    if (!!this.ucapChatList && !!this.ucapChatList.psDirectiveRef) {
      this.ucapChatList.psDirectiveRef.update();
    }
  }
}
