import { Subject } from 'rxjs';

import {
  Component,
  OnInit,
  Input,
  EventEmitter,
  Output,
  ChangeDetectorRef,
  OnDestroy,
  Inject,
  ChangeDetectionStrategy
} from '@angular/core';

import { MatDialog } from '@angular/material/dialog';

import { FileDownloadItem } from '@ucap/domain-common';
import { User } from '@ucap/domain-organization';
import { LoginSession, LoginInfo } from '@ucap/domain-authentication';
import {
  Info,
  EventJson,
  EventType,
  StickerEventJson,
  isRecalled,
  isCopyable,
  isRecallable,
  isForwardable,
  FileEventJson,
  MassTranslationEventJson,
  VideoConferenceEventJson,
  UserInfo as RoomUserInfo,
  UserInfoShort as RoomUserInfoShort,
  RoomInfo,
  RoomType
} from '@ucap/domain-chat';

import { NativeService } from '@ucap/native';

import { UCAP_NATIVE_SERVICE } from '@ucap/ng-native';

import { SelectFileInfo } from '@ucap/ng-ui/viewer';

import {
  TextDetailDialogComponent,
  TextDetailDialogData,
  TextDetailDialogResult
} from '@app/sections/chat/dialogs/text-detail.dialog.component';
import { UserPermission } from '@ucap/domain-authorization';

@Component({
  selector: 'app-chat-message-box',
  templateUrl: './message-box.component.html',
  styleUrls: ['./message-box.component.scss'],

  changeDetection: ChangeDetectionStrategy.OnPush
})
export class MessageBoxComponent implements OnInit, OnDestroy {
  private ngOnDestroySubject: Subject<void> = new Subject();

  @Input()
  message: Info<EventJson>;
  @Input()
  set roomInfo(value: RoomInfo) {
    this._roomInfo = value;

    this.expiredFileStdSeq =
      !!value && !!value.expiredFileStdSeq
        ? value.expiredFileStdSeq
        : undefined;

    this.isShowUnreadCount =
      !!this.roomInfo &&
      [RoomType.Mytalk, RoomType.Allim, RoomType.Bot, RoomType.Link].some(
        (v) => v === this.roomInfo.roomType
      )
        ? false
        : true;
  }
  get roomInfo(): RoomInfo {
    return this._roomInfo;
  }
  // tslint:disable-next-line: variable-name
  _roomInfo: RoomInfo;

  @Input()
  loginSession: LoginSession;
  @Input()
  loginInfo: LoginInfo;
  @Input()
  user: User;
  @Input()
  userPermission: UserPermission;

  @Input()
  isMe = false;
  @Input()
  translationSimpleview: boolean;
  @Input()
  senderInfo: RoomUserInfoShort | RoomUserInfo;
  @Input()
  defaultProfileImage = '';
  @Input()
  profileImageRoot = '';

  @Input()
  dateChanged = false;
  @Input()
  unreadCount: number;

  @Input()
  highlight = false;
  @Input()
  existReadToHere = false;

  @Output()
  fileViewer: EventEmitter<{
    selectFileInfo: SelectFileInfo;
    senderInfo: RoomUserInfoShort | RoomUserInfo;
  }> = new EventEmitter();

  @Output()
  fileSave: EventEmitter<{
    fileInfo: FileEventJson;
    fileDownloadItem: FileDownloadItem;
    type: string;
    senderInfo: RoomUserInfoShort | RoomUserInfo;
  }> = new EventEmitter();

  @Output()
  messageContextMenu = new EventEmitter<{
    menuType: string;
    message: Info<EventJson>;
  }>();

  @Output()
  massTranslationDetail = new EventEmitter<{
    message: Info<MassTranslationEventJson>;
    contentsType: string;
  }>();

  @Output()
  openProfile = new EventEmitter<string>();

  @Output()
  joinConference = new EventEmitter<number>();

  expiredFileStdSeq: number;
  isShowUnreadCount = false;

  EventType = EventType;

  isRecalledMessage = isRecalled;
  isCopyableMessage = isCopyable;
  isRecallableMessage = (event: Info<any>) => {
    return !!this.user && !!this.user.info && this.user.info.seq
      ? isRecallable(event, String(this.user.info.seq))
      : false;
  };
  isForwardableMessage = (event: Info) => {
    let enable: boolean = isForwardable(event, this.expiredFileStdSeq);
    if (
      !!enable &&
      event.type === EventType.File &&
      !!this.userPermission &&
      !!this.userPermission.fileTransferAllowedCompanyList &&
      this.userPermission.fileTransferAllowedCompanyList.length > 0
    ) {
      enable =
        this.userPermission.fileTransferAllowedCompanyList.indexOf(
          (event.sentMessageJson as FileEventJson).companyCode
        ) > -1;
    }
    return enable;
  };
  isMoreable = (event: Info) => {
    if (event.type === EventType.VideoConference) {
      return false;
    }
    return true;
  };

  constructor(
    private dialog: MatDialog,
    @Inject(UCAP_NATIVE_SERVICE) private nativeService: NativeService,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  ngOnInit() {}

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  /**
   * 정보성 Event 인지 판단.
   * @description 정보성 event 일 경우 프로필, 일시 를 표현하지 않는다.
   * Edit with reducers.ts / sync / updateRoomForNewEventMessage
   */
  isInformation(info: Info<EventJson>) {
    if (
      info.type === EventType.Join ||
      info.type === EventType.Exit ||
      info.type === EventType.ForcedExit ||
      info.type === EventType.RenameRoom ||
      info.type === EventType.NotificationForiOSCapture ||
      info.type === EventType.NotificationForTimerRoom ||
      info.type === EventType.GuideForRoomTimerChanged
    ) {
      return true;
    }
    return false;
  }

  /**
   * Get Sticker image.
   */
  getStickerImage(message: Info<EventJson>): string {
    if (!!message.sentMessageJson) {
      return `assets/sticker/sticker_s_${
        (message.sentMessageJson as StickerEventJson).file
      }.png`;
    } else {
      return '';
    }
  }

  onOpenProfile(userInfo: RoomUserInfoShort | RoomUserInfo): void {
    this.openProfile.emit(String(userInfo.seq));
  }

  /**
   * Url link Open Event.
   */
  onOpenLink(url: string): void {
    this.nativeService.platform_openDefaultBrowser(url);
  }

  onOpenVcDetail(message: VideoConferenceEventJson): void {
    // if (!!message && !!message.conferenceSeq) {
    //   this.joinConference.emit(message.conferenceSeq);
    // }
    console.log(message);
  }
  onJoinConference(message: VideoConferenceEventJson): void {
    if (!!message && !!message.conferenceSeq) {
      this.joinConference.emit(message.conferenceSeq);
    }
  }

  /**
   * Detail view > Mass text.
   */
  onOpenMassDetail(eventMassSeq: number): void {
    const dialogRef = this.dialog.open<
      TextDetailDialogComponent,
      TextDetailDialogData,
      TextDetailDialogResult
    >(TextDetailDialogComponent, {
      panelClass: 'mid-create-dialog',
      data: {
        message: this.message,
        roomId: this.roomInfo.roomId,
        userSeq: String(this.user.info.seq),
        deviceType: this.loginSession.deviceType,
        token: this.loginInfo.tokenString,
        eventMassSeq: Number(eventMassSeq)
      }
    });
  }

  /**
   * Detail view > Mass translation text
   */
  onOpenMassTranslationDetail(params: {
    message: Info<MassTranslationEventJson>;
    contentsType: string;
  }) {
    this.massTranslationDetail.emit(params);
  }

  onClickMessageContextMenu(menuType: string) {
    this.messageContextMenu.emit({
      menuType,
      message: this.message
    });
  }

  /** [Event] Image Viewer */
  openViewer(selectFileInfo: SelectFileInfo): void {
    this.fileViewer.emit({ selectFileInfo, senderInfo: this.senderInfo });
  }

  onClickFileSave(params: {
    fileInfo: FileEventJson;
    fileDownloadItem: FileDownloadItem;
    type: string;
  }): void {
    this.fileSave.emit({ ...params, senderInfo: this.senderInfo });
  }
}
