import moment from 'moment';

import { Subject, combineLatest } from 'rxjs';
import { takeUntil, take } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter
} from '@angular/core';

import { Store, select } from '@ngrx/store';
import { Dictionary } from '@ngrx/entity';

import {
  User,
  UserInfo,
  UserInfoSS,
  UserInfoF,
  UserInfoDN
} from '@ucap/domain-organization';
import { RoomInfo, RoomType } from '@ucap/domain-chat';

import { UpdateRequest } from '@ucap/protocol-room';

import { UserSelector } from '@ucap/ng-store-organization';
import {
  RoomSelector,
  RoomActions,
  RoomUserMap,
  RoomUserShortMap
} from '@ucap/ng-store-chat';

import { DateService } from '@ucap/ng-ui/date';
import {
  TranslatePipe as OrganizationTranslate,
  TranslateService
} from '@ucap/ng-ui-organization';

import { AppChatService } from '@app/services/app-chat.service';
import { SearchInfo } from '@app/pages/chat/models/search-info';

export type UserInfoTypes = UserInfo | UserInfoSS | UserInfoF | UserInfoDN;

@Component({
  selector: 'app-chat-room-expansion',
  templateUrl: './room-expansion.component.html',
  styleUrls: ['./room-expansion.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class RoomExpansionComponent implements OnInit, OnDestroy {
  @Input()
  set searchObj(obj: SearchInfo) {
    this._searchObj = obj;

    if (obj.isShowSearch && obj.searchWord.localeCompare('') !== 0) {
      this._onRoomSearch(obj);
    } else {
      this._searchObj.isShowSearch = false;
      this.roomList = this.originalRoomList;
      this._initRoomGroup();
    }
  }

  get searchObj(): SearchInfo {
    return this._searchObj;
  }
  // tslint:disable-next-line: variable-name
  _searchObj: SearchInfo;

  @Input()
  checkable = false;

  @Input()
  currentRoomId?: string;

  @Input()
  selectedRoomList: RoomInfo[] = [];

  @Output()
  toggleItem = new EventEmitter<{
    checked: boolean;
    roomInfo: RoomInfo;
  }>();

  @Output()
  openChatRoom: EventEmitter<RoomInfo> = new EventEmitter();

  @Output()
  searchResultList = new EventEmitter<RoomInfo[]>();

  user: User;

  processing = false;

  originalRoomList: RoomInfo[];
  roomList: RoomInfo[];
  standbyRooms: string[];
  roomUsersDictionary: Dictionary<RoomUserMap>;
  roomUsersShortDictionary: Dictionary<RoomUserShortMap>;
  roomGroup: { division: string; roomList: RoomInfo[] }[];

  organizationTranslate: OrganizationTranslate;

  RoomType = RoomType;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private appChatService: AppChatService,
    private dateService: DateService,
    private translateService: TranslateService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef
  ) {
    this.organizationTranslate = new OrganizationTranslate(
      this.translateService,
      this.changeDetectorRef
    );
  }

  ngOnInit(): void {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });

    combineLatest([
      this.store.pipe(select(RoomSelector.rooms)),
      this.store.pipe(select(RoomSelector.standbyRooms))
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([rooms, standbyRooms]) => {
        this.standbyRooms = standbyRooms;
        this.originalRoomList = rooms;
        this.roomList = rooms;

        if (!!this._searchObj && !!this._searchObj.isShowSearch) {
          this._onRoomSearch(this.searchObj);
        }

        // groupping.
        this._initRoomGroup();
      });

    combineLatest([
      this.store.pipe(
        select(
          (state: any) =>
            state.chat.room.roomUsers.entities as Dictionary<RoomUserMap>
        )
      ),
      this.store.pipe(
        select(
          (state: any) =>
            state.chat.room.roomUsersShort.entities as Dictionary<
              RoomUserShortMap
            >
        )
      )
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([roomUsers, roomUsersShort]) => {
        this.roomUsersDictionary = roomUsers;
        this.roomUsersShortDictionary = roomUsersShort;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  private _initRoomGroup() {
    this.roomGroup = [];

    // standby room exclude.
    this.roomList = (this.roomList || []).filter((info) => {
      return (
        info.isJoinRoom &&
        !this.standbyRooms.find((standbyRoom) => standbyRoom === info.roomId)
      );
    });

    this.roomList.forEach((roomInfo) => {
      const date = roomInfo.finalEventDate;
      let division = '';
      try {
        const value = this.dateService.get(date, 'YYYYMMDD');

        if (value === 'Invalid date') {
          division = date;
        } else {
          division = value;
        }
      } catch (error) {
        division = date;
      }

      const index = this.roomGroup.findIndex(
        (info) => info.division === division
      );
      if (index > -1) {
        this.roomGroup[index] = {
          ...this.roomGroup[index],
          roomList: [...this.roomGroup[index].roomList, roomInfo]
        };
      } else {
        this.roomGroup.push({
          division,
          roomList: [roomInfo]
        });
      }
    });

    this.changeDetectorRef.markForCheck();
  }

  private _onRoomSearch(obj: SearchInfo) {
    const searchRoomList: RoomInfo[] = [];

    this.processing = true;
    this.changeDetectorRef.markForCheck();

    this.originalRoomList.forEach((roomInfo) => {
      if (roomInfo.roomName.indexOf(obj.searchWord) > -1) {
        searchRoomList.push(roomInfo);
      } else {
        const roomUsers = this.appChatService.getRoomUserList(
          this.user,
          roomInfo.roomId,
          this.roomUsersDictionary,
          this.roomUsersShortDictionary
        );

        if (
          roomUsers.existUsers &&
          roomUsers.users.filter(
            (userInfo) =>
              !!userInfo.isJoinRoom &&
              (userInfo.name.indexOf(obj.searchWord) > -1 ||
                userInfo.nameEn.indexOf(obj.searchWord) > -1 ||
                userInfo.nameCn.indexOf(obj.searchWord) > -1)
          ).length > 0
        ) {
          searchRoomList.push(roomInfo);
        }
      }
    });

    this.searchResultList.emit(searchRoomList);
    this.roomList = searchRoomList.slice();
    this.processing = false;
    this._initRoomGroup();
  }

  isToday(date: any): 'T' | 'Y' | undefined {
    if (this.dateService.isToday(date)) {
      return 'T';
    } else if (this.dateService.isToday(moment(date).add(1, 'days').format())) {
      return 'Y';
    } else {
      return undefined;
    }
    // return this.dateService.isToday(date);
  }

  onToggleAlarm(roomInfo: RoomInfo): void {
    if (!roomInfo) {
      return;
    }

    this.store.dispatch(
      RoomActions.update({
        req: {
          roomId: roomInfo.roomId,
          roomName:
            roomInfo.roomName.trim().length === 0
              ? ' '
              : roomInfo.roomName.trim(),
          receiveAlarm: !roomInfo.receiveAlarm,
          syncAll: false
        } as UpdateRequest
      })
    );
    this.changeDetectorRef.markForCheck();
  }

  onDelRoom(roomInfo: RoomInfo): void {
    this.appChatService.exitRoomDialog(roomInfo);
    this.changeDetectorRef.markForCheck();
  }

  getChecked(roomInfo: RoomInfo): boolean {
    if (this.selectedRoomList.some((info) => info.roomId === roomInfo.roomId)) {
      return true;
    } else {
      return false;
    }
  }

  onToggleItem(event: { checked: boolean; roomInfo: RoomInfo }): void {
    this.toggleItem.emit(event);
  }

  onClickRoomItem(event: MouseEvent, roomInfo: RoomInfo): void {
    event.preventDefault();
    event.stopPropagation();

    if (!!this.checkable) {
      this.onToggleItem({
        checked: !this.getChecked(roomInfo),
        roomInfo
      });
    } else {
      this.openChatRoom.emit(roomInfo);
    }
  }

  onOpenChatRoom(roomInfo: RoomInfo): void {
    this.openChatRoom.emit(roomInfo);
  }
}
