import { Subject, combineLatest } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { User } from '@ucap/domain-organization';
import { RoomInfo, RoomType } from '@ucap/domain-chat';

import { I18nService } from '@ucap/ng-i18n';
import { UserSelector } from '@ucap/ng-store-organization';
import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import {
  RoomSelector,
  RoomUserMap,
  RoomUserShortMap
} from '@ucap/ng-store-chat';

import {
  TranslatePipe as OrganizationTranslate,
  TranslateService
} from '@ucap/ng-ui-organization';

import { AppChatService } from '@app/services/app-chat.service';
import { VersionInfo } from '@ucap/domain-authentication';

@Component({
  selector: 'app-chat-room-list-item-01',
  templateUrl: './room-list-item-01.component.html',
  styleUrls: ['./room-list-item-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class RoomListItem01Component implements OnInit, OnDestroy {
  private ngOnDestroySubject: Subject<void> = new Subject();

  @Input()
  roomInfo: RoomInfo;

  @Input()
  currentRoomId: string;

  @Input()
  checkable = false;

  @Input()
  selectedRoomList: RoomInfo[] = [];

  @Input()
  moreable = true;

  @Input()
  dateFormat?: string;

  @Output()
  toggleItem = new EventEmitter<{
    checked: boolean;
    roomInfo: RoomInfo;
  }>();

  @Output()
  openChatRoom = new EventEmitter<RoomInfo>();

  @Output()
  toggleAlarm = new EventEmitter<RoomInfo>();

  @Output()
  delRoom = new EventEmitter<RoomInfo>();

  user: User;
  versionInfo: VersionInfo;

  organizationTranslate: OrganizationTranslate;

  defaultProfileImage: string;
  defaultProfileImageMulti: string;

  roomUserMap: RoomUserMap;
  roomUserShortMap: RoomUserShortMap;

  RoomType = RoomType;

  constructor(
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private appChatService: AppChatService,
    private i18nService: I18nService,
    private translateService: TranslateService
  ) {
    this.translateService.setDefaultLang(this.i18nService.currentLng);
    this.translateService.use(this.i18nService.currentLng);
    this.organizationTranslate = new OrganizationTranslate(
      this.translateService,
      this.changeDetectorRef
    );

    // default image setting
    this.defaultProfileImage = this.appChatService.defaultProfileImage;
    this.defaultProfileImageMulti = this.appChatService.defaultProfileImageMulti;
  }

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });

    combineLatest([
      this.store.pipe(select(UserSelector.user)),
      this.store.pipe(select(RoomSelector.roomUser, this.roomInfo.roomId)),
      this.store.pipe(select(RoomSelector.roomUserShort, this.roomInfo.roomId))
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([user, roomUserMap, roomUserShortMap]) => {
        this.user = user;
        this.roomUserMap = roomUserMap;
        this.roomUserShortMap = roomUserShortMap;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  getRoomName(): string {
    if (!this.roomInfo) {
      return '';
    }
    const roomName = this.appChatService.getRoomName(
      this.organizationTranslate,
      this.user,
      this.roomInfo,
      this.roomUserMap,
      this.roomUserShortMap
    );

    return roomName;
  }

  getRoomProfileImage(): string {
    if (!this.roomInfo) {
      return '';
    }
    const roomImage = this.appChatService.getRoomProfileImage(
      this.user,
      this.roomInfo,
      this.roomUserMap,
      this.roomUserShortMap
    );

    return roomImage;
  }

  getChecked(roomInfo: RoomInfo): boolean {
    if (this.selectedRoomList.some((info) => info.roomId === roomInfo.roomId)) {
      return true;
    } else {
      return false;
    }
  }

  onToggleRoom(event: { checked: boolean; roomInfo: RoomInfo }): void {
    this.toggleItem.emit(event);
  }
  onOpenChatRoom(roomInfo: RoomInfo) {
    this.openChatRoom.emit(roomInfo);
  }
  onToggleAlarm(roomInfo: RoomInfo) {
    this.toggleAlarm.emit(roomInfo);
  }
  onDelRoom(roomInfo: RoomInfo) {
    this.delRoom.emit(roomInfo);
  }
}
