import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  Output,
  EventEmitter,
  Input,
  ChangeDetectorRef,
  ChangeDetectionStrategy
} from '@angular/core';

import { Store } from '@ngrx/store';

import { MatSelectChange } from '@angular/material/select';
import { MatRadioChange } from '@angular/material/radio';

import {
  UserInfo as RoomUserInfo,
  UserInfoShort as RoomUserInfoShort,
  RoomInfo
} from '@ucap/domain-chat';

import { I18nService } from '@ucap/ng-i18n';

@Component({
  selector: 'app-chat-room-setting',
  templateUrl: './room-setting.component.html',
  styleUrls: ['./room-setting.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class RoomSettingComponent implements OnInit, OnDestroy {
  @Input()
  roomInfo: RoomInfo;

  @Input()
  isDialog = false;

  @Output()
  changedRoomSetting: EventEmitter<{
    invalid?: boolean;
    roomName?: string;
    checkedMe?: string;
    timerInterval?: number;
  }> = new EventEmitter();

  private ngOnDestroySubject = new Subject<boolean>();

  constructor(
    private i18nService: I18nService,
    private changeDetectorRef: ChangeDetectorRef
  ) {}

  roomName: string;
  timerArray: { value: number; text: string }[];
  checkMe: string;
  isInvlid: boolean;

  ngOnInit() {
    this.i18nService.languageChanged$
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((_) => {
        this._setTimerArray();
      });
    this._setTimerArray();
    this.checkMe = 'all';
  }

  ngOnDestroy() {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  // onKeyupName() {
  //   this.chatSettingForm.get('roomName').markAsTouched();
  // }

  onChangeName(roomName: string) {
    this.roomName = roomName;
    this.isInvlid = false;

    if (!!this.roomName && this.roomName.trim().localeCompare('') !== 0) {
      const forbidden = /[\{\}\[\]\/?.;:|\)*~`!^+<>@\#$%&\\\=\(\'\"]/g.test(
        this.roomName
      );

      if (forbidden) {
        this.isInvlid = true;
      }
    }

    this.changedRoomSetting.emit({
      invalid: this.isInvlid,
      roomName: this.roomName,
      checkedMe: this.checkMe
    });
  }

  private _setTimerArray() {
    const hourFrom = this.i18nService.t('common:units.hourFrom');
    const minute = this.i18nService.t('common:units.minute');
    const second = this.i18nService.t('common:units.second');

    this.timerArray = [
      { value: 5, text: `5 ${second}` },
      { value: 10, text: `10 ${second}` },
      { value: 30, text: `30 ${second}` },
      { value: 60, text: `1 ${minute}` },
      { value: 300, text: `5 ${minute}` },
      { value: 600, text: `10 ${minute}` },
      { value: 1800, text: `30 ${minute}` },
      { value: 3600, text: `1 ${hourFrom}` },
      { value: 21600, text: `6 ${hourFrom}` },
      { value: 43200, text: `12 ${hourFrom}` },
      { value: 86400, text: `24 ${hourFrom}` }
    ];
  }

  onChangeRadioBtn(event: MatRadioChange) {
    if (event.value === 'me') {
      this.checkMe = 'me';
    } else {
      this.checkMe = 'all';
    }

    this.changedRoomSetting.emit({
      invalid: this.isInvlid,
      roomName: this.roomName,
      checkedMe: this.checkMe
    });
  }
  onChangeSelection(value: number) {
    const interval = value;

    this.changedRoomSetting.emit({
      timerInterval: !value ? 86400 : value
    });
  }
}
