import { Subject, combineLatest, of } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  AfterViewInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  EventEmitter,
  Output,
  ViewChild,
  ElementRef,
  Self
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { MatMenuTrigger } from '@angular/material/menu';

import {
  User,
  UserInfo,
  UserInfoSS,
  UserInfoF,
  UserInfoDN
} from '@ucap/domain-organization';
import { StatusCode, PresenceType, StatusBulkInfo } from '@ucap/domain-status';
import { GroupInfoDetail } from '@ucap/domain-group';
import { UserInfo as RoomUserInfo } from '@ucap/domain-chat';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';
import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import { BuddySelector, GroupSelector } from '@ucap/ng-store-group';
import { PresenceSelector, UserSelector } from '@ucap/ng-store-organization';

import {
  ExpansionComponent as UcapExpansionComponent,
  NodeType,
  FlatNode,
  GroupNode,
  GroupMenuType
} from '@ucap/ng-ui-group';
import { PresenceUtil, ProfileMenuType } from '@ucap/ng-ui-organization';

import { LoginSession } from '@app/models/login-session';

import { environment } from '@environments';
import { GroupOpenInfo } from '@app/models/group-open-info';
import { SortViewType } from '@app/pages/group/types/sort-view.type';
import { UserInfoTypes } from '@app/types';
import { AppGroupService } from '@app/services/app-group.service';
import { VersionInfo } from '@ucap/domain-authentication';
import { GroupDetailData } from '@ucap/protocol-sync';

@Component({
  selector: 'app-group-expansion-01',
  templateUrl: './expansion-01.component.html',
  styleUrls: ['./expansion-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class Expansion01Component implements OnInit, OnDestroy, AfterViewInit {
  @Input()
  selectedUserList: UserInfoTypes[];

  @Input()
  set showType(type: SortViewType) {
    if (!!this.loginSession && !!this.loginSession.groupInfo) {
      this.loginSession.groupInfo.showType = type;
    }
    this._showType = type;
  }
  get showType() {
    return this._showType;
  }
  // tslint:disable-next-line: variable-name
  _showType: SortViewType;

  @Input()
  loginSession: LoginSession;

  @Output()
  profileMenu: EventEmitter<any> = new EventEmitter();

  @Output()
  clicked = new EventEmitter<{ event: MouseEvent; userInfo: UserInfoTypes }>();

  @Output()
  openProfile = new EventEmitter<UserInfoTypes>();

  @Output()
  selectGroupMenu = new EventEmitter<{
    menuType: GroupMenuType;
    groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfoF[] };
    rect: any;
  }>();

  @Output()
  selectProfileMenu: EventEmitter<{
    menuType: ProfileMenuType;
    userInfo: UserInfoF;
    group: GroupInfoDetail;
    rect: any;
  }> = new EventEmitter();

  @Output()
  toggleCheckUser: EventEmitter<
    {
      checked: boolean;
      userInfo: UserInfoSS;
    }[]
  > = new EventEmitter();

  @Output()
  toggleCheckGroup = new EventEmitter<{
    isChecked: boolean;
    groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfo[] };
  }>();

  @Output()
  clickProfileMenu = new EventEmitter<{
    menuType: ProfileMenuType;
    userInfo: UserInfoTypes;
  }>();

  @Output()
  groupDestroy = new EventEmitter<GroupOpenInfo>();

  @ViewChild('groupExpansion', { static: false })
  groupExpansion: UcapExpansionComponent;

  @ViewChild('groupContextMenuTrigger', { static: true })
  groupContextMenuTrigger: MatMenuTrigger;
  groupContextMenuPosition = { x: '0px', y: '0px' };

  @ViewChild('editGroupNameTrigger', { static: true })
  editGroupNameTrigger: MatMenuTrigger;
  editGroupNamePosition = { x: '0px', y: '0px' };

  @ViewChild('profileContextMenuTrigger', { static: true })
  profileContextMenuTrigger: MatMenuTrigger;
  profileContextMenuPosition = { x: '0px', y: '0px' };

  versionInfo: VersionInfo;
  user: User;

  groupMenuEvent: MouseEvent;
  statusBulkInfos: StatusBulkInfo[];
  currentSelectUser: UserInfoF;

  displayOrder: NodeType[] = [
    NodeType.Profile,
    NodeType.Favorite,
    NodeType.Buddy,
    NodeType.Default
  ];

  favorites: UserInfoF[];
  onlineFavorites: UserInfoF[];

  groupBuddies: { group: GroupInfoDetail; buddyList: UserInfoF[] }[];
  onlineBuddies: { group: GroupInfoDetail; buddyList: UserInfoF[] }[];
  onOffBuddies: { group: GroupInfoDetail; buddyList: UserInfoF[] }[];

  groupOpenInfo: GroupOpenInfo;

  editablGroup: GroupInfoDetail = null;
  ProfileMenuType = ProfileMenuType;
  GroupMenuType = GroupMenuType;
  String = String;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService,
    private i18nService: I18nService,
    @Self() private elementRef: ElementRef,
    private appGroupService: AppGroupService
  ) {}

  ngOnInit(): void {
    this.groupOpenInfo = this.loginSession.groupInfo;

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });

    combineLatest([
      this.store.pipe(select(BuddySelector.buddies)),
      this.store.pipe(select(GroupSelector.groups)),
      this.store.pipe(select(PresenceSelector.selectAllStatusBulkInfo))
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([buddies, groups, bulkInfos]) => {
        buddies = buddies || [];
        groups = groups || [];
        bulkInfos = bulkInfos || [];

        if (buddies.length === 0 || groups.length === 0) {
          return;
        }

        buddies = this.appGroupService.extractBuddiesFromGroups(
          buddies,
          groups
        );

        const favorites = buddies
          .filter((buddy) => buddy.isFavorit)
          .sort((a, b) => (a.name < b.name ? -1 : a.name > b.name ? 1 : 0));

        if (!!favorites && 0 < favorites.length) {
          this.favorites = favorites;
          this.onlineFavorites = this.favorites.filter((favorite) => {
            const tempBulkInfo = bulkInfos.filter(
              (bulkInfo) => String(bulkInfo.userSeq) === String(favorite.seq)
            );

            if (!!tempBulkInfo && tempBulkInfo.length > 0) {
              const pcOnline = PresenceUtil.isOnline(
                tempBulkInfo[0],
                PresenceType.PC
              );
              if (!!pcOnline) {
                return favorite;
              }
            }
          });
          this.changeDetectorRef.markForCheck();
        } else if (!!favorites && favorites.length === 0) {
          this.favorites = undefined;
          this.onlineFavorites = undefined;
          this.changeDetectorRef.markForCheck();
        }

        const tempOrder: GroupInfoDetail[] = [];
        let defaultGroup: GroupInfoDetail;
        const buddyGroup: GroupInfoDetail[] = [];
        groups.forEach((group) => {
          if (0 === group.seq) {
            defaultGroup = group;
          } else {
            buddyGroup.push(group);
          }
        });

        tempOrder.push(
          ...buddyGroup.sort((a, b) =>
            a.name < b.name ? -1 : a.name > b.name ? 1 : 0
          )
        );

        if (!!defaultGroup) {
          tempOrder.push(defaultGroup);
        }

        groups = tempOrder;

        if (!!groups && 0 < groups.length) {
          this.groupBuddies = [];

          for (const group of groups) {
            this.groupBuddies.push({
              group,
              buddyList: buddies.filter((buddy) => {
                return -1 < group.userSeqs.indexOf(String(buddy.seq));
              })
            });
          }

          this.changeDetectorRef.markForCheck();
        }

        if (!!bulkInfos && bulkInfos.length > 0) {
          // 접속한 동료 리스트
          this.onlineBuddies = [];

          for (const group of groups) {
            const tempBuddyList = [];

            this.onlineBuddies.push({
              group,
              buddyList: buddies.filter((buddy) => {
                if (-1 < group.userSeqs.indexOf(String(buddy.seq))) {
                  const tempBulkinfos = bulkInfos.filter(
                    (bulk) => bulk.userSeq === buddy.seq + ''
                  );
                  if (!!tempBulkinfos && tempBulkinfos.length > 0) {
                    const pcOnline = PresenceUtil.isOnline(
                      tempBulkinfos[0],
                      PresenceType.PC
                    );

                    if (!!pcOnline) {
                      return tempBuddyList.push(buddy);
                    }
                  }
                }
              })
            });
          }
        }
        // 온/오프라인 리스트
        if (!!bulkInfos && bulkInfos.length > 0) {
          this.onOffBuddies = [];
          const onlineGroup = {
            SENDER_SEQ: String(this.user.info.seq),
            SSVC_TYPE: 5,
            SVC_TYPE: 82,
            detailData: {
              seq: -998,
              name: this.i18nService.t('group:label.online'),
              isActive: true
            }
          } as GroupDetailData;
          const offlineGroup = {
            SENDER_SEQ: String(this.user.info.seq),
            SSVC_TYPE: 5,
            SVC_TYPE: 82,
            detailData: {
              seq: -999,
              name: this.i18nService.t('group:label.offline'),
              isActive: true
            }
          } as GroupDetailData;

          const tempGroups = [onlineGroup, offlineGroup];

          for (const group of tempGroups) {
            const tempBuddyList = [];
            this.onOffBuddies.push({
              group: group.detailData,
              buddyList: buddies.filter((buddy) => {
                const tempBulkinfos = bulkInfos.filter(
                  (bulk) => bulk.userSeq + '' === buddy.seq + ''
                );
                if (!!tempBulkinfos && tempBulkinfos.length > 0) {
                  const pcOnline = PresenceUtil.isOnline(
                    tempBulkinfos[0],
                    PresenceType.PC
                  );
                  const checkOnline = group.detailData.name.localeCompare(
                    this.i18nService.t('group:label.online')
                  );
                  const checkOffline = group.detailData.name.localeCompare(
                    this.i18nService.t('group:label.offline')
                  );
                  if (!!pcOnline && checkOnline === 0) {
                    return tempBuddyList.push(buddy);
                  } else if (!pcOnline && checkOffline === 0) {
                    return tempBuddyList.push(buddy);
                  }
                }
              })
            });
          }
        }
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(PresenceSelector.selectAllStatusBulkInfo)
      )
      .subscribe((statusArr) => {
        this.statusBulkInfos = statusArr;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }

    if (!!this.groupOpenInfo && !!this.groupDestroy) {
      this.groupDestroy.emit(this.groupOpenInfo);
    }
  }

  ngAfterViewInit() {
    this._groupOpenBySessionInfo();
  }

  onClickNode(params: { node: FlatNode; isExpand }) {
    // const findSeq = this.groupOpenInfo.groupSeqs.filter(seq => node.node.groupDetail.seq === seq)[0];
    const group = params.node.node.groupDetail;
    const lastGroupSeq = group.seq;
    const showType = this.showType;
    let tempGroupSeqs: number[] = [];

    const i = this.groupOpenInfo.groupSeqs.findIndex(
      (seq) => seq === group.seq
    );

    if (!!params.isExpand) {
      tempGroupSeqs = [...this.groupOpenInfo.groupSeqs];

      if (-1 === i) {
        tempGroupSeqs.push(params.node.node.groupDetail.seq);
      }
    } else {
      if (-1 < i) {
        tempGroupSeqs = this.groupOpenInfo.groupSeqs.filter(
          (seq) => seq !== group.seq
        );
      }
    }

    this.groupOpenInfo = {
      ...this.groupOpenInfo,
      groupSeqs: tempGroupSeqs,
      lastGroupSeq,
      showType
    };
  }

  onProfileMenu(
    event: MouseEvent,
    userInfo: UserInfoTypes,
    group: GroupInfoDetail,
    isSearchData: boolean
  ) {
    this.profileMenu.emit({ event, userInfo, group, isSearchData });
  }

  onClickUser(event: MouseEvent, userInfo: UserInfoF) {
    this.currentSelectUser = userInfo;
    this.clicked.emit({ event, userInfo });
  }

  onOpenProfile(userInfo: UserInfoF) {
    this.currentSelectUser = userInfo;
    this.openProfile.emit(userInfo);
  }

  getNodeGroupStatus(groupNode: GroupNode): string {
    if (
      !!groupNode &&
      !!this.showType &&
      this.showType === SortViewType.onOff
    ) {
      return `${groupNode.children.length}`;
    }

    if (
      !!this.statusBulkInfos &&
      this.statusBulkInfos.length > 0 &&
      !!groupNode &&
      !!groupNode.groupDetail &&
      !!groupNode.groupDetail.userSeqs &&
      groupNode.groupDetail.userSeqs.length > 0 &&
      !!this.showType &&
      this.showType !== SortViewType.onOff
    ) {
      let onlineCount = 0;
      this.statusBulkInfos.map((statusInfo) => {
        groupNode.groupDetail.userSeqs.filter((seq) => {
          if (Number(seq) === Number(statusInfo.userSeq)) {
            const pcOnline = PresenceUtil.isOnline(statusInfo, PresenceType.PC);
            const mobileOnline = PresenceUtil.isOnline(
              statusInfo,
              PresenceType.MOBILE
            );

            if (pcOnline) {
              onlineCount++;
            }
          }
        });
      });

      return `(${onlineCount}/${groupNode.children.length})`;
    } else {
      return `(0/0)`;
    }
  }

  onClickMoreMenu(params: { event: MouseEvent; node: FlatNode }) {
    if (this.showType === SortViewType.onOff) {
      return;
    }

    params.event.preventDefault();
    params.event.stopPropagation();
    this.groupMenuEvent = params.event;
    this.groupContextMenuPosition.x = params.event.clientX + 'px';
    this.groupContextMenuPosition.y = params.event.clientY + 'px';
    this.groupContextMenuTrigger.menu.focusFirstItem('mouse');
    this.groupContextMenuTrigger.menuData = {
      group: params.node.node.groupDetail
    };
    this.groupContextMenuTrigger.openMenu();
  }

  onShowEditGroup(group: GroupInfoDetail) {
    const x = this.groupMenuEvent.clientX - 20;
    const y = this.groupMenuEvent.clientY - 30;

    this.editGroupNamePosition.x = x + 'px';
    this.editGroupNamePosition.y = y + 'px';
    this.editGroupNameTrigger.menu.focusFirstItem('mouse');
    this.editGroupNameTrigger.menuData = {
      group
    };
    this.editGroupNameTrigger.openMenu();
  }

  onExpandMore() {
    this.groupExpansion.expandMore();
  }
  onExpandLess() {
    this.groupExpansion.expandLess();
  }

  onSelectGroupMenu(menuType: GroupMenuType, group: GroupInfoDetail) {
    let rect: any;

    if (menuType === GroupMenuType.rename) {
      const target = this.elementRef.nativeElement.querySelector(
        '.mat-tree-node'
      );
      const clickEventY = this.groupMenuEvent.clientY;
      const clientRect = target.getBoundingClientRect();

      rect = {
        width: clientRect.width,
        height: clientRect.height,
        top: clickEventY - 100 + clientRect.height,
        left: clientRect.left,
        bottom: clientRect.bottom,
        right: clientRect.right
      };
    }

    let groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfoF[] }[];

    if (group.seq === -9999) {
      groupBuddyList = [];
      groupBuddyList.push({ group, buddyList: this.favorites });
    } else {
      groupBuddyList = this.groupBuddies.filter(
        (g) => g.group.seq === group.seq
      );
    }

    this.selectGroupMenu.emit({
      menuType,
      groupBuddyList: groupBuddyList[0],
      rect
    });
  }

  getFavoritesForShowType(): UserInfoF[] {
    if (!!this.showType && this.showType === SortViewType.all) {
      return this.favorites;
    } else if (!!this.showType && this.showType === SortViewType.onlineBuddy) {
      return this.onlineFavorites;
    } else if (!!this.showType && this.showType === SortViewType.onOff) {
      return null;
    }
    return null;
  }

  getBuddiesForShowType(): {
    group: GroupInfoDetail;
    buddyList: UserInfoF[];
  }[] {
    if (!!this.showType && this.showType === SortViewType.all) {
      return this.groupBuddies;
    } else if (!!this.showType && this.showType === SortViewType.onlineBuddy) {
      return this.onlineBuddies;
    } else if (!!this.showType && this.showType === SortViewType.onOff) {
      return this.onOffBuddies;
    }
    return null;
  }

  onClikeMoreProfile(params: {
    event: MouseEvent;
    userInfo: UserInfoTypes;
    group: GroupInfoDetail;
    rect: any;
  }) {
    params.event.preventDefault();
    params.event.stopPropagation();

    this.profileContextMenuPosition.x = params.event.clientX + 'px';
    this.profileContextMenuPosition.y = params.event.clientY + 'px';
    this.profileContextMenuTrigger.menu.focusFirstItem('mouse');
    this.profileContextMenuTrigger.menuData = {
      userInfo: params.userInfo,
      group: params.group,
      rect: params.rect
    };
    this.profileContextMenuTrigger.openMenu();
  }

  onClickProfileMenu(params: {
    menuType: ProfileMenuType;
    userInfo: UserInfoTypes;
  }) {
    this.clickProfileMenu.emit(params);
  }

  onClickProfileContextMenu(
    event: MouseEvent,
    menuType: ProfileMenuType,
    userInfo: UserInfoF,
    group: GroupInfoDetail,
    rect: any
  ) {
    const clientRect = {
      width: rect.width,
      height: rect.height,
      top: rect.top,
      left: rect.left,
      right: rect.right,
      bottom: rect.bottom
    };

    if (menuType === ProfileMenuType.profile) {
      this.onClickUser(event, userInfo as UserInfoF);
    }

    this.selectProfileMenu.emit({
      menuType,
      userInfo,
      group,
      rect: clientRect
    });

    this.profileContextMenuTrigger.closeMenu();
  }

  getGroupNameBySeq(group: GroupInfoDetail): string {
    if (!!group && group.seq === 0) {
      return this.i18nService.t('group:category.default');
    } else if (!!group && group.seq === -9999) {
      return this.i18nService.t('group:category.favorite');
    }

    return '';
  }

  getShowGroupContextMenu(menuType: GroupMenuType, group: GroupInfoDetail) {
    // 즐겨찾기 그룹 숨김메뉴
    if (
      menuType === GroupMenuType.divide ||
      menuType === GroupMenuType.rename ||
      menuType === GroupMenuType.management ||
      menuType === GroupMenuType.copyGroup ||
      menuType === GroupMenuType.moveGroup ||
      menuType === GroupMenuType.deleteGroup
    ) {
      const favoriteName = this.i18nService.t('group:category.favorite');
      if (
        !group ||
        group === undefined ||
        (group.seq < 0 && group.name === 'Favorite')
      ) {
        return false;
      }

      /** 수정불가 그룹 핸들링. */
      if (!!group && !!environment.productConfig.group.fixedGroupSeqs) {
        const fixedGroupSeqs: number[] =
          environment.productConfig.group.fixedGroupSeqs;
        if (!!fixedGroupSeqs && fixedGroupSeqs.length > 0) {
          if (fixedGroupSeqs.indexOf(group.seq) > -1) {
            return false;
          }
        }
      }
    }

    // 기본 그룹 숨김메뉴
    if (
      menuType === GroupMenuType.rename ||
      menuType === GroupMenuType.deleteGroup
    ) {
      if (!!group && group.seq === 0) {
        return false;
      }
    }

    // 그룹원 0명인 그룹 메뉴 정리
    if (
      menuType === GroupMenuType.chat ||
      menuType === GroupMenuType.message ||
      menuType === GroupMenuType.divide ||
      menuType === GroupMenuType.copyGroup ||
      menuType === GroupMenuType.moveGroup
    ) {
      if (!!group && !!group.userSeqs && group.userSeqs.length > 0) {
        return true;
      } else {
        return false;
      }
    }

    return true;
  }
  getShowProfileContextMenu(menuType: ProfileMenuType, group: GroupInfoDetail) {
    if (!group || group === undefined) {
      if (
        menuType === ProfileMenuType.favorite ||
        menuType === ProfileMenuType.message ||
        menuType === ProfileMenuType.nickname
      ) {
        // continue;
      } else {
        return false;
      }
    }

    if (!!group && group.seq === -9999) {
      if (
        menuType === ProfileMenuType.removeBuddy ||
        menuType === ProfileMenuType.moveBuddy ||
        menuType === ProfileMenuType.copyBuddy
      ) {
        return false;
      }
      return true;
    }

    /** 수정불가 그룹 핸들링. */
    if (!!group && !!environment.productConfig.group.fixedGroupSeqs) {
      const fixedGroupSeqs: number[] =
        environment.productConfig.group.fixedGroupSeqs;
      if (!!fixedGroupSeqs && fixedGroupSeqs.length > 0) {
        if (fixedGroupSeqs.indexOf(group.seq) > -1) {
          if (
            menuType === ProfileMenuType.removeBuddy ||
            menuType === ProfileMenuType.moveBuddy
          ) {
            return false;
          }
        }
      }
    }

    return true;
  }
  private _groupOpenBySessionInfo() {
    if (
      !!this.loginSession &&
      !!this.groupBuddies &&
      !!this.groupExpansion &&
      this.loginSession.groupInfo.groupSeqs.length > 0 &&
      this.groupBuddies.length > 0
    ) {
      this.showType = this.loginSession.groupInfo.showType;

      const groupInfo = this.loginSession.groupInfo;

      this.groupExpansion.expand({
        groupSeqs: groupInfo.groupSeqs,
        lastGroupSeq: groupInfo.lastGroupSeq
      });
    }
  }
}
