import { Subject, combineLatest, of } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  EventEmitter,
  Output,
  ViewChild,
  ElementRef,
  Self
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import {
  User,
  UserInfo,
  UserInfoSS,
  UserInfoF,
  UserInfoDN
} from '@ucap/domain-organization';
import { PresenceType, StatusBulkInfo } from '@ucap/domain-status';
import { GroupInfoDetail } from '@ucap/domain-group';
import { UserInfo as RoomUserInfo } from '@ucap/domain-chat';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';
import { ConfigurationSelector } from '@ucap/ng-store-authentication';
import { BuddySelector, GroupSelector } from '@ucap/ng-store-group';
import { PresenceSelector, UserSelector } from '@ucap/ng-store-organization';

import {
  ExpansionComponent as UcapExpansionComponent,
  NodeType,
  GroupNode
} from '@ucap/ng-ui-group';
import { PresenceUtil } from '@ucap/ng-ui-organization';

import { environment } from '@environments';
import { AppGroupService } from '@app/services/app-group.service';
import { VersionInfo } from '@ucap/domain-authentication';

export type UserInfoTypes =
  | UserInfo
  | UserInfoSS
  | UserInfoF
  | UserInfoDN
  | RoomUserInfo;

@Component({
  selector: 'app-group-expansion-02',
  templateUrl: './expansion-02.component.html',
  styleUrls: ['./expansion-02.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class Expansion02Component implements OnInit, OnDestroy {
  @Input()
  checkable = true;

  @Input()
  isDialog = true;

  @Input()
  selectedUserList: UserInfoTypes[];

  @Input()
  fixedUserList: UserInfoTypes[];

  @Input()
  selectedGroupHeader: GroupInfoDetail;

  @Output()
  clicked = new EventEmitter<{ event: MouseEvent; userInfo: UserInfoTypes }>();

  @Output()
  openProfile = new EventEmitter<UserInfoTypes>();

  @Output()
  toggleCheckUser: EventEmitter<
    {
      checked: boolean;
      userInfo: UserInfoSS;
    }[]
  > = new EventEmitter();

  @Output()
  toggleCheckGroup = new EventEmitter<{
    isChecked: boolean;
    groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfo[] };
  }>();

  @ViewChild('groupExpansion', { static: false })
  groupExpansion: UcapExpansionComponent;

  versionInfo: VersionInfo;
  user: User;

  statusBulkInfos: StatusBulkInfo[];
  currentSelectUser: UserInfo;

  displayOrder: NodeType[] = [
    NodeType.Profile,
    NodeType.Favorite,
    NodeType.Buddy,
    NodeType.Default
  ];

  profile: UserInfo;
  favorites: UserInfoF[];
  groupBuddies: { group: GroupInfoDetail; buddyList: UserInfoF[] }[];
  onlineBuddies: { group: GroupInfoDetail; buddyList: UserInfoF[] }[];
  onOffBuddies: { group: GroupInfoDetail; buddyList: UserInfoF[] }[];

  editablGroup: GroupInfoDetail = null;

  String = String;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService,
    private i18nService: I18nService,
    private appGroupService: AppGroupService
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
      });

    combineLatest([
      this.store.pipe(select(BuddySelector.buddies)),
      this.store.pipe(select(GroupSelector.groups)),
      this.store.pipe(select(PresenceSelector.selectAllStatusBulkInfo))
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([buddies, groups, bulkInfos]) => {
        buddies = buddies || [];
        groups = groups || [];
        bulkInfos = bulkInfos || [];

        if (buddies.length === 0 || groups.length === 0) {
          return;
        }

        buddies = this.appGroupService.extractBuddiesFromGroups(
          buddies,
          groups
        );

        const favorites = buddies
          .filter((buddy) => buddy.isFavorit)
          .sort((a, b) => (a.name < b.name ? -1 : a.name > b.name ? 1 : 0));

        if (!!favorites && 0 < favorites.length) {
          this.favorites = favorites;
          this.changeDetectorRef.markForCheck();
        } else if (!!favorites && favorites.length === 0) {
          this.favorites = undefined;
          this.changeDetectorRef.markForCheck();
        }
        const tempOrder: GroupInfoDetail[] = [];
        let defaultGroup: GroupInfoDetail;
        const buddyGroup: GroupInfoDetail[] = [];
        groups.forEach((group) => {
          if (0 === group.seq) {
            defaultGroup = group;
          } else {
            buddyGroup.push(group);
          }
        });

        tempOrder.push(
          ...buddyGroup.sort((a, b) =>
            a.name < b.name ? -1 : a.name > b.name ? 1 : 0
          )
        );

        if (!!defaultGroup) {
          tempOrder.push(defaultGroup);
        }

        groups = tempOrder;

        if (!!groups && 0 < groups.length) {
          this.groupBuddies = [];

          for (const group of groups) {
            this.groupBuddies.push({
              group,
              buddyList: buddies.filter((buddy) => {
                return -1 < group.userSeqs.indexOf(String(buddy.seq));
              })
            });
          }

          this.changeDetectorRef.markForCheck();
        }
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(PresenceSelector.selectAllStatusBulkInfo)
      )
      .subscribe((statusArr) => {
        this.statusBulkInfos = statusArr;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  psUpdate() {
    if (!!this.groupExpansion) {
      this.groupExpansion.psDirectiveRef.update();
    }
  }

  onClickUser(event: MouseEvent, userInfo: UserInfo) {
    this.currentSelectUser = userInfo;
    this.clicked.emit({ event, userInfo });
  }

  onOpenProfile(userInfo: UserInfo) {
    this.currentSelectUser = userInfo;
    this.openProfile.emit(userInfo);
  }

  getNodeGroupStatus(groupNode: GroupNode): string {
    if (
      !!this.statusBulkInfos &&
      this.statusBulkInfos.length > 0 &&
      !!groupNode &&
      !!groupNode.groupDetail &&
      !!groupNode.groupDetail.userSeqs &&
      groupNode.groupDetail.userSeqs.length > 0
    ) {
      let onlineCount = 0;
      this.statusBulkInfos.map((statusInfo) => {
        groupNode.groupDetail.userSeqs.filter((seq) => {
          if (Number(seq) === Number(statusInfo.userSeq)) {
            const pcOnline = PresenceUtil.isOnline(statusInfo, PresenceType.PC);
            const mobileOnline = PresenceUtil.isOnline(
              statusInfo,
              PresenceType.MOBILE
            );

            if (pcOnline) {
              onlineCount++;
            }
          }
        });
      });

      return `(${onlineCount}/${groupNode.children.length})`;
    } else {
      return `(0/0)`;
    }
  }

  onExpandMore() {
    this.groupExpansion.expandMore();
  }
  onExpandLess() {
    this.groupExpansion.expandLess();
  }

  getShowGroupContextMenu(menuType: string, group: GroupInfoDetail) {
    // 즐겨찾기 그룹 숨김메뉴
    if (
      menuType === 'DIV1' ||
      menuType === 'RENAME' ||
      menuType === 'MANAGE_MEMBER' ||
      menuType === 'DELETE'
    ) {
      if (
        !group ||
        group === undefined ||
        (group.seq < 0 && group.name === 'Favorite')
      ) {
        return false;
      }

      /** 수정불가 그룹 핸들링. */
      if (!!group && !!environment.productConfig.group.fixedGroupSeqs) {
        const fixedGroupSeqs: number[] =
          environment.productConfig.group.fixedGroupSeqs;
        if (!!fixedGroupSeqs && fixedGroupSeqs.length > 0) {
          if (fixedGroupSeqs.indexOf(group.seq) > -1) {
            return false;
          }
        }
      }
    }

    // 기본 그룹 숨김메뉴
    if (menuType === 'RENAME' || menuType === 'DELETE') {
      if (!!group && group.seq === 0) {
        return false;
      }
    }

    // 그룹원 0명인 그룹 메뉴 정리
    if (
      menuType === 'CHAT' ||
      menuType === 'SEND_MESSAGE' ||
      menuType === 'DIV1'
    ) {
      if (!!group && !!group.userSeqs && group.userSeqs.length > 0) {
        return true;
      } else {
        return false;
      }
    }

    return true;
  }

  /** 개별선택(토글) 이벤트 */
  onToggleCheckUser(param: { checked: boolean; userInfo: UserInfoSS }) {
    this.toggleCheckUser.emit([param]);
  }

  /** 개별 체크가능 여부 */
  getCheckableUser(userInfo: UserInfoTypes) {
    if (
      !!this.checkable &&
      !!this.fixedUserList &&
      this.fixedUserList.length > 0
    ) {
      return !this.fixedUserList.some(
        (fix) => fix.seq + '' === userInfo.seq + ''
      );
    } else {
      return this.checkable;
    }
  }
  /** 개별 체크여부 */
  getCheckedUser(userInfo: UserInfoTypes) {
    if (!!this.selectedUserList && this.selectedUserList.length > 0) {
      return (
        this.selectedUserList.filter(
          (item) => String(item.seq) === String(userInfo.seq)
        ).length > 0
      );
    }
    return false;
  }

  onToggleCheckGroup(params: {
    isChecked: boolean;
    groupBuddyList: { group: GroupInfoDetail; buddyList: UserInfo[] };
  }) {
    let trgtUserList = params.groupBuddyList.buddyList;

    // fixedUserList filtering.
    if (!!this.fixedUserList && this.fixedUserList.length > 0) {
      trgtUserList = trgtUserList.filter(
        (item) => !this.fixedUserList.some((fixed) => fixed.seq === item.seq)
      );
    }

    if (!!trgtUserList && trgtUserList.length > 0) {
      this.toggleCheckGroup.emit({
        isChecked: params.isChecked,
        groupBuddyList: {
          group: params.groupBuddyList.group,
          buddyList: trgtUserList
        }
      });
    }
  }

  getGroupNameBySeq(group: GroupInfoDetail): string {
    if (!!group && group.seq === 0) {
      return this.i18nService.t('group:category.default');
    } else if (!!group && group.seq === -9999) {
      return this.i18nService.t('group:category.favorite');
    }

    return '';
  }

  getShowProfileContextMenu(menuType: string, group: GroupInfoDetail) {
    if (!group || group === undefined) {
      if (
        menuType === 'REGISTER_FAVORITE' ||
        menuType === 'SEND_MESSAGE' ||
        menuType === 'REGISTER_NICKNAME'
      ) {
        // continue;
      } else {
        return false;
      }
    }
    /** 수정불가 그룹 핸들링. */
    if (!!group && !!environment.productConfig.group.fixedGroupSeqs) {
      const fixedGroupSeqs: number[] =
        environment.productConfig.group.fixedGroupSeqs;
      if (!!fixedGroupSeqs && fixedGroupSeqs.length > 0) {
        if (fixedGroupSeqs.indexOf(group.seq) > -1) {
          if (menuType === 'REMOVE_FROM_GROUP' || menuType === 'MOVE_BUDDY') {
            return false;
          }
        }
      }
    }

    return true;
  }
}
