import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter
} from '@angular/core';
import {
  FormBuilder,
  FormGroup,
  Validators,
  ValidatorFn,
  AbstractControl
} from '@angular/forms';

import { Store, select } from '@ngrx/store';

import { GroupInfoDetail } from '@ucap/domain-group';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';

import { StringUtil } from '@ucap/ng-core';
import { GroupSelector } from '@ucap/ng-store-group';

import { AppService } from '@app/services/app.service';

@Component({
  selector: 'app-group-name-input',
  templateUrl: './name-input.component.html',
  styleUrls: ['./name-input.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class NameInputComponent implements OnInit, OnDestroy {
  @Output()
  chagneName: EventEmitter<{
    invalid: boolean;
    groupName: string;
  }> = new EventEmitter();

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private formBuilder: FormBuilder,
    private changeDetectorRef: ChangeDetectorRef,
    private i18nService: I18nService,
    private logService: LogService,
    public appService: AppService
  ) {}

  inputForm: FormGroup;
  groupList: GroupInfoDetail[];

  ngOnInit(): void {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(GroupSelector.groups))
      .subscribe((groups) => {
        this.groupList = groups;
      });

    this.inputForm = this.formBuilder.group({
      groupName: [
        '',
        [
          Validators.required,
          StringUtil.checkSpecialCharacter(),
          this.checkBanWords(),
          this.checkSameName()
        ]
      ]
    });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onChange() {
    const invalid = this.inputForm.invalid;

    const groupName = this.inputForm.get('groupName').value;
    this.chagneName.emit({ invalid, groupName });
  }

  onKeyupName(event: KeyboardEvent) {
    this.inputForm.get('groupName').markAsTouched();
  }

  checkBanWords(): ValidatorFn {
    return (control: AbstractControl): { [key: string]: any } | null => {
      if (!control || !control.value) {
        return null;
      }

      const ban = this.appService.bannedGroupNames.filter(
        (name) =>
          name.toLowerCase() === (control.value as string).trim().toLowerCase()
      )[0];

      return !!ban && ban !== ''
        ? { groupNameBanned: { value: control.value } }
        : null;
    };
  }

  checkSameName(): ValidatorFn {
    return (control: AbstractControl): { [key: string]: any } | null => {
      if (
        !control ||
        !control.value ||
        !this.groupList ||
        0 === this.groupList.length
      ) {
        return null;
      }
      const v = (control.value as string).trim();
      const ban = -1 < this.groupList.findIndex((g) => g.name === v);
      return ban ? { groupNameSamed: { value: control.value } } : null;
    };
  }
}
