import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  EventEmitter,
  Output,
  ElementRef,
  Self
} from '@angular/core';

import { select, Store } from '@ngrx/store';

import { User, UserInfoF } from '@ucap/domain-organization';
import { GroupInfoDetail } from '@ucap/domain-group';

import { I18nService } from '@ucap/ng-i18n';

import {
  ConfigurationSelector,
  AuthorizationSelector
} from '@ucap/ng-store-authentication';
import { PresenceSelector, UserSelector } from '@ucap/ng-store-organization';

import { ucapAnimations } from '@ucap/ng-ui/core';
import { ProfileMenuType } from '@ucap/ng-ui-organization';
import { UserInfoTypes } from '@app/types';
import { VersionInfo } from '@ucap/domain-authentication';
import { UserPermission } from '@ucap/domain-authorization';

@Component({
  selector: 'app-group-profile-list-item-02',
  templateUrl: './profile-list-item-02.component.html',
  styleUrls: ['./profile-list-item-02.component.scss'],
  animations: ucapAnimations,
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileListItem02Component implements OnInit, OnDestroy {
  @Input()
  user: User;

  @Input()
  userInfo: UserInfoF;

  @Input()
  group: GroupInfoDetail;

  @Input()
  checkable = false;

  @Input()
  isMe = false;

  @Input()
  checked = false;

  @Input()
  menuButtonType = 1;

  // moreMenu, floatingProfileMenu
  @Output()
  clickMoreMenu: EventEmitter<{
    event: MouseEvent;
    userInfo: UserInfoTypes;
    group: GroupInfoDetail;
    rect: any;
  }> = new EventEmitter();

  @Output()
  clickProfileMenu: EventEmitter<{
    menuType: ProfileMenuType;
    userInfo: UserInfoTypes;
  }> = new EventEmitter();

  @Output()
  toggleCheckUser: EventEmitter<{
    checked: boolean;
    userInfo: UserInfoF;
  }> = new EventEmitter();

  @Output()
  openProfile: EventEmitter<UserInfoF> = new EventEmitter();

  @Input()
  isShow = false;
  @Input()
  isDialog = false;

  @Input()
  isMoreMenu = true;

  versionInfo: VersionInfo;
  userPermission: UserPermission;
  statusMsgClass: string;
  isShowStatusMsg = false;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private changeDetectorRef: ChangeDetectorRef,
    private i18nService: I18nService,
    @Self() private elementRef: ElementRef,
    private store: Store<any>
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        if (
          !!user &&
          !!this.userInfo &&
          String(user.info.seq) === String(this.userInfo.seq)
        ) {
          this.changeDetectorRef.markForCheck();
        }
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(AuthorizationSelector.userPermission)
      )
      .subscribe((userPermission) => {
        this.userPermission = userPermission;
      });

    this._userStatusMessage();
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onOpenProfile(userInfo: UserInfoF): void {
    this.openProfile.emit(userInfo);
  }

  onToggleCheckUser(param: { checked: boolean; userInfo: UserInfoF }) {
    this.toggleCheckUser.emit(param);
    this.changeDetectorRef.detectChanges();
  }

  onMouseover(event: MouseEvent): void {
    if (!this.isMe && !this.isDialog) {
      this.isShow = true;
    }
    event.preventDefault();
    event.stopPropagation();
  }
  onMouseleave(event: MouseEvent): void {
    if (!this.isMe && !this.isDialog) {
      this.isShow = false;
    }

    event.preventDefault();
    event.stopPropagation();
  }

  onClickProfileMenu(params: {
    menuType: ProfileMenuType;
    userInfo: UserInfoTypes;
  }) {
    this.clickProfileMenu.emit(params);
  }

  onClickMoreMenu(params: {
    event: MouseEvent;
    userInfo: UserInfoTypes;
    group: GroupInfoDetail;
    rect: any;
  }) {
    const rect = this.elementRef.nativeElement.getBoundingClientRect();
    params = { ...params, rect };
    this.clickMoreMenu.emit(params);
  }

  _userStatusMessage(): string {
    if (
      !!this.userInfo &&
      !!this.userInfo.nickName &&
      this.userInfo.nickName.trim() !== ''
    ) {
      this.statusMsgClass = 'intro-name';
      return this.userInfo.nickName;
    } else if (
      !!this.userInfo &&
      !!this.userInfo.intro &&
      this.userInfo.intro.trim() !== ''
    ) {
      this.statusMsgClass = 'intro';
      return this.userInfo.intro;
    } else {
      // this.isShowStatusMsg = true;
      this.statusMsgClass = null;
      return '';
    }
  }
}
