import { Subject } from 'rxjs';
import { takeUntil, take } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter,
  ViewChild
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import {
  User,
  UserInfo,
  UserInfoSS,
  UserInfoF,
  UserInfoDN,
  DeptSearchType
} from '@ucap/domain-organization';
import { UserInfo as RoomUserInfo } from '@ucap/domain-chat';

import { DeptUserRequest } from '@ucap/protocol-query';

import { LogService } from '@ucap/ng-logger';
import { QueryProtocolService } from '@ucap/ng-protocol-query';
import {
  LoginSelector,
  ConfigurationSelector
} from '@ucap/ng-store-authentication';
import { PresenceActions, UserSelector } from '@ucap/ng-store-organization';

import { SearchData } from '@app/ucap/organization/models/search-data';
import {
  ProfileListComponent as UcapProfileListComponent,
  ProfileMenuType
} from '@ucap/ng-ui-organization';
import { VersionInfo } from '@ucap/domain-authentication';

const DEPT_ORDER_PROPERTY = 'order';

export type UserInfoTypes =
  | UserInfo
  | UserInfoSS
  | UserInfoF
  | UserInfoDN
  | RoomUserInfo;

interface CheckedInfo {
  checked: boolean;
  userInfo: UserInfoSS;
}
@Component({
  selector: 'app-group-profile-list',
  templateUrl: './profile-list.component.html',
  styleUrls: ['./profile-list.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileListComponent implements OnInit, OnDestroy {
  @Input()
  checkable = false;

  @Input()
  isDialog = false;

  @Input()
  isMoreMenu = true;

  @Input()
  set searchData(data: SearchData) {
    if (!this.user) {
      this._searchData = data;
      return;
    }

    this.searchMember(data);
  }
  // tslint:disable-next-line: variable-name
  private _searchData: SearchData;

  @Input() selectedUser: UserInfoSS[];

  @Input()
  fixedUserList?: UserInfoTypes[];

  @Output()
  searched: EventEmitter<UserInfoSS[]> = new EventEmitter();

  @Output()
  clicked: EventEmitter<{ userInfo: UserInfoSS }> = new EventEmitter();
  @Output()
  openProfile: EventEmitter<UserInfoSS> = new EventEmitter();
  @Output()
  clickProfileMenu = new EventEmitter<{
    menuType: string;
    userInfo: UserInfoF;
  }>();

  @Output()
  toggleCheckUser: EventEmitter<CheckedInfo[]> = new EventEmitter();

  @ViewChild('ucapProfileList', { static: false })
  ucapProfileList: UcapProfileListComponent;

  set userInfos(userInfos: UserInfoSS[]) {
    this._userInfos = userInfos;
    this.searched.emit(userInfos);
    this.psUpdate();
  }
  get userInfos() {
    return this._userInfos;
  }
  // tslint:disable-next-line: variable-name
  _userInfos: UserInfoSS[] = [];

  user: User;
  versionInfo: VersionInfo;

  processing = false;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private queryProtocolService: QueryProtocolService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
        if (!!this._searchData) {
          this.searchMember(this._searchData);
          this._searchData = undefined;
        }
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  psUpdate() {
    if (!!this.ucapProfileList) {
      this.ucapProfileList.psUpdate();
    }
  }

  /** 전체 체크여부 */
  checkAll() {
    if (!this.userInfos || 0 === this.userInfos.length) {
      return;
    }

    const checked: CheckedInfo[] = [];
    this.userInfos.forEach((u) => {
      checked.push({
        checked: true,
        userInfo: u
      });
    });
    this.toggleCheckUser.emit(checked);
  }

  uncheckAll() {
    if (!this.userInfos || 0 === this.userInfos.length) {
      return;
    }

    const unchecked: CheckedInfo[] = [];
    this.userInfos.forEach((u) => {
      unchecked.push({
        checked: false,
        userInfo: u
      });
    });
    this.toggleCheckUser.emit(unchecked);
  }

  getCheckable(userInfo: UserInfoSS): boolean {
    if (!!this.fixedUserList && this.fixedUserList.length > 0) {
      return !this.fixedUserList.some(
        (item) => String(item.seq) === String(userInfo.seq)
      );
    }

    return this.checkable;
  }

  /** 개별 체크여부 */
  getCheckedUser(userInfo: UserInfoSS) {
    if (!!this.selectedUser && this.selectedUser.length > 0) {
      return (
        this.selectedUser.filter(
          (item) => String(item.seq) === String(userInfo.seq)
        ).length > 0
      );
    }
    return false;
  }

  /** 개별선택(토글) 이벤트 */
  onToggleCheckUser(param: { checked: boolean; userInfo: UserInfoSS }) {
    this.toggleCheckUser.emit([param]);
    this.changeDetectorRef.detectChanges();
  }

  onClickProfileMenu(params: {
    menuType: ProfileMenuType;
    userInfo: UserInfoF;
  }) {
    this.clickProfileMenu.emit(params);
  }

  onClickUser(userInfo: UserInfoSS): void {
    this.clicked.emit({ userInfo });
  }

  onOpenProfile(userInfo: UserInfoSS): void {
    this.openProfile.emit(userInfo);
  }

  private searchMember(searchData: SearchData) {
    if (!searchData) {
      return;
    }

    this.processing = true;
    this.changeDetectorRef.markForCheck();

    const req = {
      divCd: 'ORGS',
      companyCode: searchData.companyCode,
      searchRange: DeptSearchType.All,
      search: searchData.searchWord,
      senderCompanyCode: this.user.info.companyCode,
      senderEmployeeType: this.user.info.employeeType
    } as DeptUserRequest;

    this.queryProtocolService
      .deptUser(req)
      .pipe(take(1))
      .subscribe(
        (data) => {
          this._refreshUserInfos(data.userInfos);
        },
        (error) => {},
        () => {
          this.processing = false;
          this.changeDetectorRef.markForCheck();
        }
      );
  }

  private sort(userInfos: UserInfoSS[]): UserInfoSS[] {
    if (!userInfos || 0 === userInfos.length) {
      return userInfos;
    }

    const property = 'name';
    const ascending = undefined;
    let deptA: any;
    let deptB: any;
    let c: any;
    let d: any;

    return userInfos.slice().sort((a, b) => {
      try {
        deptA = a[DEPT_ORDER_PROPERTY];
        deptB = b[DEPT_ORDER_PROPERTY];

        if (undefined === ascending) {
          return deptA < deptB
            ? -1
            : deptA > deptB
            ? 1
            : a[property] < b[property]
            ? -1
            : a[property] > b[property]
            ? 1
            : 0;
        }

        c = ascending ? a[property] : b[property];
        d = ascending ? b[property] : a[property];

        return c < d ? -1 : c > d ? 1 : 0;
      } catch (error) {
        console.log(error);
      }
    });
  }

  private _refreshUserInfos(userInfos: UserInfoSS[]) {
    this.userInfos = this.sort(userInfos);

    // 검색 결과에 따른 프레즌스 조회.
    const userSeqList: string[] = userInfos.map((user) => user.seq);

    if (userSeqList.length > 0) {
      this.store.dispatch(
        PresenceActions.bulkInfo({
          divCd: 'orgSrch',
          userSeqs: userSeqList
        })
      );
    }
  }
}
