import { Subject, combineLatest } from 'rxjs';
import { takeUntil, take, map, withLatestFrom, tap } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  ElementRef,
  ViewChild,
  EventEmitter,
  Output
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { FileUploadItem } from '@ucap/domain-common';
import {
  User,
  UserInfoSS,
  Company,
  UserInfoF
} from '@ucap/domain-organization';
import { PresenceType, StatusCode, StatusBulkInfo } from '@ucap/domain-status';

import { LogService } from '@ucap/ng-logger';
import { I18nService } from '@ucap/ng-i18n';
import { QueryProtocolService } from '@ucap/ng-protocol-query';
import {
  AuthorizationSelector,
  ConfigurationSelector
} from '@ucap/ng-store-authentication';
import {
  PresenceSelector,
  UserSelector,
  CompanySelector,
  PresenceActions
} from '@ucap/ng-store-organization';
import { BuddySelector, GroupSelector } from '@ucap/ng-store-group';

import { PresenceUtil } from '@ucap/ng-ui-organization';
import { GroupInfoDetail } from '@ucap/domain-group';

import { UserInfoTypes } from '@app/types';

import { VersionInfo } from '@ucap/domain-authentication';
import { UserPermission } from '@ucap/domain-authorization';
@Component({
  selector: 'app-organization-profile-01',
  templateUrl: './profile-01.component.html',
  styleUrls: ['./profile-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class Profile01Component implements OnInit, OnDestroy {
  @Input()
  set userSeq(seq: string) {
    this._userSeq = seq;
    this._refreshProfile();
  }
  get userSeq(): string {
    return this._userSeq;
  }
  _userSeq: string;

  set presenceInfo(p: StatusBulkInfo) {
    this._presenceInfo = p;
    this.changeDetectorRef.markForCheck();
  }
  get presenceInfo() {
    return this._presenceInfo;
  }
  // tslint:disable-next-line: variable-name
  _presenceInfo: StatusBulkInfo;

  @Input()
  companyList: Company[];

  @Output()
  openChat: EventEmitter<UserInfoSS> = new EventEmitter();
  @Output()
  sendMessage: EventEmitter<UserInfoSS> = new EventEmitter();
  @Output()
  sendCall: EventEmitter<string> = new EventEmitter();
  @Output()
  sendSms: EventEmitter<string> = new EventEmitter();
  @Output()
  createConference: EventEmitter<number> = new EventEmitter<number>();
  @Output()
  toggleFavorit: EventEmitter<{
    userInfo: UserInfoSS;
    isFavorite: boolean;
  }> = new EventEmitter();
  @Output()
  toggleBuddy: EventEmitter<{
    userInfo: UserInfoSS;
    isBuddy: boolean;
  }> = new EventEmitter();

  @Output()
  uploadProfileImage: EventEmitter<FileUploadItem> = new EventEmitter();
  @Output()
  updateIntro: EventEmitter<string> = new EventEmitter();
  @Output()
  updateNickname: EventEmitter<{
    userInfo: UserInfoTypes;
    nickname: string;
  }> = new EventEmitter();

  @ViewChild('profileImageFileInput', { static: false })
  profileImageFileInput: ElementRef<HTMLInputElement>;

  profileImageFileUploadItem: FileUploadItem;

  userInfo: UserInfoSS;

  user: User;
  userPermission: UserPermission;
  versionInfo: VersionInfo;

  String = String;
  PresenceUtil = PresenceUtil;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private queryProtocolService: QueryProtocolService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService,
    private i18nService: I18nService
  ) {}

  PresenceType = PresenceType;
  isBuddy = false;
  isFavorite = false;
  currentNickname: string;

  companyName: string;

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
        this._refreshProfile();
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(AuthorizationSelector.userPermission)
      )
      .subscribe((userPermission) => {
        this.userPermission = userPermission;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(PresenceSelector.selectStatusBulkInfo, Number(this.userSeq))
      )
      .subscribe((statusBulkInfo) => {
        this.presenceInfo = statusBulkInfo;
      });

    combineLatest([
      this.store.pipe(select(BuddySelector.buddies)),
      this.store.pipe(select(GroupSelector.groups))
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([buddies, groups]) => {
        if (
          !!buddies &&
          !!groups &&
          (buddies.length === 0 || groups.length === 0)
        ) {
          return;
        }
        this.checkBuddy(false, buddies, groups);
        this.changeDetectorRef.markForCheck();
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(CompanySelector.companyList)
      )
      .subscribe((companyList) => {
        if (!companyList) {
          return;
        }
        this.companyList = companyList;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  _refreshProfile() {
    if (!this.user || !this.userSeq) {
      return;
    }

    this.currentNickname = '';

    this.queryProtocolService
      .dataUser({
        divCd: 'OPENPROF',
        seq: Number(this.userSeq),
        senderCompanyCode: this.user.info.companyCode,
        senderEmployeeType: this.user.info.employeeType
      })
      .pipe(
        take(1),
        withLatestFrom(
          this.store.pipe(select(BuddySelector.buddies)),
          this.store.pipe(select(GroupSelector.groups)),
          this.store.pipe(
            select(PresenceSelector.selectStatusBulkInfo, Number(this.userSeq))
          )
        ),
        tap(([res, buddies, groups, bulkInfo]) => {
          if (!bulkInfo) {
            this.store.dispatch(
              PresenceActions.bulkInfo({
                divCd: 'orgSrch',
                userSeqs: [this.userSeq]
              })
            );
          }
          this.userInfo = res.userInfo;
          this.presenceInfo = bulkInfo;

          this.checkBuddy(true, buddies, groups);

          if (
            !this.userInfo.companyName ||
            (!!this.userInfo.companyName &&
              this.userInfo.companyName.trim() === '')
          ) {
            if (!!this.companyList && this.companyList.length > 0) {
              const company = this.companyList.filter(
                (companyInfo) =>
                  companyInfo.companyCode === this.userInfo.companyCode
              );
              if (!!company) {
                this.companyName = company[0].companyName;
              }
            }
          } else {
            this.companyName = this.userInfo.companyName;
          }
          // this.changeDetectorRef.markForCheck();
        })
      )
      .subscribe();
  }
  onProfileImageView(userInfo: UserInfoSS) {}
  onChangeFileInput() {}
  onChangeIntro(intro: string) {
    this.updateIntro.emit(intro);
  }
  onChangeNickname(nickname: string) {
    this.updateNickname.emit({ userInfo: this.userInfo, nickname });
  }

  onToggleFavorite(params: { userInfo: UserInfoSS; isFavorite: boolean }) {
    this.toggleFavorit.emit(params);
  }
  onToggleBuddy(params: { userInfo: UserInfoSS; isBuddy: boolean }) {
    this.toggleBuddy.emit(params);
  }
  onOpenChat(userInfo: UserInfoSS) {
    this.openChat.emit(userInfo);
  }
  onSendCall(calleeNumber: string) {
    this.sendCall.emit(calleeNumber);
  }
  onSendSms(smsNum: string) {
    this.sendSms.emit(smsNum);
  }
  onCreateConference(userSeq: number) {
    this.createConference.emit(userSeq);
  }
  onSendMessage(userInfo: UserInfoSS) {
    this.sendMessage.emit(userInfo);
  }

  onUploadProfileImage(profileImageFileUploadItem: FileUploadItem) {
    this.uploadProfileImage.emit(profileImageFileUploadItem);
  }

  private checkBuddy(
    isSearch: boolean,
    buddies: UserInfoF[],
    groups: GroupInfoDetail[]
  ) {
    const users = buddies.filter((buddy) => {
      if (!!buddy && !!this.userInfo) {
        return buddy.seq === Number(this.userInfo.seq);
      }
    });

    this.isBuddy = users.length > 0;
    if (this.isBuddy) {
      const tempUser = users[0];

      let isFind = false;
      groups.every((g) => {
        const find = g.userSeqs.indexOf(String(tempUser.seq));

        if (find > -1) {
          isFind = true;
          return false;
        }
        return true;
      });

      if (!isFind) {
        this.isBuddy = isFind;
        return;
      }

      this.isFavorite = tempUser.isFavorit;
      this.currentNickname = tempUser.nickName;
      if (!isSearch) {
        const profileImageFile = tempUser.profileImageFile;
        const intro = tempUser.intro;

        this.userInfo = { ...this.userInfo, profileImageFile, intro };
      }
    }
  }
}
