import { Subject } from 'rxjs';
import { takeUntil, take } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter,
  NgZone,
  ViewChild
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { SortOrder } from '@ucap/domain-common';
import { User, UserInfoSS, DeptSearchType } from '@ucap/domain-organization';

import { DeptUserRequest } from '@ucap/protocol-query';

import { LogService } from '@ucap/ng-logger';
import {
  ConfigurationSelector,
  AuthorizationSelector
} from '@ucap/ng-store-authentication';
import { QueryProtocolService } from '@ucap/ng-protocol-query';
import {
  DepartmentActions,
  DepartmentSelector,
  PresenceActions,
  UserSelector
} from '@ucap/ng-store-organization';

import {
  ProfileListComponent as UcapProfileListComponent,
  ProfileMenuType
} from '@ucap/ng-ui-organization';

import { SearchData } from '../models/search-data';
import { VersionInfo } from '@ucap/domain-authentication';
import { UserPermission } from '@ucap/domain-authorization';

const DEPT_ORDER_PROPERTY = 'order';

interface CheckedInfo {
  checked: boolean;
  userInfo: UserInfoSS;
}

@Component({
  selector: 'app-organization-profile-list',
  templateUrl: './profile-list.component.html',
  styleUrls: ['./profile-list.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileListComponent implements OnInit, OnDestroy {
  @Input()
  set searchData(data: SearchData) {
    if (!this.user) {
      this._searchData = data;
      return;
    }
    this.searchMember(data);
  }
  // tslint:disable-next-line: variable-name
  private _searchData: SearchData;

  @Input()
  selectedUser: UserInfoSS[];

  @Input()
  nodeType = '01';
  @Input()
  menuButtonType = '1';

  @Input()
  cacheSize: number;

  @Input()
  /** Cycle order of sorting by use ascending: undefined -> true -> false */
  set sortOrder(value: SortOrder) {
    this._sortOrder = value;
    this.userInfos = this.sort(this.userInfos);
  }
  get sortOrder() {
    return this._sortOrder;
  }
  // tslint:disable-next-line: variable-name
  _sortOrder: SortOrder = {
    property: 'name',
    ascending: undefined
  };

  @Output()
  searched: EventEmitter<UserInfoSS[]> = new EventEmitter();

  @Output()
  changedCheck: EventEmitter<CheckedInfo[]> = new EventEmitter();

  @Output()
  openProfile: EventEmitter<UserInfoSS> = new EventEmitter();

  @Output()
  clickProfileMenu: EventEmitter<{
    menuType: ProfileMenuType;
    userInfo: UserInfoSS;
  }> = new EventEmitter();

  @ViewChild('ucapProfileList', { static: false })
  ucapProfileList: UcapProfileListComponent;

  set userInfos(userInfos: UserInfoSS[]) {
    this._userInfos = userInfos;
    this.searched.emit(userInfos);
    this.psUpdate();
  }
  get userInfos() {
    return this._userInfos;
  }
  // tslint:disable-next-line: variable-name
  _userInfos: UserInfoSS[] = [];

  user: User;
  versionInfo: VersionInfo;

  showFabUserSeq: string;

  userPermission: UserPermission;
  processing = false;
  ProfileMenuType = ProfileMenuType;
  String = String;

  private ngOnDestroySubject: Subject<void> = new Subject();
  private myDeptDestroySubject: Subject<void>;

  constructor(
    private queryProtocolService: QueryProtocolService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });

    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
        if (!!this._searchData) {
          const tempData = { ...this._searchData };
          this._searchData = undefined;
          this.searchMember(tempData);
        }
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(AuthorizationSelector.userPermission)
      )
      .subscribe((userPermission) => {
        this.userPermission = userPermission;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  checkAll() {
    if (!this.userInfos || 0 === this.userInfos.length) {
      return;
    }

    const checked: CheckedInfo[] = [];
    this.userInfos.forEach((u) => {
      checked.push({
        checked: true,
        userInfo: u
      });
    });
    this.changedCheck.emit(checked);
  }

  uncheckAll() {
    if (!this.userInfos || 0 === this.userInfos.length) {
      return;
    }

    const unchecked: CheckedInfo[] = [];
    this.userInfos.forEach((u) => {
      unchecked.push({
        checked: false,
        userInfo: u
      });
    });
    this.changedCheck.emit(unchecked);
  }

  psUpdate() {
    if (!!this.ucapProfileList) {
      this.ucapProfileList.psUpdate();
    }
  }
  /** 개별 체크여부 */
  getCheckedUser(userInfo: UserInfoSS) {
    if (!!this.selectedUser && this.selectedUser.length > 0) {
      return (
        this.selectedUser.filter((item) => item.seq === userInfo.seq).length > 0
      );
    }
    return false;
  }

  /** 개별선택(토글) 이벤트 */
  onChangeCheckUser(param: { checked: boolean; userInfo: UserInfoSS }) {
    this.changedCheck.emit([param]);
  }

  onOpenProfile(userInfo: UserInfoSS): void {
    // alert('Open Profile');
    this.openProfile.emit(userInfo);
  }

  private getMyDeptMember() {
    this.myDeptDestroySubject = new Subject();

    const req: DeptUserRequest = {
      divCd: 'ORG',
      companyCode: this.user.companyCode,
      seq: this.user.departmentCode,
      search: '',
      searchRange: DeptSearchType.All,
      senderCompanyCode: this.user.companyCode,
      senderEmployeeType: this.user.info.employeeType
    };

    this.processing = true;

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        takeUntil(this.myDeptDestroySubject),
        select(DepartmentSelector.myDepartmentUserInfoList)
      )
      .subscribe(
        (myDepartmentUserInfoList) => {
          if (!myDepartmentUserInfoList) {
            this.store.dispatch(DepartmentActions.myDeptUser({ req }));
            return;
          }
          this._refreshUserInfos(myDepartmentUserInfoList);

          this.myDeptDestroySubject.next();
          this.myDeptDestroySubject.complete();
          this.myDeptDestroySubject = undefined;
        },
        (error) => {},
        () => {
          console.log('this.ngOnDestroySubject');
          this.processing = false;
        }
      );
  }

  private sort(userInfos: UserInfoSS[]): UserInfoSS[] {
    if (!userInfos || 0 === userInfos.length) {
      return userInfos;
    }

    const property = this.sortOrder.property;
    const ascending = this.sortOrder.ascending;
    let deptA: any;
    let deptB: any;
    let c: any;
    let d: any;

    return userInfos.slice().sort((a, b) => {
      try {
        deptA = a[DEPT_ORDER_PROPERTY];
        deptB = b[DEPT_ORDER_PROPERTY];

        if (undefined === ascending) {
          return deptA < deptB
            ? -1
            : deptA > deptB
            ? 1
            : a[property] < b[property]
            ? -1
            : a[property] > b[property]
            ? 1
            : 0;
        }

        c = ascending ? a[property] : b[property];
        d = ascending ? b[property] : a[property];

        return c < d ? -1 : c > d ? 1 : 0;
      } catch (error) {
        console.log(error);
      }
    });
  }

  private searchMember(searchData: SearchData) {
    if (
      !searchData ||
      (!searchData.bySearch && undefined === searchData.deptSeq)
    ) {
      this.getMyDeptMember();
      return;
    }

    if (!!this.myDeptDestroySubject) {
      this.myDeptDestroySubject.next();
      this.myDeptDestroySubject.complete();
      this.myDeptDestroySubject = undefined;
    }

    let req: DeptUserRequest;
    if (searchData.bySearch) {
      req = {
        divCd: 'ORGS',
        companyCode: searchData.companyCode,
        searchRange: DeptSearchType.All,
        search: searchData.searchWord,
        senderCompanyCode: this.user.info.companyCode,
        senderEmployeeType: this.user.info.employeeType
      };
    } else {
      req = {
        divCd: 'ORG',
        companyCode: this.user.companyCode,
        seq: Number(searchData.deptSeq),
        search: '',
        searchRange: DeptSearchType.All,
        senderCompanyCode: this.user.companyCode,
        senderEmployeeType: this.user.info.employeeType
      };
    }

    this.processing = true;
    this.queryProtocolService
      .deptUser(req)
      .pipe(take(1))
      .subscribe(
        (data) => {
          this._refreshUserInfos(data.userInfos);
        },
        (error) => {},
        () => {
          this.processing = false;
        }
      );
  }

  private _refreshUserInfos(userInfos: UserInfoSS[]) {
    this.userInfos = this.sort(userInfos);

    // 검색 결과에 따른 프레즌스 조회.
    const userSeqList: string[] = userInfos.map((user) => user.seq);

    if (userSeqList.length > 0) {
      this.store.dispatch(
        PresenceActions.bulkInfo({
          divCd: 'orgSrch',
          userSeqs: userSeqList
        })
      );
    }
  }

  onMouseover(event: MouseEvent, userInfo: UserInfoSS): void {
    this.showFabUserSeq = userInfo.seq;

    event.preventDefault();
    event.stopPropagation();
  }
  onMouseleave(event: MouseEvent): void {
    this.showFabUserSeq = undefined;

    event.preventDefault();
    event.stopPropagation();
  }

  onClickProfileMenu(params: {
    menuType: ProfileMenuType;
    userInfo: UserInfoSS;
  }) {
    this.clickProfileMenu.emit(params);
  }
}
