import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Output,
  EventEmitter
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { User, UserInfoUpdateType } from '@ucap/domain-organization';
import { StatusType, StatusCode } from '@ucap/domain-status';

import { LogService } from '@ucap/ng-logger';

import {
  PresenceActions,
  UserActions,
  UserSelector
} from '@ucap/ng-store-organization';
import { ConfigurationSelector } from '@ucap/ng-store-authentication';

import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { UserStore } from '@app/models/user-store';
import { AppActions } from '@app/store/actions';
import { VersionInfo } from '@ucap/domain-authentication';

@Component({
  selector: 'app-organization-profile-menu-01',
  templateUrl: './profile-menu-01.component.html',
  styleUrls: ['./profile-menu-01.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileMenu01Component implements OnInit, OnDestroy {
  @Output()
  selectedProfileManage: EventEmitter<void> = new EventEmitter();

  @Output()
  selectedNotice: EventEmitter<void> = new EventEmitter();

  @Output()
  selectedSettings: EventEmitter<void> = new EventEmitter();

  @Output()
  selectedLogout: EventEmitter<void> = new EventEmitter();

  @Output()
  selectedExit: EventEmitter<void> = new EventEmitter();

  @Output()
  done: EventEmitter<void> = new EventEmitter();

  user: User;

  userStore: UserStore;

  versionInfo: VersionInfo;

  idleTimePeriods: number[] = [10, 20, 30];

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private appAuthenticationService: AppAuthenticationService,
    private logService: LogService
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
        this.changeDetectorRef.markForCheck();
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(ConfigurationSelector.versionInfo)
      )
      .subscribe((versionInfo) => {
        this.versionInfo = versionInfo;
      });

    this.appAuthenticationService
      .getUserStore$()
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe((userStore) => {
        this.userStore = userStore;
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onChangedIntro(intro: string) {
    this.store.dispatch(
      UserActions.modifyInfo({
        req: {
          type: UserInfoUpdateType.Intro,
          info: intro
        }
      })
    );
  }

  onClickProfileManage(event: Event) {
    this.selectedProfileManage.emit();
  }

  onClickNotice(event: Event) {
    this.selectedNotice.emit();
  }

  onClickSettings(event: Event) {
    this.selectedSettings.emit();
  }

  onChangedStatusMessage(data: { index: number; message: string }) {
    console.log('onChangedStatusMessage', data);
    this.store.dispatch(
      PresenceActions.messageUpdate({
        req: {
          index: data.index,
          statusMessage: data.message
        }
      })
    );
  }

  onSelectedStatus(status: 'online' | 'away' | 'busy1' | 'busy2' | 'busy3') {
    let statusCode: StatusCode;
    let statusMessage: string;
    switch (status) {
      case 'online':
        statusCode = StatusCode.OnLine;
        break;
      case 'away':
        statusCode = StatusCode.Away;
        break;
      case 'busy1':
        statusCode = StatusCode.Busy;
        statusMessage = this.user.statusMessage1;
        break;
      case 'busy2':
        statusCode = StatusCode.Busy;
        statusMessage = this.user.statusMessage2;
        break;
      case 'busy3':
        statusCode = StatusCode.Busy;
        statusMessage = this.user.statusMessage3;
        break;

      default:
        break;
    }

    this.store.dispatch(
      PresenceActions.status({
        req: {
          statusDivisionType: StatusType.Messenger,
          statusType: statusCode,
          statusMessage
        }
      })
    );

    this.done.emit();
  }

  onChangedIdleCheckTime(idleCheckTime: number) {
    this.appAuthenticationService.setUserStore({
      ...this.userStore,
      idleCheckTime
    });
    this.store.dispatch(
      AppActions.idleTimeChanged({ idleTime: idleCheckTime })
    );
  }

  onLogout() {
    this.selectedLogout.emit();
  }

  onExit() {
    this.selectedExit.emit();
  }
}
