import { Subject } from 'rxjs';
import { takeUntil, take } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Input,
  Output,
  EventEmitter,
  NgZone,
  ViewChild
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import {
  User,
  DeptSearchType,
  DeptInfo,
  UserInfoSS
} from '@ucap/domain-organization';

import { DeptUserRequest } from '@ucap/protocol-query';

import { LogService } from '@ucap/ng-logger';
import { QueryProtocolService } from '@ucap/ng-protocol-query';
import {
  DepartmentSelector,
  PresenceActions,
  UserSelector
} from '@ucap/ng-store-organization';

import { UserInfoTypes } from '@app/types';

import { BreadcrumbComponent } from '@ucap/ng-ui/breadcrumb';

import { PerfectScrollbarDirective } from 'ngx-perfect-scrollbar';

const DEPT_ORDER_PROPERTY = 'order';

enum NodeType {
  dept = 'dept',
  member = 'member'
}

interface CheckedInfo {
  checked: boolean;
  userInfo: UserInfoTypes;
}

interface NavigationNode {
  nodeType: NodeType;
  node: DeptInfo | UserInfoTypes;
}

@Component({
  selector: 'app-organization-profile-navigation-list',
  templateUrl: './profile-navigation-list.component.html',
  styleUrls: ['./profile-navigation-list.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class ProfileNavigationListComponent implements OnInit, OnDestroy {
  @Input()
  selectedUser: UserInfoTypes[];

  @Input()
  fixedUserList: UserInfoTypes[];

  @Input()
  displayRootDepartment = false;

  @Input()
  cacheSize = 0;

  @Input()
  checkable = true;

  @Input()
  useFAB = false;

  @Output()
  searched: EventEmitter<UserInfoTypes[]> = new EventEmitter();

  @Output()
  changedCheck: EventEmitter<CheckedInfo[]> = new EventEmitter();

  @ViewChild('breadcrumbForOrganization', { static: false })
  breadcrumbForOrganization: BreadcrumbComponent;

  @ViewChild(PerfectScrollbarDirective, { static: false })
  psDirectiveRef: PerfectScrollbarDirective;

  set departmentCode(departmentCode: number) {
    this._departmentCode = departmentCode;
    this._changedDepartmentCode();
  }
  get departmentCode(): number {
    return this._departmentCode;
  }
  // tslint:disable-next-line: variable-name
  _departmentCode: number;

  set userInfos(userInfos: UserInfoTypes[]) {
    this._userInfos = userInfos;
    this._changedItems();
    this.searched.emit(userInfos);
  }
  get userInfos() {
    return this._userInfos;
  }
  // tslint:disable-next-line: variable-name
  _userInfos: UserInfoTypes[] = [];

  set childLevelDeptInfoList(list: DeptInfo[]) {
    this._childLevelDeptInfoList = list;
    this._changedItems();
  }
  get childLevelDeptInfoList() {
    return this._childLevelDeptInfoList;
  }
  // tslint:disable-next-line: variable-name
  _childLevelDeptInfoList: DeptInfo[] = [];

  user: User;

  breadcrumbs: DeptInfo[];
  departmentInfoList: DeptInfo[] = [];
  nodes: NavigationNode[] = [];

  processing = false;

  itemSize = 60;
  statusMsgClass: string;
  NodeType = NodeType;
  String = String;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private queryProtocolService: QueryProtocolService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {}

  ngOnInit(): void {
    this.store
      .pipe(takeUntil(this.ngOnDestroySubject), select(UserSelector.user))
      .subscribe((user) => {
        this.user = user;
        this.departmentCode = user.departmentCode;
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(DepartmentSelector.departmentInfoList)
      )
      .subscribe((departmentInfoList) => {
        this.departmentInfoList = departmentInfoList;
        this._changedDepartmentCode();
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  checkAll() {
    if (!this.userInfos || 0 === this.userInfos.length) {
      return;
    }

    const checked: CheckedInfo[] = [];
    this.userInfos.forEach((u) => {
      checked.push({
        checked: true,
        userInfo: u
      });
    });
    this.changedCheck.emit(checked);
  }

  uncheckAll() {
    if (!this.userInfos || 0 === this.userInfos.length) {
      return;
    }

    const unchecked: CheckedInfo[] = [];
    this.userInfos.forEach((u) => {
      unchecked.push({
        checked: false,
        userInfo: u
      });
    });
    this.changedCheck.emit(unchecked);
  }

  /** 개별 체크가능 여부 */
  getCheckableUser(userInfo: UserInfoTypes) {
    if (!this.checkable) {
      return false;
    }

    let rtnValue = true;

    if (!!this.fixedUserList && this.fixedUserList.length > 0) {
      rtnValue = !this.fixedUserList.some(
        (fix) => fix.seq + '' === userInfo.seq + ''
      );
    }

    if (!!rtnValue && !!this.user.info.seq) {
      rtnValue = String(this.user.info.seq) !== String(userInfo.seq);
    }

    return rtnValue;
  }
  /** 개별 체크여부 */
  getCheckedUser(userInfo: UserInfoTypes) {
    if (!!this.selectedUser && this.selectedUser.length > 0) {
      return (
        this.selectedUser.filter(
          (item) => String(item.seq) === String(userInfo.seq)
        ).length > 0
      );
    }
    return false;
  }

  /** 개별선택(토글) 이벤트 */
  onChangeCheckUser(param: { checked: boolean; userInfo: UserInfoTypes }) {
    this.changedCheck.emit([param]);
  }

  onOpenProfile(userInfo: UserInfoTypes): void {
    alert('Open Profile');
  }

  onClickDepartment(deptInfo: DeptInfo) {
    this.departmentCode = deptInfo.seq;
  }

  private sort(userInfos: UserInfoTypes[]): UserInfoTypes[] {
    if (!userInfos || 0 === userInfos.length) {
      return userInfos;
    }

    const property = 'name';
    const ascending = true;
    let deptA: any;
    let deptB: any;

    return userInfos.slice().sort((a, b) => {
      try {
        deptA = a[DEPT_ORDER_PROPERTY];
        deptB = b[DEPT_ORDER_PROPERTY];

        return deptA < deptB
          ? -1
          : deptA > deptB
          ? 1
          : a[property] < b[property]
          ? -1
          : a[property] > b[property]
          ? 1
          : 0;
      } catch (error) {
        console.log(error);
      }
    });
  }

  private _searchMember(departmentCode: number) {
    const req: DeptUserRequest = {
      divCd: 'ORG',
      companyCode: this.user.companyCode,
      seq: departmentCode,
      search: '',
      searchRange: DeptSearchType.All,
      senderCompanyCode: this.user.companyCode,
      senderEmployeeType: this.user.info.employeeType
    };

    this.processing = true;
    this.queryProtocolService
      .deptUser(req)
      .pipe(take(1))
      .subscribe(
        (data) => {
          this._refreshUserInfos(data.userInfos);
        },
        (error) => {
          console.log('error', error);
        },
        () => {
          this.processing = false;
          this.changeDetectorRef.markForCheck();

          if (!!this.psDirectiveRef) {
            this.psDirectiveRef.update();
          }
        }
      );
  }

  private _refreshUserInfos(userInfos: UserInfoTypes[]) {
    this.userInfos = this.sort(userInfos);

    // 검색 결과에 따른 프레즌스 조회.
    const userSeqList: string[] = userInfos.map((user) => String(user.seq));

    if (userSeqList.length > 0) {
      this.store.dispatch(
        PresenceActions.bulkInfo({
          divCd: 'orgSrch',
          userSeqs: userSeqList
        })
      );
    }
  }

  private _changedItems() {
    const nodeList: NavigationNode[] = [];

    if (
      !!this.childLevelDeptInfoList &&
      0 < this.childLevelDeptInfoList.length
    ) {
      this.childLevelDeptInfoList.forEach((d) => {
        nodeList.push({
          nodeType: NodeType.dept,
          node: d
        });
      });
    }

    if (!!this.userInfos && 0 < this.userInfos.length) {
      this.userInfos.forEach((u) => {
        nodeList.push({
          nodeType: NodeType.member,
          node: u
        });
      });
    }

    this.nodes = nodeList;
  }

  private _changedDepartmentCode() {
    if (!this.departmentCode || !this.departmentInfoList) {
      return;
    }

    const hierarchy: DeptInfo[] = [];
    this._hierarchyDepartmentInfo(this.departmentCode, hierarchy);

    if (0 === hierarchy.length) {
      return;
    }

    const currentDepthInfo = hierarchy[0];
    this.breadcrumbs = hierarchy.reverse();
    this.childLevelDeptInfoList = this._getChildLevelDepthInfos(
      currentDepthInfo
    );

    this._searchMember(this.departmentCode);
    this.changeDetectorRef.markForCheck();
  }

  private _hierarchyDepartmentInfo(
    departmentCode: number,
    hierarchy: DeptInfo[]
  ) {
    const depthInfo = this.departmentInfoList.find(
      (deptInfo) => deptInfo.seq === departmentCode
    );
    if (!depthInfo) {
      return;
    }

    if (0 === depthInfo.parentSeq) {
      if (this.displayRootDepartment) {
        hierarchy.push(depthInfo);
      }
      return;
    }

    hierarchy.push(depthInfo);

    this._hierarchyDepartmentInfo(depthInfo.parentSeq, hierarchy);
  }

  private _getChildLevelDepthInfos(deptInfo: DeptInfo): DeptInfo[] {
    return this.departmentInfoList
      .filter((d) => deptInfo.seq === d.parentSeq)
      .sort((a, b) => (a.name < b.name ? -1 : a.name > b.name ? 1 : 0));
  }

  _userStatusMessage(userInfo: UserInfoSS): string {
    if (!!userInfo && !!userInfo.intro && userInfo.intro.trim() !== '') {
      this.statusMsgClass = 'intro';
      return userInfo.intro;
    }

    this.statusMsgClass = null;
    return '';
  }
}
