import { Subject } from 'rxjs';
import { takeUntil } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Output,
  EventEmitter,
  Input,
  ViewChild
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { Company } from '@ucap/domain-organization';

import { LogService } from '@ucap/ng-logger';
import { CompanySelector } from '@ucap/ng-store-organization';

import { AppAuthenticationService } from '@app/services/app-authentication.service';
import { SearchData } from '../models/search-data';
import { UserStore } from '@app/models/user-store';
import { SearchForTenantComponent as UcapSearchForTenantComponent } from '@ucap/ng-ui-organization';

@Component({
  selector: 'app-organization-search-for-tenant',
  templateUrl: './search-for-tenant.component.html',
  styleUrls: ['./search-for-tenant.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class SearchForTenantComponent implements OnInit, OnDestroy {
  @Output() searchDataChange: EventEmitter<SearchData> = new EventEmitter();
  @Input()
  isBackspaceCanceled = false;

  @Input() set searchData(value: SearchData) {
    let data: SearchData = {
      ...value
    };

    if (!data.companyCode) {
      data = {
        ...data,
        companyCode: this.userStore.companyCode
      };
    }
    this._searchData = {
      ...data
    };
    if (!!this.searchForTenant && !!value && !!value.searchWord) {
      this.searchForTenant.changeSearchword(value.searchWord);
    }
  }
  get searchData() {
    return this._searchData;
  }
  // tslint:disable-next-line: variable-name
  _searchData: SearchData;

  @Input()
  set filteredTimerChatAuth(value: boolean) {
    this._filteredTimerChatAuth = value;
    this._filteredCompanyList();
  }
  get filteredTimerChatAuth(): boolean {
    return this._filteredTimerChatAuth;
  }
  // tslint:disable-next-line: variable-name
  _filteredTimerChatAuth = false;

  @Input()
  set filteredConferenceAuth(value: boolean) {
    this._filteredConferenceAuth = value;
    this._filteredCompanyList();
  }
  get filteredConferenceAuth(): boolean {
    return this._filteredConferenceAuth;
  }
  // tslint:disable-next-line: variable-name
  _filteredConferenceAuth = false;

  @Input()
  set filteredUcapUse(value: boolean) {
    this._filteredUcapUse = value;
    this._filteredCompanyList();
  }
  get filteredUcapUse(): boolean {
    return this._filteredUcapUse;
  }
  // tslint:disable-next-line: variable-name
  _filteredUcapUse = false;

  @Output()
  canceled: EventEmitter<void> = new EventEmitter();

  @ViewChild('ucapOrgSearchForTenant', { static: true })
  searchForTenant: UcapSearchForTenantComponent;

  originalCompanyList: Company[];
  filteredCompanyList: Company[];
  preCompanyCode: string;

  private ngOnDestroySubject: Subject<void> = new Subject();
  private userStore: UserStore;

  constructor(
    private appAuthenticationService: AppAuthenticationService,
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService
  ) {
    this.userStore = this.appAuthenticationService.getUserStore();
  }

  ngOnInit(): void {
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(CompanySelector.companyList)
      )
      .subscribe((companyList) => {
        this.originalCompanyList = companyList;
        this._filteredCompanyList();
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onChanged(data: { companyCode: string; searchWord: string }): void {
    if (!!data && data.searchWord.localeCompare('') === 0) {
      return;
    }
    this.preCompanyCode = data.companyCode;

    this.searchDataChange.emit({
      companyCode: data.companyCode,
      searchWord: data.searchWord
    });
  }

  onCanceled(): void {
    this.searchForTenant.changeSearchword('');
    this.searchForTenant.inputSearchWord.nativeElement.value = '';

    this.canceled.emit();
  }

  onKeyupBackspace(event: MouseEvent) {
    if (!!this.isBackspaceCanceled) {
      this.searchForTenant.onClickCancel(event);
    }
  }

  private _filteredCompanyList() {
    let filteredList = [];
    if (!!this.originalCompanyList) {
      filteredList = this.originalCompanyList
        .filter((item) => {
          if (!!this.filteredTimerChatAuth && !item.companyTimerChatAuth) {
            return false;
          } else {
            return true;
          }
        })
        .filter((item) => {
          if (!!this.filteredConferenceAuth && !item.companyConfAuth) {
            return false;
          } else {
            return true;
          }
        })
        .filter((item) => {
          if (!!this.filteredUcapUse && !item.ucapUse) {
            return false;
          } else {
            return true;
          }
        });
    }

    this.filteredCompanyList = filteredList;
  }
}
