import { Subject, combineLatest } from 'rxjs';
import { takeUntil, take } from 'rxjs/operators';

import {
  Component,
  OnInit,
  OnDestroy,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  EventEmitter,
  Output,
  ViewChild,
  Input,
  AfterViewInit
} from '@angular/core';

import { Store, select } from '@ngrx/store';

import { DeptInfo } from '@ucap/domain-organization';

import { LogService } from '@ucap/ng-logger';

import {
  DepartmentSelector,
  PresenceSelector
} from '@ucap/ng-store-organization';

import { TreeComponent as UCAPTreeComponent } from '@ucap/ng-ui-organization';

import { environment } from '@environments';
import { AppSelector } from '@app/store/state';
import { WindowSizeMode } from '@app/types/window-size-mode.type';
import { AppUiService } from '@app/services/app-ui.service';

@Component({
  selector: 'app-organization-tree',
  templateUrl: './tree.component.html',
  styleUrls: ['./tree.component.scss'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class TreeComponent implements OnInit, OnDestroy {
  @Input()
  set initialExpanded(seq: number) {
    this.currentDeptSeq = seq;
    if (!!this.treeData) {
      this.expand(seq);
      return;
    }
    this._initialExpanded = seq;
  }

  @Input()
  windowSizeMode: WindowSizeMode;

  // tslint:disable-next-line: variable-name
  _initialExpanded: number;

  @Output()
  clicked: EventEmitter<DeptInfo> = new EventEmitter();

  @ViewChild('treeList', { static: true })
  treeList: UCAPTreeComponent;

  treeData: {
    deptInfoList: DeptInfo[];
    displayRoot?: boolean;
    expanded?: number[];
  };

  currentDeptSeq: number;
  openedLeftSideNav: boolean;

  private ngOnDestroySubject: Subject<void> = new Subject();

  constructor(
    private store: Store<any>,
    private changeDetectorRef: ChangeDetectorRef,
    private logService: LogService,
    private appUiService: AppUiService
  ) {}

  ngOnInit(): void {
    combineLatest([
      this.store.pipe(select(DepartmentSelector.departmentInfoList))
    ])
      .pipe(takeUntil(this.ngOnDestroySubject))
      .subscribe(([deptInfoList]) => {
        this.treeData = {
          deptInfoList,
          displayRoot: environment.productConfig.organization.displayRoot,
          expanded: !!this._initialExpanded
            ? [this._initialExpanded]
            : undefined
        };

        this.changeDetectorRef.markForCheck();
        if (!!deptInfoList) {
          this._initialExpanded = undefined;
        }
      });

    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(AppSelector.openedLeftSideNav)
      )
      .subscribe((openedLeftSideNav) => {
        this.openedLeftSideNav = openedLeftSideNav;
        if (!!openedLeftSideNav) {
          this.expand(this.currentDeptSeq);
        }
      });
    this.store
      .pipe(
        takeUntil(this.ngOnDestroySubject),
        select(PresenceSelector.statusBulkInfoProcessing)
      )
      .subscribe((statusBulkInfo) => {
        if (!!this.openedLeftSideNav) {
          this.expand(this.currentDeptSeq);
        }
      });
  }

  ngOnDestroy(): void {
    if (!!this.ngOnDestroySubject) {
      this.ngOnDestroySubject.next();
      this.ngOnDestroySubject.complete();
    }
  }

  onClickNode(node: DeptInfo) {
    this.currentDeptSeq = node.seq;
    this.clicked.emit(node);
  }

  expand(deptSeq: number) {
    this.treeList.expand(deptSeq);

    if (!!this.treeList && this.treeList.vsvList) {
      this.treeList.vsvList.checkViewportSize();
    }
  }

  expandAll() {
    this.treeList.expandAll();
  }

  collapse(deptSeq: number) {
    this.treeList.collapse(deptSeq);
  }

  collapseAll() {
    this.treeList.collapseAll();
  }
}
